//>>built
define(
//begin v1.x content
({
	'bold': 'Fed',
	'copy': 'Kopiér',
	'cut': 'Klip',
	'delete': 'Slet',
	'indent': 'Indrykning',
	'insertHorizontalRule': 'Vandret linje',
	'insertOrderedList': 'Nummereret liste',
	'insertUnorderedList': 'Punktliste',
	'italic': 'Kursiv',
	'justifyCenter': 'Centreret',
	'justifyFull': 'Lige margener',
	'justifyLeft': 'Venstrejusteret',
	'justifyRight': 'Højrejusteret',
	'outdent': 'Udrykning',
	'paste': 'Sæt ind',
	'redo': 'Annullér Fortryd',
	'removeFormat': 'Fjern format',
	'selectAll': 'Markér alle',
	'strikethrough': 'Gennemstreget',
	'subscript': 'Sænket skrift',
	'superscript': 'Hævet skrift',
	'underline': 'Understreget',
	'undo': 'Fortryd',
	'unlink': 'Fjern link',
	'createLink': 'Opret link',
	'toggleDir': 'Skift retning',
	'insertImage': 'Indsæt billede',
	'insertTable': 'Indsæt/redigér tabel',
	'toggleTableBorder': 'Skift tabelramme',
	'deleteTable': 'Slet tabel',
	'tableProp': 'Tabelegenskab',
	'htmlToggle': 'HTML-kilde',
	'foreColor': 'Forgrundsfarve',
	'hiliteColor': 'Baggrundsfarve',
	'plainFormatBlock': 'Afsnitstypografi',
	'formatBlock': 'Afsnitstypografi',
	'fontSize': 'Skriftstørrelse',
	'fontName': 'Skriftnavn',
	'tabIndent': 'Indrykning med tabulator',
	"fullScreen": "Aktivér/deaktivér fuldskærm",
	"viewSource": "Vis HTML-kilde",
	"print": "Udskriv",
	"newPage": "Ny side",
	/* Error messages */
	'systemShortcut': 'Funktionen "${0}" kan kun bruges i din browser med en tastaturgenvej. Brug ${1}.'
})
//end v1.x content
);
