//>>built
define(
//begin v1.x content
({
// local representation of all CSS3 named colors, companion to dojo.colors.  To be used where descriptive information
// is required for each color, such as a palette widget, and not for specifying color programatically.

//Note: due to the SVG 1.0 spec additions, some of these are alternate spellings for the same color e.g. gray vs. gray.
//TODO: should we be using unique rgb values as keys instead and avoid these duplicates, or rely on the caller to do the reverse mapping?
aliceblue: "бозғылт көк",
antiquewhite: "ақ антик",
aqua: "су түсі",
aquamarine: "жасылдау-көк",
azure: "көкшіл",
beige: "сарғыш",
bisque: "бисквит",
black: "қара",
blanchedalmond: "ағартылған бадам",
blue: "көк",
blueviolet: "көк-ақшыл көк",
brown: "қоңыр",
burlywood: "ағаш тамыры",
cadetblue: "кадет көк",
chartreuse: "жасылдау-сары",
chocolate: "шоколад",
coral: "коралл",
cornflowerblue: "көктікен көк",
cornsilk: "жібек",
crimson: "таңқұрай",
cyan: "циан",
darkblue: "күңгірт көк",
darkcyan: "күңгірт циан",
darkgoldenrod: "қара алтын",
darkgray: "қою сұры",
darkgreen: "қою жасыл",
darkgrey: "қою сұры", // same as darkgray
darkkhaki: "қою хаки",
darkmagenta: "қою күлгін",
darkolivegreen: "қою қоңырлау жасыл",
darkorange: "қою қызғылт сары",
darkorchid: "күңгірт орсель",
darkred: "күңгірт қызыл",
darksalmon: "қою сарылау қызғылт",
darkseagreen: "қою теңіз толқыны",
darkslateblue: "күңгірт грифель көк",
darkslategray: "күңгірт көкшіл сұры",
darkslategrey: "күңгірт көкшіл сұры", // same as darkslategray
darkturquoise: "күңгірт көгілдір",
darkviolet: "күңгірт күлгін",
deeppink: "қою қызғылт",
deepskyblue: "қою аспан көк",
dimgray: "күңгірт сұры",
dimgrey: "күңгірт сұры", // same as dimgray
dodgerblue: "көк доджер",
firebrick: "қызыл кірпіш",
floralwhite: "гүлді ақ",
forestgreen: "шөпті жасыл",
fuchsia: "фуксия",
gainsboro: "gainsboro",
ghostwhite: "елесті ақ",
gold: "сары түсті",
goldenrod: "алтын",
gray: "сұры",
green: "жасыл",
greenyellow: "жасыл-сары",
grey: "сұры", // same as gray
honeydew: "балдай",
hotpink: "ашық қызғылт",
indianred: "үнділік қызыл",
indigo: "индиго",
ivory: "піл сүйег",
khaki: "хаки",
lavender: "бозғылт ақшыл көк",
lavenderblush: "күңгірт ақшыл қызыл",
lawngreen: "көгал жасыл",
lemonchiffon: "лимон шиффон",
lightblue: "ақшыл көк",
lightcoral: "ашық коралл",
lightcyan: "ашық көгілдір",
lightgoldenrodyellow: "ашық сары түсті сары",
lightgray: "ашық сұры",
lightgreen: "ақшыл жасыл",
lightgrey: "ашық сұры", // same as lightgray
lightpink: "ақшыл қызғылт",
lightsalmon: "ашық сарғыш қызғылт",
lightseagreen: "ашық теңіз толқыны",
lightskyblue: "ашық аспан көк",
lightslategray: "ашық көкшіл сұры",
lightslategrey: "ашық көкшіл сұры", // same as lightslategray
lightsteelblue: "ашық сұрғылт көк",
lightyellow: "ашық сары",
lime: "әк",
limegreen: "әк жасыл",
linen: "зығыр",
magenta: "фуксин",
maroon: "сарғылт",
mediumaquamarine: "орташа жасылдау көк",
mediumblue: "орташа көк",
mediumorchid: "орташа ақшыл",
mediumpurple: "орташа күлгін",
mediumseagreen: "орташа теңіз толқыны",
mediumslateblue: "орташа көкшіл сұры",
mediumspringgreen: "орташа ашық жасыл",
mediumturquoise: "орташа көгілдір",
mediumvioletred: "орташа ақшыл көк-қызыл",
midnightblue: "түн ортасы көк",
mintcream: "жалбыз майы",
mistyrose: "көмескі қызғылт",
moccasin: "мокасин",
navajowhite: "навахо ақ",
navy: "қара-көк",
oldlace: "ескі бау",
olive: "зәйтүнді",
olivedrab: "жасылдау сары",
orange: "қызғылт сары",
orangered: "қызғылт сары қызыл",
orchid: "орхидея",
palegoldenrod: "бозғылт алтын",
palegreen: "бозғылт жасыл",
paleturquoise: "бозғылт көгілдір",
palevioletred: "бозғылт ақшыл көк-қызыл",
papayawhip: "папайя қамшысы",
peachpuff: "шабдалы",
peru: "перу",
pink: "қызғылт",
plum: "алхоры",
powderblue: "жасылдау көк",
purple: "күлгін",
red: "қызыл",
rosybrown: "қызғылт қоңыр",
royalblue: "патша көк",
saddlebrown: "тоқым қоңыр",
salmon: "сомон",
sandybrown: "құмды қоңыр",
seagreen: "теңіз толқыны",
seashell: "теңіз қабыршағы",
sienna: "сиенна",
silver: "күміс түстес",
skyblue: "аспан көк",
slateblue: "грифель көк",
slategray: "көкшіл сұры",
slategrey: "көкшіл сұры", // same as slategray
snow: "қар",
springgreen: "көктем жасыл",
steelblue: "көкшіл сұрғылт",
tan: "сарғыш қоңыр",
teal: "шүрегей",
thistle: "артишок",
tomato: "қызанақ",
turquoise: "көгілдір",
violet: "күлгін",
wheat: "бидай",
white: "ақ",
whitesmoke: "ақ түтін",
yellow: "сары",
yellowgreen: "сарғыш жасыл"
})
//end v1.x content
);

