//>>built
define(["dijit","dojo","dojox"],function(_1,_2,_3){
_2.provide("dojox.drawing.plugins._Plugin");
_3.drawing.plugins._Plugin=_3.drawing.util.oo.declare(function(_4){
this._cons=[];
_2.mixin(this,_4);
if(this.button&&this.onClick){
this.connect(this.button,"onClick",this,"onClick");
}
},{util:null,keys:null,mouse:null,drawing:null,stencils:null,anchors:null,canvas:null,node:null,button:null,type:"dojox.drawing.plugins._Plugin",connect:function(){
this._cons.push(_2.connect.apply(_2,arguments));
},disconnect:function(_5){
if(!_5){
return;
}
if(!_2.isArray(_5)){
_5=[_5];
}
_2.forEach(_5,_2.disconnect,_2);
}});
});
