//>>built
define(
//begin v1.x content
({
	fontSize: "크기",
	fontName: "글꼴",
	formatBlock: "서식",

	serif: "serif",
	"sans-serif": "sans-serif",
	monospace: "monospace",
	cursive: "cursive",
	fantasy: "fantasy",

	noFormat: "없음",
	p: "단락",
	h1: "제목",
	h2: "부제목",
	h3: "하위 부제목",
	pre: "서식이 지정됨",

	1: "가장 작게",
	2: "조금 작게",
	3: "작게",
	4: "중간",
	5: "크게",
	6: "조금 크게",
	7: "가장 크게"
})
//end v1.x content
);
