//>>built
define(
//begin v1.x content
({
// local representation of all CSS3 named colors, companion to dojo.colors.  To be used where descriptive information
// is required for each color, such as a palette widget, and not for specifying color programatically.

//Note: due to the SVG 1.0 spec additions, some of these are alternate spellings for the same color e.g. gray vs. gray.
//TODO: should we be using unique rgb values as keys instead and avoid these duplicates, or rely on the caller to do the reverse mapping?
aliceblue: "艾莉斯蓝",
antiquewhite: "古董白",
aqua: "水绿色",
aquamarine: "碧绿色",
azure: "浅天蓝",
beige: "米色",
bisque: "乳黄色",
black: "黑色",
blanchedalmond: "杏仁白",
blue: "蓝色",
blueviolet: "蓝紫色",
brown: "褐色",
burlywood: "原木色",
cadetblue: "军队蓝",
chartreuse: "浅黄绿色",
chocolate: "巧克力色",
coral: "珊瑚红",
cornflowerblue: "藏蓝色",
cornsilk: "玉米黄",
crimson: "绯红色",
cyan: "青色",
darkblue: "深蓝色",
darkcyan: "深青色",
darkgoldenrod: "深金黄色",
darkgray: "深灰色",
darkgreen: "深绿色",
darkgrey: "深灰色", // same as darkgray
darkkhaki: "深褐色",
darkmagenta: "深洋红色",
darkolivegreen: "深橄榄绿色",
darkorange: "深橙色",
darkorchid: "暗兰花紫",
darkred: "深红色",
darksalmon: "深橙红",
darkseagreen: "深海绿色",
darkslateblue: "深石板蓝",
darkslategray: "深石板灰",
darkslategrey: "深石板灰", // same as darkslategray
darkturquoise: "深青绿色",
darkviolet: "深紫罗兰",
deeppink: "深粉色",
deepskyblue: "深天蓝色",
dimgray: "暗灰色",
dimgrey: "暗灰色", // same as dimgray
dodgerblue: "宝蓝",
firebrick: "砖红色",
floralwhite: "花白",
forestgreen: "森林绿",
fuchsia: "紫红色",
gainsboro: "亮灰色",
ghostwhite: "苍白",
gold: "金黄色",
goldenrod: "鲜黄",
gray: "灰色",
green: "绿色",
greenyellow: "绿黄色",
grey: "灰色", // same as gray
honeydew: "蜜色",
hotpink: "暗粉",
indianred: "印度红",
indigo: "靛蓝色",
ivory: "象牙色",
khaki: "黄褐色",
lavender: "淡紫色",
lavenderblush: "淡紫红色",
lawngreen: "草绿色",
lemonchiffon: "柠檬色",
lightblue: "淡蓝色",
lightcoral: "浅珊瑚红",
lightcyan: "浅青色",
lightgoldenrodyellow: "浅金黄色",
lightgray: "浅灰色",
lightgreen: "浅绿色",
lightgrey: "浅灰色", // same as lightgray
lightpink: "浅粉色",
lightsalmon: "浅橙红色",
lightseagreen: "浅海绿色",
lightskyblue: "浅天蓝色",
lightslategray: "浅石板灰",
lightslategrey: "浅石板灰", // same as lightslategray
lightsteelblue: "浅钢蓝色",
lightyellow: "浅黄色",
lime: "酸橙色",
limegreen: "暗黄绿色",
linen: "亚麻布色",
magenta: "洋红色",
maroon: "褐紫红色",
mediumaquamarine: "淡碧绿色",
mediumblue: "淡蓝色",
mediumorchid: "淡兰花紫",
mediumpurple: "淡紫色",
mediumseagreen: "淡海绿色",
mediumslateblue: "淡灰蓝色",
mediumspringgreen: "淡草绿色",
mediumturquoise: "淡青绿色",
mediumvioletred: "淡紫罗兰",
midnightblue: "蓝黑色",
mintcream: "薄荷乳白",
mistyrose: "粉红玫瑰",
moccasin: "鹿皮黄",
navajowhite: "印地安黄",
navy: "藏青色",
oldlace: "旧布黄",
olive: "橄榄色",
olivedrab: "暗橄榄色",
orange: "橙色",
orangered: "桔红色",
orchid: "兰花紫",
palegoldenrod: "浅金黄色",
palegreen: "淡绿色",
paleturquoise: "淡青绿色",
palevioletred: "浅紫罗兰",
papayawhip: "粉木瓜橙",
peachpuff: "粉桃红",
peru: "秘鲁棕",
pink: "粉红色",
plum: "梅红色",
powderblue: "粉蓝色",
purple: "紫色",
red: "红色",
rosybrown: "玫瑰褐色",
royalblue: "亮蓝色",
saddlebrown: "鞍具褐色",
salmon: "橙红色",
sandybrown: "浅褐色",
seagreen: "海绿色",
seashell: "贝壳白",
sienna: "赭色",
silver: "银白色",
skyblue: "天蓝色",
slateblue: "石板蓝",
slategray: "石板灰",
slategrey: "石板灰", // same as slategray
snow: "雪白",
springgreen: "浅草绿色",
steelblue: "钢蓝色",
tan: "茶色",
teal: "青色",
thistle: "蓟色",
tomato: "番茄色",
turquoise: "青绿色",
violet: "紫罗兰色",
wheat: "淡黄色",
white: "白色",
whitesmoke: "烟白色",
yellow: "黄色",
yellowgreen: "黄绿色"
})
//end v1.x content
);
