//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "M/y",
	"dateFormatItem-yyyyMMMEd": "EEE. d. MMM y G",
	"dateFormatItem-yQ": "Q yyyy",
	"dayPeriods-format-wide-pm": "e.m.",
	"dateFormatItem-MMMEd": "E d. MMM",
	"dateFormatItem-hms": "h.mm.ss a",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-MMdd": "dd/MM",
	"dateFormatItem-MMM": "MMM",
	"dayPeriods-format-wide-am": "f.m.",
	"timeFormat-full": "HH.mm.ss zzzz",
	"dateFormatItem-yyyy": "y G",
	"dateFormatItem-Ed": "E d.",
	"dateFormatItem-yMMM": "MMM y",
	"days-standAlone-narrow": [
		"S",
		"M",
		"T",
		"O",
		"T",
		"F",
		"L"
	],
	"dateFormat-long": "d. MMMM y G",
	"timeFormat-medium": "HH.mm.ss",
	"dateFormatItem-Hm": "HH.mm",
	"dateFormatItem-yyMM": "MM/y G",
	"dateFormat-medium": "d. MMM y G",
	"dateFormatItem-Hms": "HH.mm.ss",
	"dateFormatItem-yyMMM": "MMM y G",
	"dateFormatItem-ms": "mm.ss",
	"dateFormatItem-MMMMEd": "E, d. MMMM",
	"dateFormatItem-yyyyMEd": "EEE. d/M/y G",
	"dateFormatItem-MMMd": "d. MMM",
	"dateFormatItem-yyQ": "Q. 'kvartal' y G",
	"timeFormat-long": "HH.mm.ss z",
	"timeFormat-short": "HH.mm",
	"dateFormatItem-H": "HH",
	"quarters-format-abbr": [
		"K1",
		"K2",
		"K3",
		"K4"
	],
	"days-format-abbr": [
		"søn",
		"man",
		"tir",
		"ons",
		"tor",
		"fre",
		"lør"
	],
	"dateFormatItem-M": "M",
	"dateFormatItem-yyyyQQQ": "QQQ y G",
	"dateFormatItem-MEd": "E. d/M",
	"dateFormatItem-hm": "h.mm a",
	"dateFormat-short": "d/M/y G",
	"dateFormatItem-yyyyM": "M/y G",
	"dateFormatItem-yMMMEd": "EEE. d. MMM y",
	"dateFormat-full": "EEEE d. MMMM y G",
	"dateFormatItem-Md": "d/M",
	"dateFormatItem-yyyyQ": "Q y G",
	"dateFormatItem-yMEd": "EEE. d/M/y",
	"dateFormatItem-yyyyMMM": "MMM y G",
	"dateFormatItem-d": "d.",
	"quarters-format-wide": [
		"1. kvartal",
		"2. kvartal",
		"3. kvartal",
		"4. kvartal"
	],
	"days-format-wide": [
		"søndag",
		"mandag",
		"tirsdag",
		"onsdag",
		"torsdag",
		"fredag",
		"lørdag"
	]
}
//end v1.x content
);