//>>built
define(
//begin v1.x content
{
	"HKD_displayName": "Hongkongi dollár",
	"CHF_displayName": "Svájci frank",
	"JPY_symbol": "¥",
	"CAD_displayName": "Kanadai dollár",
	"CNY_displayName": "Kínai jüan renminbi",
	"USD_symbol": "$",
	"AUD_displayName": "Ausztrál dollár",
	"JPY_displayName": "Japán jen",
	"USD_displayName": "USA dollár",
	"GBP_displayName": "Brit font sterling",
	"EUR_displayName": "Euro"
}
//end v1.x content
);