//>>built
define(
//begin v1.x content
({
	'bold': '粗体',
	'copy': '复制',
	'cut': '剪切',
	'delete': '删除',
	'indent': '增加缩进',
	'insertHorizontalRule': '水平线',
	'insertOrderedList': '编号列表',
	'insertUnorderedList': '符号列表',
	'italic': '斜体',
	'justifyCenter': '居中',
	'justifyFull': '对齐',
	'justifyLeft': '左对齐',
	'justifyRight': '右对齐',
	'outdent': '减少缩进',
	'paste': '粘贴',
	'redo': '重做',
	'removeFormat': '除去格式',
	'selectAll': '全选',
	'strikethrough': '删除线',
	'subscript': '下标',
	'superscript': '上标',
	'underline': '下划线',
	'undo': '撤销',
	'unlink': '除去链接',
	'createLink': '创建链接',
	'toggleDir': '固定方向',
	'insertImage': '插入图像',
	'insertTable': '插入/编辑表',
	'toggleTableBorder': '切换表边框',
	'deleteTable': '删除表',
	'tableProp': '表属性',
	'htmlToggle': 'HTML 源代码',
	'foreColor': '前景色',
	'hiliteColor': '背景色',
	'plainFormatBlock': '段落样式',
	'formatBlock': '段落样式',
	'fontSize': '字体大小',
	'fontName': '字体名称',
	'tabIndent': '制表符缩进',
	"fullScreen": "切换全屏幕",
	"viewSource": "查看 HTML 源代码",
	"print": "打印",
	"newPage": "新建页面",
	/* Error messages */
	'systemShortcut': '只能在浏览器中通过键盘快捷方式执行“${0}”操作。使用 ${1}。'
})

//end v1.x content
);
