
.soria .dijitButtonNode {
	
	border: 1px solid #8ba0bd;
	border-bottom:1px solid #657c9c;
	padding: 0.1em 0.2em 0.2em 0.2em;
	background: #bcd5f0 url("../images/buttonEnabled.png") repeat-x top left;
}
.soria .dijitSelect .dijitButtonContents {
	border-right: none;
}
.soria .dijitButtonText {
	text-align: center;
	padding: 0 0.3em;
}
.soria .dijitArrowButton {
	color: #111;
}
.soria .dijitComboButton .dijitDownArrowButton {
	padding-right:4px;
}
.soria .dijitTextBoxReadOnly,
.soria .dijitTextBoxReadOnly .dijitButtonNode,
.soria .dijitButtonDisabled .dijitButtonNode,
.soria .dijitToggleButtonDisabled .dijitButtonNode,
.soria .dijitDropDownButtonDisabled .dijitButtonNode,
.soria .dijitComboButtonDisabled .dijitButtonNode,
.soria .dijitTextBoxDisabled,
.soria .dijitTextBoxDisabled .dijitButtonNode {
	
	border-color: #b9bbdd #b9bbdd #b9bbdd #b9bbdd;
	background:#c3d3e5 url("../images/buttonDisabled.png") top repeat-x;
	opacity: 0.60;
}
.dj_ie6 .soria .dijitReadOnly input,
.dj_ie7 .soria .dijitReadOnly input,
.dj_ie6 .soria .dijitComboButtonDisabled .dijitButtonText,
.dj_ie7 .soria .dijitComboButtonDisabled .dijitButtonText {
	
	color: #aaa;
}
.soria .dijitButtonHover .dijitButtonNode,
.soria .dijitButtonNodeHover,
.soria .dijitToggleButtonHover .dijitButtonNode,
.soria .dijitDropDownButtonHover .dijitButtonNode,
.soria .dijitButtonContentsHover,
.soria .dijitUpArrowButtonHover,
.soria .dijitDownArrowButtonHover {
	
	
	color:#243C5F;
	background:#acc5e2 url("../images/buttonHover.png") repeat-x top left;
}
.soria .dijitButtonActive .dijitButtonNode,
.soria .dijitToggleButtonActive .dijitButtonNode,
.soria .dijitDropDownButtonActive .dijitButtonNode,
.soria .dijitComboButtonActive .dijitButtonContents,
.soria .dijitStackController .dijitToggleButtonChecked .dijitButtonNode {
	
	border-color:#657c9c;
	background: #91b4e5 url("../images/buttonActive.png") top left repeat-x;
}
.soria .dijitUpArrowButtonActive,
.soria .dijitDownArrowButtonActive {
	
	background: #91b4e5 url("../images/buttonActive.png") top left repeat-x;
}
.soria .dijitArrowButtonInner {
	background-image: url("../images/spriteArrows.png");
	background-repeat: no-repeat;
	background-position: 0 center;
	width: 11px;
	height: 11px;
}
.soria .dijitComboBox .dijitArrowButtonInner {
        background-position: 0 center;
}
.soria .dijitLeftArrowButton .dijitArrowButtonInner {
	background-position: -11px center;
}
.soria .dijitUpArrowButton .dijitArrowButtonInner {
	background-position: -22px center;
}
.soria .dijitRightArrowButton .dijitArrowButtonInner {
	background-position: -33px center;
}
.dj_ie6 .soria .dijitArrowButtonInner {
	background-image: url("../images/spriteArrows.gif");
}
.dj_ie .soria .dijitSpinner .dijitUpArrowButton .dijitArrowButtonInner {
	margin-top: 1px; 
}
.soria .dijitSpinnerButtonContainer {
	width: auto;
	padding: 0;
}
.soria .dijitSpinner .dijitArrowButton {
	width: 15px;
}
.soria .dijitSpinner .dijitSpinnerButtonInner {
	width: 15px;
}
.soria .dijitSpinner .dijitArrowButtonInner .dijitInputField {
	padding: 0;
}
