//>>built
define(
//begin v1.x content
/* used by both the editor and textarea widgets to provide information to screen reader users */
({
	iframeEditTitle: 'өңдеу аумағы',  // primary title for editable IFRAME, for screen readers when focus is in the editing area
	iframeFocusTitle: 'өңдеу аумағының жақтауы'  // secondary title for editable IFRAME when focus is on outer container
									 //  to let user know that focus has moved out of editing area and to the
									 //  parent element of the editing area
})
//end v1.x content
);

