/* Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */

#ifndef BINARY_LOG_INCLUDED
#define	BINARY_LOG_INCLUDED

/*
  This file is a convenience header file meant for the users, for inclusion
  of other related header files. Please refrain from adding any definitions
  to this header.
*/
#include "binlog_event.h"
#include "control_events.h"
#include "statement_events.h"
#include "load_data_events.h"
#include "rows_event.h"
#include "binary_log_types.h"

#endif	/* BINARY_LOG_INCLUDED */
