//>>built
define(
//begin v1.x content
({
	fontSize: "Mida",
	fontName: "Tipus de lletra",
	formatBlock: "Format",

	serif: "serif",
	"sans-serif": "sans-serif",
	monospace: "monoespai",
	cursive: "Cursiva",
	fantasy: "Fantasia",

	noFormat: "Cap",
	p: "Paràgraf",
	h1: "Títol",
	h2: "Subtítol",
	h3: "Subsubtítol",
	pre: "Format previ",

	1: "xx-petit",
	2: "x-petit",
	3: "petit",
	4: "mitjà",
	5: "gran",
	6: "x-gran",
	7: "xx-gran"
})
//end v1.x content
);
