/* claro/form/Common_rtl.css */

@import "../variables";

/*claro should not have the icon on the container
.claro .dijitTextBoxRtlError .dijitValidationIcon {
	border-left-width: 0;
	border-right-width: 1px;
}*/
.claro .dijitTextBoxRtlError .dijitValidationContainer {
	border-left-width: 0;
	border-right-width: 1px;
}
