//>>built
define(
//begin v1.x content
{
	"months-format-narrow": [
		"1月",
		"2月",
		"3月",
		"4月",
		"5月",
		"6月",
		"7月",
		"8月",
		"9月",
		"10月",
		"11月",
		"12月"
	],
	"field-weekday": "周天",
	"dateFormatItem-yQQQ": "y年QQQ",
	"dateFormatItem-yMEd": "y年M月d日，E",
	"dateFormatItem-MMMEd": "MMMd日E",
	"eraNarrow": [
		"公元前",
		"公元"
	],
	"dayPeriods-format-wide-earlyMorning": "清晨",
	"dayPeriods-format-wide-morning": "上午",
	"dateFormat-long": "y年M月d日",
	"months-format-wide": [
		"1月",
		"2月",
		"3月",
		"4月",
		"5月",
		"6月",
		"7月",
		"8月",
		"9月",
		"10月",
		"11月",
		"12月"
	],
	"dateTimeFormat-medium": "{1} {0}",
	"dayPeriods-format-wide-pm": "下午",
	"dateFormat-full": "y年M月d日EEEE",
	"dateFormatItem-Md": "M-d",
	"field-era": "时期",
	"dateFormatItem-yM": "yyyy-M",
	"months-standAlone-wide": [
		"一月",
		"二月",
		"三月",
		"四月",
		"五月",
		"六月",
		"七月",
		"八月",
		"九月",
		"十月",
		"十一月",
		"十二月"
	],
	"timeFormat-short": "ah:mm",
	"quarters-format-wide": [
		"第1季度",
		"第2季度",
		"第3季度",
		"第4季度"
	],
	"timeFormat-long": "zah时mm分ss秒",
	"field-year": "年",
	"dateFormatItem-yMMM": "y年MMM",
	"dateFormatItem-yQ": "y年QQQ",
	"dateFormatItem-yyyyMMMM": "y年MMMM",
	"field-hour": "小时",
	"dateFormatItem-MMdd": "MM-dd",
	"months-format-abbr": [
		"1月",
		"2月",
		"3月",
		"4月",
		"5月",
		"6月",
		"7月",
		"8月",
		"9月",
		"10月",
		"11月",
		"12月"
	],
	"dateFormatItem-yyQ": "yy年第Q季度",
	"timeFormat-full": "zzzzah时mm分ss秒",
	"field-day-relative+0": "今天",
	"field-day-relative+1": "明天",
	"field-day-relative+2": "后天",
	"dateFormatItem-H": "H时",
	"months-standAlone-abbr": [
		"一月",
		"二月",
		"三月",
		"四月",
		"五月",
		"六月",
		"七月",
		"八月",
		"九月",
		"十月",
		"十一月",
		"十二月"
	],
	"quarters-format-abbr": [
		"1季",
		"2季",
		"3季",
		"4季"
	],
	"quarters-standAlone-wide": [
		"第1季度",
		"第2季度",
		"第3季度",
		"第4季度"
	],
	"dateFormatItem-M": "M月",
	"days-standAlone-wide": [
		"星期日",
		"星期一",
		"星期二",
		"星期三",
		"星期四",
		"星期五",
		"星期六"
	],
	"dateFormatItem-yyMMM": "yy年MMM",
	"timeFormat-medium": "ah:mm:ss",
	"dateFormatItem-Hm": "H:mm",
	"quarters-standAlone-abbr": [
		"1季",
		"2季",
		"3季",
		"4季"
	],
	"eraAbbr": [
		"公元前",
		"公元"
	],
	"field-minute": "分钟",
	"field-dayperiod": "上午/下午",
	"days-standAlone-abbr": [
		"周日",
		"周一",
		"周二",
		"周三",
		"周四",
		"周五",
		"周六"
	],
	"dayPeriods-format-wide-night": "晚上",
	"dateFormatItem-d": "d日",
	"dateFormatItem-ms": "mm:ss",
	"field-day-relative+-1": "昨天",
	"dateFormatItem-h": "ah时",
	"dateTimeFormat-long": "{1}{0}",
	"field-day-relative+-2": "前天",
	"dateFormatItem-MMMd": "MMMd日",
	"dayPeriods-format-wide-midDay": "中午",
	"dateFormatItem-MEd": "M-dE",
	"dateTimeFormat-full": "{1}{0}",
	"field-day": "日",
	"days-format-wide": [
		"星期日",
		"星期一",
		"星期二",
		"星期三",
		"星期四",
		"星期五",
		"星期六"
	],
	"field-zone": "区域",
	"dateFormatItem-y": "y年",
	"months-standAlone-narrow": [
		"1月",
		"2月",
		"3月",
		"4月",
		"5月",
		"6月",
		"7月",
		"8月",
		"9月",
		"10月",
		"11月",
		"12月"
	],
	"dateFormatItem-yyMM": "yy-MM",
	"dateFormatItem-hm": "ah:mm",
	"days-format-abbr": [
		"周日",
		"周一",
		"周二",
		"周三",
		"周四",
		"周五",
		"周六"
	],
	"dateFormatItem-yMMMd": "y年MMMd日",
	"eraNames": [
		"公元前",
		"公元"
	],
	"days-format-narrow": [
		"日",
		"一",
		"二",
		"三",
		"四",
		"五",
		"六"
	],
	"field-month": "月",
	"days-standAlone-narrow": [
		"日",
		"一",
		"二",
		"三",
		"四",
		"五",
		"六"
	],
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "上午",
	"dateFormatItem-MMMMdd": "MMMMdd日",
	"dayPeriods-format-wide-weeHours": "凌晨",
	"dateFormat-short": "yy-M-d",
	"dayPeriods-format-wide-afternoon": "下午",
	"field-second": "秒钟",
	"dateFormatItem-yMMMEd": "y年MMMd日EEE",
	"dateFormatItem-Ed": "d日E",
	"field-week": "周",
	"dateFormat-medium": "yyyy-M-d",
	"dateFormatItem-yyyyM": "y年M月",
	"dateTimeFormat-short": "{1} {0}",
	"dateFormatItem-Hms": "H:mm:ss",
	"dateFormatItem-hms": "ah:mm:ss",
	"dateFormatItem-yyyy": "y年"
}
//end v1.x content
);