//>>built
define(
//begin v1.x content
({
	'bold': 'Έντονα',
	'copy': 'Αντιγραφή',
	'cut': 'Αποκοπή',
	'delete': 'Διαγραφή',
	'indent': 'Εσοχή',
	'insertHorizontalRule': 'Οριζόντια γραμμή',
	'insertOrderedList': 'Αριθμημένη λίστα',
	'insertUnorderedList': 'Λίστα με κουκίδες',
	'italic': 'Πλάγια',
	'justifyCenter': 'Στοίχιση στο κέντρο',
	'justifyFull': 'Πλήρης στοίχιση',
	'justifyLeft': 'Στοίχιση αριστερά',
	'justifyRight': 'Στοίχιση δεξιά',
	'outdent': 'Μείωση περιθωρίου',
	'paste': 'Επικόλληση',
	'redo': 'Ακύρωση αναίρεσης',
	'removeFormat': 'Αφαίρεση μορφοποίησης',
	'selectAll': 'Επιλογή όλων',
	'strikethrough': 'Διαγράμμιση',
	'subscript': 'Δείκτης',
	'superscript': 'Εκθέτης',
	'underline': 'Υπογράμμιση',
	'undo': 'Αναίρεση',
	'unlink': 'Αφαίρεση σύνδεσης',
	'createLink': 'Δημιουργία σύνδεσης',
	'toggleDir': 'Εναλλαγή κατεύθυνσης',
	'insertImage': 'Εισαγωγή εικόνας',
	'insertTable': 'Εισαγωγή/Τροποποίηση πίνακα',
	'toggleTableBorder': 'Εναλλαγή εμφάνισης περιγράμματος πίνακα',
	'deleteTable': 'Διαγραφή πίνακα',
	'tableProp': 'Ιδιότητα πίνακα',
	'htmlToggle': 'Πρωτογενής κώδικας HTML',
	'foreColor': 'Χρώμα προσκηνίου',
	'hiliteColor': 'Χρώμα φόντου',
	'plainFormatBlock': 'Στυλ παραγράφου',
	'formatBlock': 'Στυλ παραγράφου',
	'fontSize': 'Μέγεθος γραμματοσειράς',
	'fontName': 'Όνομα γραμματοσειράς',
	'tabIndent': 'Εσοχή με το πλήκτρο Tab',
	"fullScreen": "Εναλλαγή κατάστασης πλήρους οθόνης",
	"viewSource": "Προβολή προέλευσης HTML",
	"print": "Εκτύπωση",
	"newPage": "Νέα σελίδα",
	/* Error messages */
	'systemShortcut': 'Σε αυτό το πρόγραμμα πλοήγησης, η ενέργεια "${0}" είναι διαθέσιμη μόνο με τη χρήση μιας συντόμευσης πληκτρολογίου. Χρησιμοποιήστε τη συντόμευση ${1}.'
})

//end v1.x content
);
