//>>built
define(
//begin v1.x content
{
	"dateFormat-medium": "dd/MM/yyyy",
	"dateFormatItem-MMMEd": "E d MMM",
	"dateFormatItem-yMEd": "EEE، d/M/yyyy",
	"eraNarrow": [
		"هـ"
	],
	"months-format-narrow": [
		"م",
		"ص",
		"ر",
		"ر",
		"ج",
		"ج",
		"ر",
		"ش",
		"ر",
		"ش",
		"ذ",
		"ذ"
	],
	"timeFormat-full": "zzzz h:mm:ss a",
	"dateFormatItem-Md": "d/M",
	"months-standAlone-narrow": [
		"م",
		"ص",
		"ر",
		"ر",
		"ج",
		"ج",
		"ر",
		"ش",
		"ر",
		"ش",
		"ذ",
		"ذ"
	],
	"months-standAlone-wide": [
		"محرم",
		"صفر",
		"ربيع الأول",
		"ربيع الآخر",
		"جمادى الأولى",
		"جمادى الآخرة",
		"رجب",
		"شعبان",
		"رمضان",
		"شوال",
		"ذو القعدة",
		"ذو الحجة"
	],
	"eraNames": [
		"هـ"
	],
	"days-standAlone-narrow": [
		"ح",
		"ن",
		"ث",
		"ر",
		"خ",
		"ج",
		"س"
	],
	"dayPeriods-format-wide-pm": "م",
	"months-standAlone-abbr": [
		"محرم",
		"صفر",
		"ربيع الأول",
		"ربيع الآخر",
		"جمادى الأولى",
		"جمادى الآخرة",
		"رجب",
		"شعبان",
		"رمضان",
		"شوال",
		"ذو القعدة",
		"ذو الحجة"
	],
	"dayPeriods-format-wide-am": "ص",
	"quarters-standAlone-narrow": [
		"١",
		"٢",
		"٣",
		"٤"
	],
	"timeFormat-medium": "h:mm:ss a",
	"dateFormat-long": "d MMMM، y",
	"dateFormat-short": "d/M/yyyy",
	"dateFormatItem-yMMMEd": "EEE، d MMMM y",
	"months-format-wide": [
		"محرم",
		"صفر",
		"ربيع الأول",
		"ربيع الآخر",
		"جمادى الأولى",
		"جمادى الآخرة",
		"رجب",
		"شعبان",
		"رمضان",
		"شوال",
		"ذو القعدة",
		"ذو الحجة"
	],
	"dateFormatItem-yM": "M/yyyy",
	"timeFormat-short": "h:mm a",
	"months-format-abbr": [
		"محرم",
		"صفر",
		"ربيع الأول",
		"ربيع الآخر",
		"جمادى الأولى",
		"جمادى الآخرة",
		"رجب",
		"شعبان",
		"رمضان",
		"شوال",
		"ذو القعدة",
		"ذو الحجة"
	],
	"eraAbbr": [
		"هـ"
	],
	"timeFormat-long": "z h:mm:ss a",
	"days-format-wide": [
		"الأحد",
		"الإثنين",
		"الثلاثاء",
		"الأربعاء",
		"الخميس",
		"الجمعة",
		"السبت"
	],
	"dateFormatItem-yQ": "yyyy Q",
	"dateFormatItem-yMMM": "MMM y",
	"quarters-format-wide": [
		"الربع الأول",
		"الربع الثاني",
		"الربع الثالث",
		"الربع الرابع"
	],
	"dateFormat-full": "EEEE، d MMMM، y",
	"dateFormatItem-MMMd": "d MMM",
	"days-format-abbr": [
		"أحد",
		"إثنين",
		"ثلاثاء",
		"أربعاء",
		"خميس",
		"جمعة",
		"سبت"
	]
}
//end v1.x content
);