/* Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */

/*
  These are server-only members of enum_mysql_show_type defined in plugin.h.
  The values are only meant for internal use and their use by plugins is
  not supported.
*/
, SHOW_KEY_CACHE_LONG,
SHOW_KEY_CACHE_LONGLONG,
SHOW_LONG_STATUS,
SHOW_DOUBLE_STATUS,
SHOW_HAVE,
SHOW_MY_BOOL,
SHOW_HA_ROWS,
SHOW_SYS,
SHOW_LONG_NOFLUSH,
SHOW_LONGLONG_STATUS,
SHOW_LEX_STRING,
SHOW_SIGNED_LONG
