
.claro .dijitToolbar {
  border-bottom: 1px solid #b5bcc7;
  background-color: #efefef;
  background-image: url("images/commonHighlight.png");
  background-position: 0 0;
  background-repeat: repeat-x;
  padding: 2px 0 2px 4px;
  zoom: 1;
}
.claro .dijitToolbar label {
  padding: 0 3px 0 6px;
}
.claro .dijitToolbar .dijitButton .dijitButtonNode,
.claro .dijitToolbar .dijitDropDownButton .dijitButtonNode,
.claro .dijitToolbar .dijitComboButton .dijitButtonNode,
.claro .dijitToolbar .dijitToggleButton .dijitButtonNode,
.claro .dijitToolbar .dijitComboBox .dijitButtonNode {
  border-width: 0;
  
  padding: 2px;
  -moz-border-radius: 2px;
  border-radius: 2px;
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
  -webkit-transition-property: background-color;
  -moz-transition-property: background-color;
  transition-property: background-color;
  -webkit-transition-duration: 0.3s, 0.35s;
  -moz-transition-duration: 0.3s, 0.35s;
  transition-duration: 0.3s, 0.35s;
  background-image: url("images/commonHighlight.png");
  background-position: 0 -30px;
  background-repeat: repeat-x;
  background-color: rgba(171, 214, 255, 0);
}
.dj_ie .claro .dijitToolbar .dijitButton .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitDropDownButton .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitComboButton .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitToggleButton .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitComboBox .dijitButtonNode {
  background-color: transparent;
  
}
.dj_ie6 .claro .dijitToolbar .dijitButton .dijitButtonNode,
.dj_ie6 .claro .dijitToolbar .dijitDropDownButton .dijitButtonNode,
.dj_ie6 .claro .dijitToolbar .dijitComboButton .dijitButtonNode,
.dj_ie6 .claro .dijitToolbar .dijitToggleButton .dijitButtonNode,
.dj_ie6 .claro .dijitToolbar .dijitComboBox .dijitButtonNode {
  background: none;
  
}
.dj_ie .claro .dijitToolbar .dijitButtonHover .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitDropDownButtonHover .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitComboButton .dijitButtonNodeHover,
.dj_ie .claro .dijitToolbar .dijitComboButton .dijitDownArrowButtonHover,
.dj_ie .claro .dijitToolbar .dijitToggleButtonHover .dijitButtonNode {
  
  background-color: #abd6ff;
}
.dj_ie .claro .dijitToolbar .dijitButtonActive .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitDropDownButtonActive .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitComboButtonActive .dijitButtonNode,
.dj_ie .claro .dijitToolbar .dijitToggleButtonActive .dijitButtonNode {
  
  background-color: #abd6ff;
}
.claro .dijitToolbar .dijitComboButton .dijitStretch {
  
  -moz-border-radius: 2px 0 0 2px;
  border-radius: 2px 0 0 2px;
}
.claro .dijitToolbar .dijitComboButton .dijitArrowButton {
  
  -moz-border-radius: 0 2px 2px 0;
  border-radius: 0 2px 2px 0;
}
.claro .dijitToolbar .dijitComboBox .dijitButtonNode {
  padding: 0;
}
.claro .dijitToolbar .dijitButtonHover .dijitButtonNode,
.claro .dijitToolbar .dijitDropDownButtonHover .dijitButtonNode,
.claro .dijitToolbar .dijitToggleButtonHover .dijitButtonNode,
.claro .dijitToolbar .dijitComboButtonHover .dijitButtonNode {
  background-position: 0 0;
  border-width: 1px;
  background-color: #abd6ff;
  padding: 1px;
}
.claro .dijitToolbar .dijitComboButtonHover .dijitButtonNode, .claro .dijitToolbar .dijitComboButtonHover .dijitDownArrowButton {
  background-position: 0 0;
  background-color: #f3ffff;
}
.claro .dijitToolbar .dijitComboButtonHover .dijitButtonNodeHover, .claro .dijitToolbar .dijitComboButtonHover .dijitDownArrowButtonHover {
  background-color: #abd6ff;
}
.claro .dijitToolbar .dijitButtonActive .dijitButtonNode, .claro .dijitToolbar .dijitDropDownButtonActive .dijitButtonNode, .claro .dijitToolbar .dijitToggleButtonActive .dijitButtonNode {
  border-width: 1px;
  background-color: #7dbdfa;
  background-position: 0 -177px;
  padding: 1px;
}
.claro .dijitToolbar .dijitComboButtonActive {
  -webkit-transition-duration: 0.2s;
  -moz-transition-duration: 0.2s;
  transition-duration: 0.2s;
  border-width: 1px;
  padding: 0;
}
.claro .dijitToolbar .dijitComboButtonActive .dijitButtonNode, .claro .dijitToolbar .dijitComboButtonActive .dijitDownArrowButton {
  background-color: #f3ffff;
  background-position: 0 -177px;
  padding: 2px;
}
.claro .dijitToolbar .dijitComboButtonActive .dijitButtonNodeActive {
  background-color: #7dbdfa;
}
.claro .dijitToolbar .dijitComboButtonActive .dijitDownArrowButtonActive {
  background-color: #7dbdfa;
}
.claro .dijitToolbar .dijitComboButtonHover .dijitDownArrowButton, .claro .dijitToolbar .dijitComboButtonActive .dijitDownArrowButton {
  border-left-width: 0;
}
.claro .dijitToolbar .dijitComboButtonHover .dijitDownArrowButton {
  padding-left: 2px;
  
}
.claro .dijitToolbar .dijitToggleButtonChecked .dijitButtonNode {
  margin: 0;
  
  border-width: 1px;
  border-style: solid;
  background-image: none;
  border-color: #759dc0;
  background-color: #ffffff;
  padding: 1px;
}
.dj_ie6 .claro .dijitToolbar {
  background-image: none;
}
.claro .dijitToolbarSeparator {
  
  background: url("../../icons/images/editorIconsEnabled.png");
}
.claro .dijitDisabled .dijitToolbar {
  background: none;
  background-color: #efefef;
  border-bottom: 1px solid #d3d3d3;
}
.claro .dijitToolbar .dijitComboBoxDisabled .dijitArrowButtonInner {
  background-position: 0 50%;
}
