//>>built
define(
//begin v1.x content
{
	"HKD_displayName": "Hongkong-dollar",
	"CHF_displayName": "sveitsiske franc",
	"CHF_symbol": "CHF",
	"JPY_symbol": "JPY",
	"CAD_displayName": "kanadiske dollar",
	"CNY_displayName": "kinesiske yuan renminbi",
	"USD_symbol": "USD",
	"AUD_displayName": "australske dollar",
	"JPY_displayName": "japanske yen",
	"CAD_symbol": "CAD",
	"USD_displayName": "amerikanske dollar",
	"EUR_symbol": "EUR",
	"CNY_symbol": "CNY",
	"GBP_displayName": "britiske pund sterling",
	"GBP_symbol": "GBP",
	"AUD_symbol": "AUD",
	"EUR_displayName": "euro"
}
//end v1.x content
);