//>>built
define(
//begin v1.x content
({
// local representation of all CSS3 named colors, companion to dojo.colors.  To be used where descriptive information
// is required for each color, such as a palette widget, and not for specifying color programatically.

//Note: due to the SVG 1.0 spec additions, some of these are alternate spellings for the same color e.g. gray vs. gray.
//TODO: should we be using unique rgb values as keys instead and avoid these duplicates, or rely on the caller to do the reverse mapping?
aliceblue: "blanco azulado",
antiquewhite: "blanco antiguo",
aqua: "aguamarina",
aquamarine: "aguamarina 2",
azure: "blanco cielo",
beige: "beige",
bisque: "miel",
black: "negro",
blanchedalmond: "almendra pálido",
blue: "azul",
blueviolet: "azul violáceo",
brown: "marrón",
burlywood: "madera",
cadetblue: "azul cadete",
chartreuse: "verde pálido 2",
chocolate: "chocolate",
coral: "coral",
cornflowerblue: "azul aciano",
cornsilk: "crudo",
crimson: "carmesí",
cyan: "cian",
darkblue: "azul oscuro",
darkcyan: "cian oscuro",
darkgoldenrod: "ocre oscuro",
darkgray: "gris oscuro",
darkgreen: "verde oscuro",
darkgrey: "gris oscuro", // same as darkgray
darkkhaki: "caqui oscuro",
darkmagenta: "magenta oscuro",
darkolivegreen: "verde oliva oscuro",
darkorange: "naranja oscuro",
darkorchid: "orquídea oscuro",
darkred: "rojo oscuro",
darksalmon: "salmón oscuro",
darkseagreen: "verde mar oscuro",
darkslateblue: "azul pizarra oscuro",
darkslategray: "gris pizarra oscuro",
darkslategrey: "gris pizarra oscuro", // same as darkslategray
darkturquoise: "turquesa oscuro",
darkviolet: "violeta oscuro",
deeppink: "rosa fuerte",
deepskyblue: "azul cielo fuerte",
dimgray: "gris marengo",
dimgrey: "gris marengo", // same as dimgray
dodgerblue: "azul fuerte",
firebrick: "teja",
floralwhite: "blanco manteca",
forestgreen: "verde pino",
fuchsia: "fucsia",
gainsboro: "azul gainsboro",
ghostwhite: "blanco ligero",
gold: "oro",
goldenrod: "ocre",
gray: "gris",
green: "verde",
greenyellow: "amarillo verdoso",
grey: "gris", // same as gray
honeydew: "flor de rocío",
hotpink: "rosa oscuro",
indianred: "rojo teja",
indigo: "añil",
ivory: "marfil",
khaki: "caqui",
lavender: "lavanda",
lavenderblush: "lavanda rosácea",
lawngreen: "verde césped",
lemonchiffon: "amarillo pastel",
lightblue: "azul claro",
lightcoral: "coral claro",
lightcyan: "cian claro",
lightgoldenrodyellow: "ocre claro",
lightgray: "gris claro",
lightgreen: "verde claro",
lightgrey: "gris claro", // same as lightgray
lightpink: "rosa claro",
lightsalmon: "salmón claro",
lightseagreen: "verde mar claro",
lightskyblue: "azul cielo claro",
lightslategray: "gris pizarra claro",
lightslategrey: "gris pizarra claro", // same as lightslategray
lightsteelblue: "azul acero claro",
lightyellow: "amarillo claro",
lime: "lima",
limegreen: "lima limón",
linen: "blanco arena",
magenta: "magenta",
maroon: "granate",
mediumaquamarine: "aguamarina medio",
mediumblue: "azul medio",
mediumorchid: "orquídea medio",
mediumpurple: "púrpura medio",
mediumseagreen: "verde mar medio",
mediumslateblue: "azul pizarra medio",
mediumspringgreen: "verde primavera medio",
mediumturquoise: "turquesa medio",
mediumvioletred: "rojo violáceo medio",
midnightblue: "azul medianoche",
mintcream: "crema menta",
mistyrose: "rosa difuminado",
moccasin: "arena",
navajowhite: "blanco navajo",
navy: "azul marino",
oldlace: "encaje antiguo",
olive: "verde oliva",
olivedrab: "verde oliva pardusco",
orange: "naranja",
orangered: "rojo anaranjado",
orchid: "orquídea",
palegoldenrod: "ocre pálido",
palegreen: "verde pálido",
paleturquoise: "turquesa pálido",
palevioletred: "rojo violáceo pálido",
papayawhip: "papaya claro",
peachpuff: "melocotón",
peru: "perú",
pink: "rosa",
plum: "ciruela",
powderblue: "azul suave",
purple: "púrpura",
red: "rojo",
rosybrown: "marrón rosáceo",
royalblue: "azul real",
saddlebrown: "cuero",
salmon: "salmón",
sandybrown: "marrón arcilla",
seagreen: "verde mar",
seashell: "blanco marfil",
sienna: "siena",
silver: "plateado",
skyblue: "azul cielo",
slateblue: "azul pizarra",
slategray: "gris pizarra",
slategrey: "gris pizarra", // same as slategray
snow: "nieve",
springgreen: "verde fuerte",
steelblue: "azul acero",
tan: "canela",
teal: "verde azulado",
thistle: "cardo",
tomato: "tomate",
turquoise: "turquesa",
violet: "violeta",
wheat: "trigo",
white: "blanco",
whitesmoke: "blanco ahumado",
yellow: "amarillo",
yellowgreen: "verde amarillento"
})
//end v1.x content
);
