
.claro .dijitDialog {
  border: 1px solid #759dc0;
  -webkit-box-shadow: 0 1px 5px rgba(0, 0, 0, 0.25);
  -moz-box-shadow: 0 1px 5px rgba(0, 0, 0, 0.25);
  box-shadow: 0 1px 5px rgba(0, 0, 0, 0.25);
}
.claro .dijitDialogPaneContent {
  background: #ffffff repeat-x top left;
  border-top: 1px solid #759dc0;
  padding: 10px 8px;
  position: relative;
}
.claro .dijitDialogPaneContentArea {
  
  margin: -10px -8px;
  padding: 10px 8px;
}
.claro .dijitDialogPaneActionBar {
  
  background-color: #efefef;
  padding: 3px 5px 2px 7px;
  text-align: right;
  border-top: 1px solid #d3d3d3;
  margin: 10px -8px -10px;
}
.claro .dijitDialogPaneActionBar .dijitButton {
  float: none;
}
.claro .dijitDialogTitleBar {
  
  border: 1px solid #ffffff;
  border-top: none;
  background-color: #abd6ff;
  background-image: url("images/titlebar.png");
  background-repeat: repeat-x;
  padding: 5px 7px 4px 7px;
}
.claro .dijitDialogTitle {
  
  padding: 0 1px;
  font-size: 1.091em;
  color: #000000;
}
.claro .dijitDialogCloseIcon {
  
  background: url("images/dialogCloseIcon.png");
  background-repeat: no-repeat;
  position: absolute;
  right: 5px;
  height: 15px;
  width: 21px;
}
.dj_ie6 .claro .dijitDialogCloseIcon {
  background-image: url("images/dialogCloseIcon8bit.png");
}
.claro .dijitDialogCloseIconHover {
  background-position: -21px;
}
.claro .dijitDialogCloseIconActive {
  background-position: -42px;
}
.claro .dijitTooltip, .claro .dijitTooltipDialog {
  
  background: transparent;
  
}
.dijitTooltipBelow {
  
  padding-top: 13px;
  padding-left: 3px;
  padding-right: 3px;
}
.dijitTooltipAbove {
  
  padding-bottom: 13px;
  padding-left: 3px;
  padding-right: 3px;
}
.claro .dijitTooltipContainer {
  
  background-color: #ffffff;
  background-image: url("images/tooltipGradient.png");
  background-repeat: repeat-x;
  background-position: bottom;
  border: 1px solid #759dc0;
  padding: 6px 8px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: 0 1px 3px rgba(0, 0, 0, 0.25);
  -moz-box-shadow: 0 1px 3px rgba(0, 0, 0, 0.25);
  box-shadow: 0 1px 3px rgba(0, 0, 0, 0.25);
  font-size: 1em;
  color: #000000;
}
.dj_ie6 .claro .dijitTooltipContainer {
  background-image: none;
}
.claro .dijitTooltipConnector {
  
  border: 0;
  z-index: 2;
  background-image: url("images/tooltip.png");
  background-repeat: no-repeat;
  width: 16px;
  height: 14px;
}
.dj_ie6 .claro .dijitTooltipConnector {
  background-image: url("images/tooltip8bit.png");
}
.claro .dijitTooltipABRight .dijitTooltipConnector {
  
  left: auto !important;
  right: 3px;
}
.claro .dijitTooltipBelow .dijitTooltipConnector {
  
  top: 0;
  left: 3px;
  background-position: -31px 0;
  width: 16px;
  height: 14px;
}
.claro .dijitTooltipAbove .dijitTooltipConnector {
  
  bottom: 0;
  left: 3px;
  background-position: -15px 0;
  width: 16px;
  height: 14px;
}
.dj_ie7 .claro .dijitTooltipAbove .dijitTooltipConnector, .dj_ie6 .claro .dijitTooltipAbove .dijitTooltipConnector {
  bottom: -1px;
}
.claro .dijitTooltipLeft {
  padding-right: 14px;
}
.claro .dijitTooltipLeft .dijitTooltipConnector {
  
  right: 0;
  background-position: 0 0;
  width: 16px;
  height: 14px;
}
.claro .dijitTooltipRight {
  padding-left: 14px;
}
.claro .dijitTooltipRight .dijitTooltipConnector {
  
  left: 0;
  background-position: -48px 0;
  width: 16px;
  height: 14px;
}
.claro .dijitDialogUnderlay {
  background: #ffffff;
}
