//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yyMMM": "MMM yy",
	"dateFormatItem-Md": "d/M",
	"dateFormatItem-yMEd": "EEE, d/M/yyyy",
	"timeFormat-full": "HH:mm:ss zzzz",
	"timeFormat-medium": "HH:mm:ss",
	"dateFormatItem-yyyyMMMM": "MMMM y",
	"dateFormatItem-MEd": "E, d/M",
	"dateFormat-medium": "d MMM y",
	"dateFormatItem-MMdd": "dd/MM",
	"dateFormatItem-yyyyMM": "MM/yyyy",
	"dateFormat-full": "EEEE, d MMMM y",
	"timeFormat-long": "HH:mm:ss z",
	"timeFormat-short": "HH:mm",
	"dateFormat-short": "dd/MM/yyyy",
	"dateFormatItem-MMMMd": "d MMMM",
	"dateFormat-long": "d MMMM y",
	"dateFormatItem-MMMEd": "E d MMM"
}
//end v1.x content
);