//>>built
define(
//begin v1.x content
{
	"quarters-format-abbr": [
		"Τ1",
		"Τ2",
		"Τ3",
		"Τ4"
	],
	"dateFormat-medium": "d MMM y",
	"dateFormatItem-MMMEd": "E, d MMM",
	"dateFormatItem-MEd": "E, d/M",
	"dateFormatItem-yMEd": "EEE, d/M/yyyy",
	"eraNarrow": [
		"π.μ."
	],
	"timeFormat-full": "h:mm:ss a zzzz",
	"dateFormatItem-Md": "d/M",
	"dateFormatItem-EEEd": "EEE d",
	"eraNames": [
		"π.μ."
	],
	"days-standAlone-narrow": [
		"Κ",
		"Δ",
		"Τ",
		"Τ",
		"Π",
		"Π",
		"Σ"
	],
	"dayPeriods-format-wide-pm": "μ.μ.",
	"dayPeriods-format-wide-am": "π.μ.",
	"timeFormat-medium": "h:mm:ss a",
	"dateFormat-long": "d MMMM y",
	"dateFormat-short": "d/M/yy",
	"dateFormatItem-yMMMEd": "EEE, d MMM y",
	"dateFormatItem-yM": "M/yyyy",
	"timeFormat-short": "h:mm a",
	"eraAbbr": [
		"π.μ."
	],
	"timeFormat-long": "h:mm:ss a z",
	"days-format-wide": [
		"Κυριακή",
		"Δευτέρα",
		"Τρίτη",
		"Τετάρτη",
		"Πέμπτη",
		"Παρασκευή",
		"Σάββατο"
	],
	"dateFormatItem-yMMM": "LLL y",
	"quarters-format-wide": [
		"1ο τρίμηνο",
		"2ο τρίμηνο",
		"3ο τρίμηνο",
		"4ο τρίμηνο"
	],
	"dateFormat-full": "EEEE, d MMMM y",
	"dateFormatItem-MMMd": "d MMM",
	"days-format-abbr": [
		"Κυρ",
		"Δευ",
		"Τρι",
		"Τετ",
		"Πεμ",
		"Παρ",
		"Σαβ"
	]
}
//end v1.x content
);