//>>built
define(
//begin v1.x content
({
	'bold': 'Pogrubienie',
	'copy': 'Kopiuj',
	'cut': 'Wytnij',
	'delete': 'Usuń',
	'indent': 'Wcięcie',
	'insertHorizontalRule': 'Linia pozioma',
	'insertOrderedList': 'Lista numerowana',
	'insertUnorderedList': 'Lista wypunktowana',
	'italic': 'Kursywa',
	'justifyCenter': 'Wyrównaj do środka',
	'justifyFull': 'Wyrównaj do lewej i prawej',
	'justifyLeft': 'Wyrównaj do lewej',
	'justifyRight': 'Wyrównaj do prawej',
	'outdent': 'Usuń wcięcie',
	'paste': 'Wklej',
	'redo': 'Ponów',
	'removeFormat': 'Usuń formatowanie',
	'selectAll': 'Wybierz wszystko',
	'strikethrough': 'Przekreślenie',
	'subscript': 'Indeks dolny',
	'superscript': 'Indeks górny',
	'underline': 'Podkreślenie',
	'undo': 'Cofnij',
	'unlink': 'Usuń odsyłacz',
	'createLink': 'Utwórz odsyłacz',
	'toggleDir': 'Przełącz kierunek',
	'insertImage': 'Wstaw obraz',
	'insertTable': 'Wstaw/edytuj tabelę',
	'toggleTableBorder': 'Przełącz ramkę tabeli',
	'deleteTable': 'Usuń tabelę',
	'tableProp': 'Właściwość tabeli',
	'htmlToggle': 'Kod źródłowy HTML',
	'foreColor': 'Kolor pierwszego planu',
	'hiliteColor': 'Kolor tła',
	'plainFormatBlock': 'Styl akapitu',
	'formatBlock': 'Styl akapitu',
	'fontSize': 'Wielkość czcionki',
	'fontName': 'Nazwa czcionki',
	'tabIndent': 'Wcięcie o tabulator',
	"fullScreen": "Przełącz pełny ekran",
	"viewSource": "Wyświetl kod źródłowy HTML",
	"print": "Drukuj",
	"newPage": "Nowa strona",
	/* Error messages */
	'systemShortcut': 'Działanie ${0} jest dostępne w tej przeglądarce wyłącznie przy użyciu skrótu klawiaturowego. Należy użyć klawiszy ${1}.',
	'ctrlKey':'Ctrl+${0}'
})

//end v1.x content
);
