
.nihilo .dijitToggleButton .dijitRadio,
.nihilo .dijitToggleButton .dijitRadioIcon {
	background-image: url('../images/spriteRadio.gif');
}
.nihilo .dijitRadio,
.nihilo .dijitRadioIcon	{		
	background-image: url('../images/spriteRadio.gif'); 
	background-repeat: no-repeat;
	width: 16px;
	height: 16px;
	margin: 0;
	padding: 0;
}
.nihilo .dijitRadio,
.nihilo .dijitToggleButton .dijitRadioIcon {
	
	background-position: -16px;
}
.nihilo .dijitRadioChecked,
.nihilo .dijitToggleButtonChecked .dijitRadioIcon {
	
	background-position: 0;
}
.nihilo .dijitRadioDisabled {
	
	background-position: -48px;
}
.nihilo .dijitRadioCheckedDisabled {
	
	background-position: -32px;
}
.nihilo .dijitRadioHover {
	
	background-position: -80px;
}
.nihilo .dijitRadioCheckedHover {
	
	background-position: -64px;
}