//>>built
define(
//begin v1.x content
({
	'bold': 'Negrita',
	'copy': 'Copiar',
	'cut': 'Cortar',
	'delete': 'Suprimir',
	'indent': 'Sangría',
	'insertHorizontalRule': 'Regla horizontal',
	'insertOrderedList': 'Lista numerada',
	'insertUnorderedList': 'Lista con viñetas',
	'italic': 'Cursiva',
	'justifyCenter': 'Alinear centro',
	'justifyFull': 'Justificar',
	'justifyLeft': 'Alinear izquierda',
	'justifyRight': 'Alinear derecha',
	'outdent': 'Anular sangría',
	'paste': 'Pegar',
	'redo': 'Rehacer',
	'removeFormat': 'Eliminar formato',
	'selectAll': 'Seleccionar todo',
	'strikethrough': 'Tachado',
	'subscript': 'Subíndice',
	'superscript': 'Superíndice',
	'underline': 'Subrayado',
	'undo': 'Deshacer',
	'unlink': 'Eliminar enlace',
	'createLink': 'Crear enlace',
	'toggleDir': 'Conmutar dirección',
	'insertImage': 'Insertar imagen',
	'insertTable': 'Insertar/Editar tabla',
	'toggleTableBorder': 'Conmutar borde de tabla',
	'deleteTable': 'Suprimir tabla',
	'tableProp': 'Propiedad de tabla',
	'htmlToggle': 'Fuente HTML',
	'foreColor': 'Color de primer plano',
	'hiliteColor': 'Color de segundo plano',
	'plainFormatBlock': 'Estilo de párrafo',
	'formatBlock': 'Estilo de párrafo',
	'fontSize': 'Tamaño de font',
	'fontName': 'Nombre de font',
	'tabIndent': 'Sangría de tabulador',
	"fullScreen": "Conmutar pantalla completa",
	"viewSource": "Ver fuente HTML",
	"print": "Imprimir",
	"newPage": "Nueva página",
	/* Error messages */
	'systemShortcut': 'La acción "${0}" sólo está disponible en su navegador mediante un atajo de teclado. Utilice ${1}.',
	'ctrlKey':'control+${0}'
})

//end v1.x content
);
