//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "M/y",
	"dateFormatItem-yyyyMMMEd": "EEE, d MMM y G",
	"dateFormatItem-yQ": "Q y",
	"dayPeriods-format-wide-pm": "p.m.",
	"eraNames": [
		"AH"
	],
	"dateFormatItem-MMMEd": "E d MMM",
	"dateFormatItem-hms": "hh:mm:ss a",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-MMM": "LLL",
	"months-standAlone-narrow": [
		"1",
		"2",
		"3",
		"4",
		"5",
		"6",
		"7",
		"8",
		"9",
		"10",
		"11",
		"12"
	],
	"dayPeriods-format-wide-am": "a.m.",
	"dateFormatItem-MMMdd": "dd-MMM",
	"dateFormatItem-yyyy": "y G",
	"months-standAlone-abbr": [
		"Muh.",
		"Saf.",
		"Rab. I",
		"Rab. II",
		"Jum. I",
		"Jum. II",
		"Raj.",
		"Sha.",
		"Ram.",
		"Shaw.",
		"Dhuʻl-Q.",
		"Dhuʻl-H."
	],
	"dateFormatItem-yMMM": "MMM y",
	"days-standAlone-narrow": [
		"D",
		"L",
		"M",
		"M",
		"J",
		"V",
		"S"
	],
	"eraAbbr": [
		"AH"
	],
	"dateFormatItem-yyyyMMMM": "MMMM 'de' y G",
	"dateFormat-long": "d 'de' MMMM 'de' y G",
	"dateFormatItem-EEEd": "EEE d",
	"dateFormatItem-Hm": "HH:mm",
	"dateFormatItem-MMd": "d/MM",
	"dateFormatItem-yyMM": "MM/y G",
	"dateFormat-medium": "dd/MM/y G",
	"dateFormatItem-Hms": "HH:mm:ss",
	"dateFormatItem-yyMMM": "MMM-y G",
	"dateFormatItem-yyQQQQ": "QQQQ 'de' y G",
	"dateFormatItem-ms": "mm:ss",
	"months-standAlone-wide": [
		"Muharram",
		"Safar",
		"Rabiʻ I",
		"Rabiʻ II",
		"Jumada I",
		"Jumada II",
		"Rajab",
		"Shaʻban",
		"Ramadan",
		"Shawwal",
		"Dhuʻl-Qiʻdah",
		"Dhuʻl-Hijjah"
	],
	"dateFormatItem-yyyyMEd": "EEE d/M/y G",
	"dateFormatItem-MMMd": "d MMM",
	"dateFormatItem-yyQ": "Q y G",
	"months-format-abbr": [
		"Muh.",
		"Saf.",
		"Rab. I",
		"Rab. II",
		"Jum. I",
		"Jum. II",
		"Raj.",
		"Sha.",
		"Ram.",
		"Shaw.",
		"Dhuʻl-Q.",
		"Dhuʻl-H."
	],
	"dateFormatItem-MMMMd": "d 'de' MMMM",
	"quarters-format-abbr": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"days-format-abbr": [
		"dom",
		"lun",
		"mar",
		"mié",
		"jue",
		"vie",
		"sáb"
	],
	"dateFormatItem-M": "L",
	"dateFormatItem-MEd": "E, d/M",
	"dateFormatItem-yyyyQQQ": "QQQ y G",
	"months-format-narrow": [
		"1",
		"2",
		"3",
		"4",
		"5",
		"6",
		"7",
		"8",
		"9",
		"10",
		"11",
		"12"
	],
	"dateFormatItem-hm": "hh:mm a",
	"dateFormat-short": "dd/MM/y G",
	"dateFormatItem-yyyyM": "M/y G",
	"dateFormatItem-yMMMEd": "EEE, d MMM y",
	"dateFormat-full": "EEEE d 'de' MMMM 'de' y G",
	"dateFormatItem-Md": "d/M",
	"dateFormatItem-yyyyQ": "Q y G",
	"dateFormatItem-yMEd": "EEE d/M/y",
	"months-format-wide": [
		"Muharram",
		"Safar",
		"Rabiʻ I",
		"Rabiʻ II",
		"Jumada I",
		"Jumada II",
		"Rajab",
		"Shaʻban",
		"Ramadan",
		"Shawwal",
		"Dhuʻl-Qiʻdah",
		"Dhuʻl-Hijjah"
	],
	"dateFormatItem-yyyyMMM": "MMM y G",
	"dateFormatItem-d": "d",
	"quarters-format-wide": [
		"1er trimestre",
		"2º trimestre",
		"3er trimestre",
		"4º trimestre"
	],
	"eraNarrow": [
		"AH"
	],
	"days-format-wide": [
		"domingo",
		"lunes",
		"martes",
		"miércoles",
		"jueves",
		"viernes",
		"sábado"
	],
	"dateFormatItem-h": "hh a"
}
//end v1.x content
);