//>>built
define(
//begin v1.x content
{
	"months-format-narrow": [
		"G",
		"F",
		"M",
		"A",
		"M",
		"G",
		"L",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"field-weekday": "giorno della settimana",
	"dateFormatItem-yyQQQQ": "QQQQ yy",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-yMEd": "EEE, d/M/y",
	"dateFormatItem-MMMEd": "EEE d MMM",
	"eraNarrow": [
		"aC",
		"dC"
	],
	"dateFormat-long": "dd MMMM y",
	"months-format-wide": [
		"gennaio",
		"febbraio",
		"marzo",
		"aprile",
		"maggio",
		"giugno",
		"luglio",
		"agosto",
		"settembre",
		"ottobre",
		"novembre",
		"dicembre"
	],
	"dayPeriods-format-wide-pm": "p.",
	"dateFormat-full": "EEEE d MMMM y",
	"dateFormatItem-Md": "d/M",
	"field-era": "era",
	"dateFormatItem-yM": "M/y",
	"months-standAlone-wide": [
		"Gennaio",
		"Febbraio",
		"Marzo",
		"Aprile",
		"Maggio",
		"Giugno",
		"Luglio",
		"Agosto",
		"Settembre",
		"Ottobre",
		"Novembre",
		"Dicembre"
	],
	"timeFormat-short": "HH:mm",
	"quarters-format-wide": [
		"1o trimestre",
		"2o trimestre",
		"3o trimestre",
		"4o trimestre"
	],
	"timeFormat-long": "HH:mm:ss z",
	"field-year": "anno",
	"dateFormatItem-yMMM": "MMM y",
	"dateFormatItem-yQ": "Q-yyyy",
	"dateFormatItem-yyyyMMMM": "MMMM y",
	"field-hour": "ora",
	"dateFormatItem-MMdd": "dd/MM",
	"months-format-abbr": [
		"gen",
		"feb",
		"mar",
		"apr",
		"mag",
		"giu",
		"lug",
		"ago",
		"set",
		"ott",
		"nov",
		"dic"
	],
	"dateFormatItem-yyQ": "Q yy",
	"timeFormat-full": "HH:mm:ss zzzz",
	"field-day-relative+0": "oggi",
	"field-day-relative+1": "domani",
	"field-day-relative+2": "dopodomani",
	"field-day-relative+3": "tra tre giorni",
	"months-standAlone-abbr": [
		"gen",
		"feb",
		"mar",
		"apr",
		"mag",
		"giu",
		"lug",
		"ago",
		"set",
		"ott",
		"nov",
		"dic"
	],
	"quarters-format-abbr": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"quarters-standAlone-wide": [
		"1o trimestre",
		"2o trimestre",
		"3o trimestre",
		"4o trimestre"
	],
	"dateFormatItem-M": "L",
	"days-standAlone-wide": [
		"Domenica",
		"Lunedì",
		"Martedì",
		"Mercoledì",
		"Giovedì",
		"Venerdì",
		"Sabato"
	],
	"timeFormat-medium": "HH:mm:ss",
	"dateFormatItem-Hm": "HH:mm",
	"quarters-standAlone-abbr": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"eraAbbr": [
		"aC",
		"dC"
	],
	"field-minute": "minuto",
	"field-dayperiod": "periodo del giorno",
	"days-standAlone-abbr": [
		"dom",
		"lun",
		"mar",
		"mer",
		"gio",
		"ven",
		"sab"
	],
	"dateFormatItem-d": "d",
	"dateFormatItem-ms": "mm:ss",
	"field-day-relative+-1": "ieri",
	"dateFormatItem-h": "hh a",
	"field-day-relative+-2": "l'altro ieri",
	"field-day-relative+-3": "tre giorni fa",
	"dateFormatItem-MMMd": "d MMM",
	"dateFormatItem-MEd": "EEE d/M",
	"field-day": "giorno",
	"days-format-wide": [
		"domenica",
		"lunedì",
		"martedì",
		"mercoledì",
		"giovedì",
		"venerdì",
		"sabato"
	],
	"field-zone": "zona",
	"dateFormatItem-y": "y",
	"months-standAlone-narrow": [
		"G",
		"F",
		"M",
		"A",
		"M",
		"G",
		"L",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dateFormatItem-yyMM": "MM/yy",
	"dateFormatItem-hm": "hh:mm a",
	"days-format-abbr": [
		"dom",
		"lun",
		"mar",
		"mer",
		"gio",
		"ven",
		"sab"
	],
	"eraNames": [
		"a.C.",
		"d.C"
	],
	"days-format-narrow": [
		"D",
		"L",
		"M",
		"M",
		"G",
		"V",
		"S"
	],
	"field-month": "mese",
	"days-standAlone-narrow": [
		"D",
		"L",
		"M",
		"M",
		"G",
		"V",
		"S"
	],
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "m.",
	"dateFormatItem-MMMMdd": "dd MMMM",
	"dateFormat-short": "dd/MM/yy",
	"field-second": "secondo",
	"dateFormatItem-yMMMEd": "EEE d MMM y",
	"dateFormatItem-Ed": "E d",
	"field-week": "settimana",
	"dateFormat-medium": "dd/MMM/y",
	"dateFormatItem-Hms": "HH:mm:ss",
	"dateFormatItem-hms": "hh:mm:ss a"
}
//end v1.x content
);