
.claro .dijitTextBox, .claro .dijitInputInner {
  color: #000000;
}
.claro .dijitTextBoxError .dijitValidationContainer {
  background-color: #d46464;
  background-image: url("../form/images/error.png");
  background-position: top center;
  border: solid #d46464 0;
  border-left-width: 1px;
  width: 9px;
}
.claro .dijitTextBoxError .dijitValidationIcon {
  width: 0;
  background-color: transparent !important;
  
}
.claro .dijitTextArea, .claro .dijitInputField .dijitPlaceHolder {
  padding: 2px;
}
.claro .dijitTextBox .dijitInputField {
  padding: 1px 2px;
}
.dj_gecko .claro .dijitTextBox .dijitInputInner, .dj_webkit .claro .dijitTextBox .dijitInputInner {
  padding: 1px;
}
.claro .dijitTextBox, .claro .dijitTextBox .dijitButtonNode {
  
  border-color: #b5bcc7;
  -webkit-transition-property: background-color, border;
  -moz-transition-property: background-color, border;
  transition-property: background-color, border;
  -webkit-transition-duration: 0.35s;
  -moz-transition-duration: 0.35s;
  transition-duration: 0.35s;
}
.claro .dijitTextBox {
  background-color: #ffffff;
}
.claro .dijitTextBoxHover, .claro .dijitTextBoxHover .dijitButtonNode {
  border-color: #759dc0;
  -webkit-transition-duration: 0.25s;
  -moz-transition-duration: 0.25s;
  transition-duration: 0.25s;
}
.claro .dijitTextBoxHover {
  background-color: #e5f2fe;
  background-image: url("../form/images/textBox_back.png");
  background-repeat: repeat-x;
}
.claro .dijitTextBoxError, .claro .dijitTextBoxError .dijitButtonNode {
  border-color: #d46464;
}
.claro .dijitTextBoxError, .claro .dijitTextBoxError .dijitInputContainer {
  background-color: #ffffff;
}
.claro .dijitTextBoxFocused, .claro .dijitTextBoxFocused .dijitButtonNode {
  border-color: #759dc0;
  -webkit-transition-duration: 0.1s;
  -moz-transition-duration: 0.1s;
  transition-duration: 0.1s;
}
.claro .dijitTextBoxFocused {
  background-color: #ffffff;
  background-image: url("../form/images/textBox_back.png");
  background-repeat: repeat-x;
}
.claro .dijitTextBoxFocused .dijitInputContainer {
  background: #ffffff;
}
.claro .dijitTextBoxErrorFocused, .claro .dijitTextBoxErrorFocused .dijitButtonNode {
  border-color: #ce5050;
}
.claro .dijitTextBoxDisabled, .claro .dijitTextBoxDisabled .dijitButtonNode {
  border-color: #d3d3d3;
}
.claro .dijitTextBoxDisabled, .claro .dijitTextBoxDisabled .dijitInputContainer {
  background-color: #efefef;
  background-image: none;
}
.claro .dijitTextBoxDisabled, .claro .dijitTextBoxDisabled .dijitInputInner {
  color: #818181;
}
.dj_webkit .claro .dijitTextBoxDisabled input {
  
  color: #757575;
}
.dj_webkit .claro textarea.dijitTextAreaDisabled {
  
  color: #1b1b1b;
}
.claro .dijitComboBox .dijitArrowButtonInner {
  background-image: url("../form/images/commonFormArrows.png");
  background-position: -35px 53%;
  background-repeat: no-repeat;
  margin: 0;
  width: 16px;
  border: 1px solid #ffffff;
}
.claro .dijitToolbar .dijitComboBox .dijitArrowButtonInner {
  border: none;
}
.claro .dijitToolbar .dijitComboBox .dijitArrowButtonInner {
  border: none;
}
.claro .dijitTextBox .dijitInputInner, .claro .dijitTextBox .dijitValidationContainer {
  padding: 1px 0;
}
.claro .dijitComboBox .dijitButtonNode {
  background-color: #efefef;
  background-image: url("../form/images/formHighlight.png");
  background-repeat: repeat-x;
}
.claro .dijitComboBoxOpenOnClickHover .dijitButtonNode, .claro .dijitComboBox .dijitDownArrowButtonHover {
  background-color: #abd6ff;
}
.claro .dijitComboBoxOpenOnClickHover .dijitArrowButtonInner, .claro .dijitComboBox .dijitDownArrowButtonHover .dijitArrowButtonInner {
  background-position: -70px 53%;
}
.claro .dijitComboBox .dijitHasDropDownOpen {
  background-color: #7dbdfa;
  background-position: 0 -177px;
  padding: 1px;
}
.claro .dijitComboBox .dijitHasDropDownOpen .dijitArrowButtonInner {
  background-position: -70px 53%;
  border: 0 none;
}
.claro div.dijitComboBoxDisabled .dijitArrowButtonInner {
  
  background-position: 0 50%;
  background-color: #efefef;
}
.dj_ff3 .claro .dijitInputField input[type="hidden"] {
  display: none;
  height: 0;
  width: 0;
}
.dj_ie6 .claro .dijitTextBox, .dj_ie6 .claro .dijitComboBox .dijitButtonNode {
  background-image: none;
}
.dj_borderbox .claro .dijitComboBox .dijitHasDropDownOpen .dijitArrowButtonInner {
  width: 18px;
}
.dj_borderbox .claro .dijitComboBoxFocused .dijitHasDropDownOpen .dijitArrowButtonInner {
  width: 16px;
}
