
.claro .dijitInlineEditBoxDisplayMode {
  border: 1px solid transparent;
}
.claro .dijitInlineEditBoxDisplayModeHover {
  background-color: #e5f2fe;
  border: solid 1px #759dc0;
}
.dj_ie6 .claro .dijitInlineEditBoxDisplayMode {
  border: none;
}
