//>>built
define(
//begin v1.x content
({
	createLinkTitle: "Propriedades de Link",
	insertImageTitle: "Propriedades de Imagem",
	url: "URL:",
	text: "Descrição:",
	target: "Destino:",
	set: "Definir",
	currentWindow: "Janela Atual",
	parentWindow: "Janela Pai",
	topWindow: "Primeira Janela",
	newWindow: "Nova Janela"
})

//end v1.x content
);
