/* Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */

#ifndef LOGGER_H
#define LOGGER_H

namespace keyring {

class ILogger
{
public:
  virtual void log(plugin_log_level level, const char *message)= 0;
  virtual ~ILogger() {}
};

class Logger : public ILogger
{
public:
  Logger(MYSQL_PLUGIN plugin_info_ptr)
    : plugin_info_ptr(plugin_info_ptr)
  {}
  ~Logger() {}
  void log(plugin_log_level level, const char *message)
  {
    my_plugin_log_message(&plugin_info_ptr, level, "%s", message);
  }
private:
  MYSQL_PLUGIN plugin_info_ptr;
};

} //namespace keyring

#endif //LOGGER_H
