
.dijitEditorRtl .dijitEditorIcon {
	background-image: url('images/editorIconsEnabled_rtl.png'); 
}
.dijitEditorRtlDisabled .dijitEditorIcon {
	background-image: url('images/editorIconsDisabled_rtl.png'); 
}
.dijitToolbarRtl .dijitToolbarSeparator {
	background-image: url('images/editorIconsEnabled_rtl.png');
}
