
.dijitRtl .dijitPlaceHolder {
	left: auto;
	right: 0;
}
.dijitMenuItemRtl {
	text-align: right;
}
.dj_iequirks .dijitComboButtonRtl button {
	
	float:left;
}
.dijitTextBoxRtl .dijitValidationContainer,
.dijitTextBoxRtl .dijitSpinnerButtonContainer,
.dijitComboBoxRtl .dijitArrowButtonContainer {
	
	border-right-width: 1px !important;
	border-left-width: 0 !important;
}
.dijitSpinnerRtl .dijitSpinnerButtonContainer .dijitArrowButton {
	right: 0;
	left: auto;
}
.dj_ie7 .dijitInputContainer {
	
	display: inline-block;
}
.dijitTextBoxRtl .dijitSpinnerButtonContainer,
.dijitTextBoxRtl .dijitValidationContainer,
.dijitTextBoxRtl .dijitArrowButtonContainer {
	float: left;
}
.dijitCalendarRtl .dijitCalendarNextYear {
	margin:0 0.55em 0 0;
}
.dijitCalendarRtl .dijitCalendarPreviousYear {
	margin:0 0 0 0.55em;
}
.dijitSliderRtl .dijitSliderImageHandleV {
	left:auto;
}
.dijitSliderRtl .dijitSliderImageHandleH {
	left:-50%;
}
.dijitSliderRtl .dijitSliderMoveableH {
	right:auto;
	left:0;
}
.dijitSliderRtl .dijitRuleContainerV {
	float:right;
}
.dj_ie .dijitSliderRtl .dijitRuleContainerV {
	text-align:right;
}
.dj_ie .dijitSliderRtl .dijitRuleLabelV {
	text-align:left;
}
.dj_ie .dijitSliderRtl .dijitRuleLabelH {
	zoom:1;
}
.dijitSliderRtl .dijitSliderProgressBarH {
	
	float:right;
	right:0;
	left:auto;
}
.dijitRtl .dijitContentPaneLoading .dijitIconLoading,
.dijitRtl .dijitContentPaneError .dijitIconError {
	margin-right: 0;
	margin-left: 9px;
}
.dijitTabControllerRtl .nowrapTabStrip {
    text-align: right;  
}
.dijitTabRtl .dijitTabCloseButton {
	margin-left: 0;
	margin-right: 1em;
}
.dj_ie .dijitTimePickerRtl .dijitTimePickerItem {
	width:100%;
}
.dijitColorPaletteRtl .dijitColorPaletteUnder {
	
	left: auto;
	right: 0;
}
.dijitSelectRtl .dijitButtonContents {
	text-align: right;
}
.dijitRtl .nihilo .dijitTab {
	margin-right:0;
	margin-left:2px;		
}
.dijitRtl .nihilo .dijitTabContainer .tabStripButton {
	margin-left: 0;
}
.dijitRtl .nihilo .dijitTabContainerTopStrip,
.dijitRtl .nihilo .dijitTabContainerBottomStrip,
.dijitRtl .nihilo .dijitTabContainerTop-tabs,
.dijitRtl .nihilo .dijitTabContainerBottom-tabs {
	padding-left: 0;
	padding-right: 3px;
}
.dijitRtl .nihilo .dijitTabInnerDiv {
	padding-left: 3px;
	padding-right: 4px;
}
.dijitRtl .nihilo .dijitTabPaneWrapper {
	#zoom: 1;
}
.dj_ie-rtl .nihilo .dijitTabContainerLeft-tabs {
	margin-left: 0 !important;
}
.dj_ie-rtl .nihilo .dijitTabContainerRight-tabs {
	margin-right: 0 !important;
}
.dijitRtl .nihilo .dijitTabContainerLeft-tabs .dijitTab,
.dijitRtl .nihilo .dijitTabContainerRight-tabs .dijitTab {
	margin-left:0;
}
.dj_ie-rtl .nihilo .dijitTab .dijitTabInnerDiv{
	width : 1%;
}
.dijitRtl .nihilo .dijitAccordionArrow {
	background-position: 0 top;
}
.dijitRtl .nihilo .dijitAccordionTitleSelected .dijitAccordionArrow {
	
	background-position: -15px top;
}
.dijitRtl .nihilo .dijitSliderProgressBarH,
.dijitRtl .nihilo .dijitSliderRemainingBarH,
.dijitRtl .nihilo .dijitSliderLeftBumper,
.dijitRtl .nihilo .dijitSliderRightBumper,
.dijitRtl .nihilo .dijitSliderTopBumper {
	background-position: top right;
}
.dijitRtl .nihilo .dijitSliderProgressBarV,
.dijitRtl .nihilo .dijitSliderRemainingBarV,
.dijitRtl .nihilo .dijitSliderBottomBumper {
	background-position: bottom right;
}
.dijitRtl .nihilo .dijitSliderLeftBumper {
	border-left-width: 0;
	border-right-width: 1px;
}
.dijitRtl .nihilo .dijitSliderRightBumper {
	border-left-width: 1px;
	border-right-width: 0;
}
.dijitRtl .nihilo .dijitSliderIncrementIconH {
	background:url(images/spriteRoundedIconsSmall.png) no-repeat left top;
	#background:url(images/spriteRoundedIconsSmall.gif) no-repeat left top;
}
.dijitRtl .nihilo .dijitSliderDecrementIconH {
	background:url(images/spriteRoundedIconsSmall.png) no-repeat -30px top;
	#background:url(images/spriteRoundedIconsSmall.gif) no-repeat -30px top;
}
.dijitRtl .nihilo .dijitComboBox .dijitButtonNode {
	border-width: 0 0 0 1px;
}
.dijitRtl .nihilo .dijitSelect .dijitButtonContents {
	border-left: none;
	border-right-width: 1px;
}
.dijitRtl .nihilo .dijitTreeNode,
.dijitRtl .nihilo .dijitTreeExpandoLeaf {
	
    background-image: none;
}
.dijitRtl .nihilo .dijitTreeContent {
    padding-left: 0;
    padding-right: 1px;
}
.dijitRtl .nihilo .dijitTreeExpandoOpened {
	
	background: url(images/spriteTree_rtl.gif) no-repeat -18px top;
}
.dijitRtl .nihilo .dijitTreeExpandoClosed {
	
	background-image: url(images/spriteTree_rtl.gif);
}
.dijitRtl .nihilo .dijitTitlePane .dijitClosed .dijitArrowNode {
	background-position: 0 top;
}
.dj_ie6-rtl .nihilo .dijitTitlePane .dijitClosed .dijitArrowNode {
	background-position: 0 top;
}
.dijitRtl .nihilo .dijitCalendarDecrease {
	background-position: -30px top;
}
.dijitRtl .nihilo .dijitCalendarIncrease {
	background-position: 0 top;
}
.dj_ie6-rtl .nihilo .dijitTimePickerMarkerHover,
.dj_ie7-rtl .nihilo .dijitTimePickerMarkerHover {
        border-top: 0; 
}
.dijitRtl .nihilo .dijitDialogTitleBar .dijitDialogCloseIcon {
	right: auto;
	left: 5px;
}
.dijitRtl .nihilo .dijitMenuItem .dijitMenuItemIcon {
	padding-left: 3px;
	padding-right: 0;
}
.dijitRtl .nihilo .dijitMenuItem .dijitMenuExpand {
	background-position: 0 top;
}
.dijitRtl .nihilo .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsEnabled_rtl.png); 
}
.dijitRtl .nihilo .dijitDisabled .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsDisabled_rtl.png); 
}
.dijitEditorRtl .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsEnabled_rtl.png); 
}
.dijitEditorRtlDisabled .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsDisabled_rtl.png); 
}
.dijitToolbarRtl .dijitToolbarSeparator {
	background-image: url(../../icons/images/editorIconsEnabled_rtl.png);
}
