//>>built
// generated by build app
define("dojox/cometd/RestChannels-loadInit", [], {
	names:["dijit","dojo","dojox"],
	def:function(dijit,dojo,dojox){
dojo.provide("dojox.cometd.RestChannels");
dojo.require("dojox.rpc.Client");
dojo.require("dojo._base.url");
dojo.requireIf(dojox.data && !!dojox.data.JsonRestStore,"dojox.data.restListener");

}
});

