
.claro .dijitTreeNode {
  zoom: 1;
  
}
.claro .dijitTreeIsRoot {
  background-image: none;
}
.claro .dijitTreeRow, .claro .dijitTreeNode .dojoDndItemBefore, .claro .dijitTreeNode .dojoDndItemAfter {
  
  padding: 4px 1px 2px 0;
  margin: 0 1px;
  
  background-color: none;
  background-color: transparent;
  background-color: rgba(171, 214, 255, 0);
  background-position: 0 0;
  background-repeat: repeat-x;
  border-color: rgba(118, 157, 192, 0);
  border-width: 0;
  color: #000000;
  -webkit-transition-property: background-color, border-color;
  -moz-transition-property: background-color, border-color;
  transition-property: background-color, border-color;
  -webkit-transition-duration: 0.25s;
  -moz-transition-duration: 0.25s;
  transition-duration: 0.25s;
  -webkit-transition-timing-function: ease-out;
  -moz-transition-timing-function: ease-out;
  transition-timing-function: ease-out;
}
.claro .dijitTreeRowSelected {
  background-repeat: repeat-x;
  background-color: #cfe5fa;
  background-image: url("images/commonHighlight.png");
  padding: 3px 0 1px;
  margin: 0;
  border: solid 1px #759dc0;
  color: #000000;
}
.claro .dijitTreeRowHover {
  background-color: #abd6ff;
  background-image: url("images/commonHighlight.png");
  padding: 3px 0 1px;
  margin: 0;
  border: solid 1px #759dc0;
  color: #000000;
  -webkit-transition-duration: 0.25s;
  -moz-transition-duration: 0.25s;
  transition-duration: 0.25s;
}
.claro .dijitTreeRowActive {
  background-color: #7dbdfa;
  background-image: url("images/commonHighlight.png");
  background-position: 0 -177px;
  padding: 3px 0 1px;
  margin-left: 0;
  border: solid 1px #759dc0;
  color: #000000;
}
.dj_ie6 .claro .dijitTreeRow {
  background-image: none;
}
.claro .dijitTreeRowFocused {
  background-repeat: repeat;
}
.claro .dijitTreeExpando {
  background-image: url("images/treeExpandImages.png");
  width: 16px;
  height: 16px;
  background-position: -35px 0;
  
}
.dj_ie6 .claro .dijitTreeExpando {
  background-image: url("images/treeExpandImages8bit.png");
}
.claro .dijitTreeRowHover .dijitTreeExpandoOpened {
  background-position: -53px 0;
}
.claro .dijitTreeExpandoClosed {
  background-position: 1px 0;
}
.claro .dijitTreeRowHover .dijitTreeExpandoClosed {
  background-position: -17px 0;
}
.claro .dijitTreeExpandoLeaf, .dj_ie6 .claro .dijitTreeExpandoLeaf {
  background-image: none;
}
.claro .dijitTreeExpandoLoading {
  background-image: url("images/loadingAnimation.gif");
}
.claro .dijitTreeNode .dojoDndItemBefore .dijitTreeContent {
  border-top: 2px solid #759dc0;
}
.claro .dijitTreeNode .dojoDndItemAfter .dijitTreeContent {
  border-bottom: 2px solid #759dc0;
}
