//>>built
define(
//begin v1.x content
{
	"HKD_displayName": "dólar de Hong Kong",
	"CHF_displayName": "franco suizo",
	"CAD_displayName": "dólar canadiense",
	"CNY_displayName": "yuan renminbi chino",
	"AUD_displayName": "dólar australiano",
	"JPY_displayName": "yen japonés",
	"USD_displayName": "dólar estadounidense",
	"GBP_displayName": "libra esterlina británica",
	"EUR_displayName": "euro"
}
//end v1.x content
);