//>>built
define(
//begin v1.x content
({
	'bold': 'Kalın',
	'copy': 'Kopyala',
	'cut': 'Kes',
	'delete': 'Sil',
	'indent': 'Girinti',
	'insertHorizontalRule': 'Yatay Kural',
	'insertOrderedList': 'Numaralı Liste',
	'insertUnorderedList': 'Madde İşaretli Liste',
	'italic': 'İtalik',
	'justifyCenter': 'Ortaya Hizala',
	'justifyFull': 'Yasla',
	'justifyLeft': 'Sola Hizala',
	'justifyRight': 'Sağa Hizala',
	'outdent': 'Çıkıntı',
	'paste': 'Yapıştır',
	'redo': 'Yinele',
	'removeFormat': 'Biçimi Kaldır',
	'selectAll': 'Tümünü Seç',
	'strikethrough': 'Üstü Çizili',
	'subscript': 'Alt Simge',
	'superscript': 'Üst Simge',
	'underline': 'Altı Çizili',
	'undo': 'Geri Al',
	'unlink': 'Bağlantıyı Kaldır',
	'createLink': 'Bağlantı Oluştur',
	'toggleDir': 'Yönü Değiştir',
	'insertImage': 'Resim Ekle',
	'insertTable': 'Tablo Ekle/Düzenle',
	'toggleTableBorder': 'Tablo Kenarlığını Göster/Gizle',
	'deleteTable': 'Tabloyu Sil',
	'tableProp': 'Tablo Özelliği',
	'htmlToggle': 'HTML Kaynağı',
	'foreColor': 'Ön Plan Rengi',
	'hiliteColor': 'Arka Plan Rengi',
	'plainFormatBlock': 'Paragraf Stili',
	'formatBlock': 'Paragraf Stili',
	'fontSize': 'Yazı Tipi Boyutu',
	'fontName': 'Yazı Tipi Adı',
	'tabIndent': 'Sekme Girintisi',
	"fullScreen": "Tam Ekranı Aç/Kapat",
	"viewSource": "HTML Kaynağını Görüntüle",
	"print": "Yazdır",
	"newPage": "Yeni Sayfa",
	/* Error messages */
	'systemShortcut': '"${0}" işlemi yalnızca tarayıcınızda bir klavye kısayoluyla birlikte kullanılabilir. Şunu kullanın: ${1}.'
})

//end v1.x content
);
