
.claro .dijitSliderBar {
  border-style: solid;
  outline: 1px;
}
.claro .dijitSliderFocused .dijitSliderBar {
  border-color: #759dc0;
}
.claro .dijitSliderHover .dijitSliderBar {
  border-color: #759dc0;
}
.claro .dijitSliderDisabled .dijitSliderBar {
  background-image: none;
  border-color: #d3d3d3;
}
.claro .dijitRuleLabel {
  color: #000000;
}
.claro .dijitRuleLabelsContainerH {
  padding: 2px 0;
}
.claro .dijitSlider .dijitSliderProgressBarH, .claro .dijitSlider .dijitSliderLeftBumper {
  background-image: url("../form/images/sliderHorizontal.png");
  background-repeat: repeat-x;
  background-position: 0 -20px;
  border-color: #b5bcc7;
  background-color: #cfe5fa;
}
.claro .dijitSlider .dijitSliderRemainingBarH, .claro .dijitSlider .dijitSliderRightBumper {
  background-image: url("../form/images/sliderHorizontal.png");
  background-repeat: repeat-x;
  background-position: 0 -11px;
  border-color: #b5bcc7;
  background-color: #ffffff;
}
.claro .dijitSliderRightBumper {
  border-right: solid 1px #b5bcc7;
}
.claro .dijitSliderLeftBumper {
  border-left: solid 1px #b5bcc7;
}
.claro .dijitSliderHover .dijitSliderProgressBarH, .claro .dijitSliderHover .dijitSliderLeftBumper {
  background-position: 0 -20px;
  background-color: #abd6ff;
  border-color: #759dc0;
}
.claro .dijitSliderHover .dijitSliderRemainingBarH, .claro .dijitSliderHover .dijitSliderRightBumper {
  background-position: 0 0;
  background-color: #ffffff;
  border-color: #759dc0;
}
.claro .dijitSliderFocused .dijitSliderProgressBarH, .claro .dijitSliderFocused .dijitSliderLeftBumper {
  background-position: 0 -30px;
  background-color: #abd6ff;
  border-color: #759dc0;
}
.claro .dijitSliderFocused .dijitSliderRemainingBarH, .claro .dijitSliderFocused .dijitSliderRightBumper {
  background-position: 0 -9px;
  background-color: #ffffff;
  border-color: #759dc0;
}
.claro .dijitSliderDisabled .dijitSliderProgressBarH, .claro .dijitSliderDisabled .dijitSliderLeftBumper {
  background-color: #d3d3d3;
  
  background-image: none;
}
.claro .dijitSliderDisabled .dijitSliderRemainingBarH, .claro .dijitSliderDisabled .dijitSliderRightBumper {
  background-color: #efefef;
}
.claro .dijitRuleLabelsContainerV {
  padding: 0 2px;
}
.claro .dijitSlider .dijitSliderProgressBarV, .claro .dijitSlider .dijitSliderBottomBumper {
  background-image: url("../form/images/sliderVertical.png");
  background-repeat: repeat-y;
  background-position: -36px 0;
  border-color: #b5bcc7;
  background-color: #cfe5fa;
}
.claro .dijitSlider .dijitSliderRemainingBarV, .claro .dijitSlider .dijitSliderTopBumper {
  background-image: url("../form/images/sliderVertical.png");
  background-repeat: repeat-y;
  background-position: -3px 0;
  border-color: #b5bcc7;
  background-color: #ffffff;
}
.claro .dijitSliderBottomBumper {
  border-bottom: solid 1px #b5bcc7;
}
.claro .dijitSliderTopBumper {
  border-top: solid 1px #b5bcc7;
}
.claro .dijitSliderHover .dijitSliderProgressBarV, .claro .dijitSliderHover .dijitSliderBottomBumper {
  background-position: -36px 0;
  background-color: #abd6ff;
}
.claro .dijitSliderHover .dijitSliderRemainingBarV, .claro .dijitSliderHover .dijitSliderTopBumper {
  background-position: 0 0;
  background-color: #ffffff;
}
.claro .dijitSliderFocused .dijitSliderProgressBarV, .claro .dijitSliderFocused .dijitSliderBottomBumper {
  background-position: -56px 0;
  background-color: #abd6ff;
}
.claro .dijitSliderFocused .dijitSliderRemainingBarV, .claro .dijitSliderFocused .dijitSliderTopBumper {
  background-position: -18px 0;
  background-color: #ffffff;
}
.claro .dijitSliderDisabled .dijitSliderProgressBarV, .claro .dijitSliderDisabled .dijitSliderBottomBumper {
  background-color: #d3d3d3;
  
}
.claro .dijitSliderDisabled .dijitSliderRemainingBarV, .claro .dijitSliderDisabled .dijitSliderTopBumper {
  background-color: #efefef;
}
.claro .dijitSliderImageHandleH {
  border: 0;
  width: 18px;
  height: 16px;
  background-image: url("../form/images/sliderThumbs.png");
  background-repeat: no-repeat;
  background-position: 0 0;
}
.claro .dijitSliderHover .dijitSliderImageHandleH {
  background-position: -18px 0;
}
.claro .dijitSliderFocused .dijitSliderImageHandleH {
  background-position: -36px 0;
}
.claro .dijitSliderProgressBarH .dijitSliderThumbHover {
  background-position: -36px 0;
}
.claro .dijitSliderProgressBarH .dijitSliderThumbActive {
  background-position: -36px 0;
}
.claro .dijitSliderReadOnly .dijitSliderImageHandleH, .claro .dijitSliderDisabled .dijitSliderImageHandleH {
  background-position: -54px 0;
}
.claro .dijitSliderImageHandleV {
  border: 0;
  width: 18px;
  height: 16px;
  background-image: url("../form/images/sliderThumbs.png");
  background-repeat: no-repeat;
  background-position: -289px 0;
}
.claro .dijitSliderHover .dijitSliderImageHandleV {
  background-position: -307px 0;
}
.claro .dijitSliderFocused .dijitSliderImageHandleV {
  background-position: -325px 0;
}
.claro .dijitSliderProgressBarV .dijitSliderThumbHover {
  background-position: -325px 0;
}
.claro .dijitSliderProgressBarV .dijitSliderThumbActive {
  background-position: -325px 0;
}
.claro .dijitSliderReadOnly .dijitSliderImageHandleV, .claro .dijitSliderDisabled .dijitSliderImageHandleV {
  background-position: -343px 0;
}
.claro .dijitSliderButtonContainerH {
  padding: 1px 3px 1px 2px;
}
.claro .dijitSliderButtonContainerV {
  padding: 3px 1px 2px 1px;
}
.claro .dijitSliderDecrementIconH,
.claro .dijitSliderIncrementIconH,
.claro .dijitSliderDecrementIconV,
.claro .dijitSliderIncrementIconV {
  background-image: url("../form/images/commonFormArrows.png");
  background-repeat: no-repeat;
  background-color: #efefef;
  -moz-border-radius: 2px;
  border-radius: 2px;
  border: solid 1px #b5bcc7;
  font-size: 1px;
}
.claro .dijitSliderDecrementIconH, .claro .dijitSliderIncrementIconH {
  height: 12px;
  width: 9px;
}
.claro .dijitSliderDecrementIconV, .claro .dijitSliderIncrementIconV {
  height: 9px;
  width: 12px;
}
.claro .dijitSliderActive .dijitSliderDecrementIconH,
.claro .dijitSliderActive .dijitSliderIncrementIconH,
.claro .dijitSliderActive .dijitSliderDecrementIconV,
.claro .dijitSliderActive .dijitSliderIncrementIconV,
.claro .dijitSliderHover .dijitSliderDecrementIconH,
.claro .dijitSliderHover .dijitSliderIncrementIconH,
.claro .dijitSliderHover .dijitSliderDecrementIconV,
.claro .dijitSliderHover .dijitSliderIncrementIconV {
  
  border: solid 1px #759dc0;
  background-color: #ffffff;
}
.claro .dijitSliderDecrementIconH {
  background-position: -357px 50%;
}
.claro .dijitSliderActive .dijitSliderDecrementIconH.claro .dijitSliderHover .dijitSliderDecrementIconH {
  background-position: -393px 50%;
}
.claro .dijitSliderIncrementIconH {
  background-position: -251px 50%;
}
.claro .dijitSliderActive .dijitSliderIncrementIconH.claro .dijitSliderHover .dijitSliderIncrementIconH {
  background-position: -283px 50%;
}
.claro .dijitSliderDecrementIconV {
  background-position: -38px 50%;
}
.claro .dijitSliderActive .dijitSliderDecrementIconV.claro .dijitSliderHover .dijitSliderDecrementIconV {
  background-position: -73px 50%;
}
.claro .dijitSliderIncrementIconV {
  background-position: -143px 49%;
}
.claro .dijitSliderActive .dijitSliderIncrementIconV.claro .dijitSliderHover .dijitSliderIncrementIconV {
  background-position: -178px 49%;
}
.claro .dijitSliderButtonContainerV .dijitSliderDecrementButtonHover,
.claro .dijitSliderButtonContainerH .dijitSliderDecrementButtonHover,
.claro .dijitSliderButtonContainerV .dijitSliderIncrementButtonHover,
.claro .dijitSliderButtonContainerH .dijitSliderIncrementButtonHover {
  background-color: #cfe5fa;
}
.claro .dijitSliderButtonContainerV .dijitSliderDecrementButtonActive,
.claro .dijitSliderButtonContainerH .dijitSliderDecrementButtonActive,
.claro .dijitSliderButtonContainerV .dijitSliderIncrementButtonActive,
.claro .dijitSliderButtonContainerH .dijitSliderIncrementButtonActive {
  background-color: #abd6ff;
  border-color: #759dc0;
}
.claro .dijitSliderButtonInner {
  visibility: hidden;
}
.claro .dijitSliderDisabled .dijitSliderBar {
  border-color: #d3d3d3;
}
.claro .dijitSliderReadOnly *, .claro .dijitSliderDisabled * {
  border-color: #d3d3d3;
  color: #818181;
}
.claro .dijitSliderReadOnly .dijitSliderDecrementIconH, .claro .dijitSliderDisabled .dijitSliderDecrementIconH {
  background-position: -321px 50%;
  background-color: #efefef;
}
.claro .dijitSliderReadOnly .dijitSliderIncrementIconH, .claro .dijitSliderDisabled .dijitSliderIncrementIconH {
  background-position: -215px 50%;
  background-color: #efefef;
}
.claro .dijitSliderReadOnly .dijitSliderDecrementIconV, .claro .dijitSliderDisabled .dijitSliderDecrementIconV {
  background-position: -3px 49%;
  background-color: #efefef;
}
.claro .dijitSliderReadOnly .dijitSliderIncrementIconV, .claro .dijitSliderDisabled .dijitSliderIncrementIconV {
  background-position: -107px 49%;
  background-color: #efefef;
}
.dj_ie6 .claro .dijitSlider .dijitSliderProgressBarH,
.dj_ie6 .claro .dijitSlider .dijitSliderLeftBumper,
.dj_ie6 .claro .dijitSlider .dijitSliderRemainingBarH,
.dj_ie6 .claro .dijitSlider .dijitSliderRightBumper,
.dj_ie6 .claro .dijitSlider .dijitSliderProgressBarV,
.dj_ie6 .claro .dijitSlider .dijitSliderTopBumper,
.dj_ie6 .claro .dijitSlider .dijitSliderRemainingBarV,
.dj_ie6 .claro .dijitSlider .dijitSliderBottomBumper {
  background-image: none;
}
