//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "L.yyyy",
	"dateFormatItem-yQ": "Q/yyyy",
	"months-standAlone-abbr-leap": "adár II",
	"dayPeriods-format-wide-pm": "ip.",
	"dateFormatItem-MMMEd": "E d. MMM",
	"dateFormatItem-hms": "h.mm.ss a",
	"dateFormatItem-yQQQ": "QQQ y",
	"days-standAlone-wide": [
		"sunnuntai",
		"maanantai",
		"tiistai",
		"keskiviikko",
		"torstai",
		"perjantai",
		"lauantai"
	],
	"months-standAlone-narrow": [
		"T",
		"H",
		"K",
		"T",
		"S",
		"A",
		"A",
		"N",
		"I",
		"S",
		"T",
		"A",
		"E"
	],
	"dayPeriods-format-wide-am": "ap.",
	"timeFormat-full": "H.mm.ss zzzz",
	"months-standAlone-narrow-leap": "A",
	"months-standAlone-abbr": [
		"tišrí",
		"hešván",
		"kislév",
		"tevét",
		"ševát",
		"adár I",
		"adár",
		"nisán",
		"ijjár",
		"siván",
		"tammúz",
		"ab",
		"elúl"
	],
	"dateFormatItem-yMMM": "LLL y",
	"days-standAlone-narrow": [
		"S",
		"M",
		"T",
		"K",
		"T",
		"P",
		"L"
	],
	"dateFormat-long": "d. MMMM y",
	"timeFormat-medium": "H.mm.ss",
	"dateFormatItem-Hm": "H.mm",
	"dateFormat-medium": "d.M.yyyy",
	"dateFormatItem-Hms": "H.mm.ss",
	"dateFormatItem-ms": "mm.ss",
	"months-standAlone-wide": [
		"tišríkuu",
		"hešvánkuu",
		"kislévkuu",
		"tevétkuu",
		"ševátkuu",
		"adárkuu I",
		"adárkuu",
		"nisánkuu",
		"ijjárkuu",
		"sivánkuu",
		"tammúzkuu",
		"abkuu",
		"elúlkuu"
	],
	"dateFormatItem-MMMd": "d. MMM",
	"timeFormat-long": "H.mm.ss z",
	"months-format-abbr": [
		"tišríkuuta",
		"hešvánkuuta",
		"kislévkuuta",
		"tevétkuuta",
		"ševátkuuta",
		"adárkuuta I",
		"adárkuuta",
		"nisánkuuta",
		"ijjárkuuta",
		"sivánkuuta",
		"tammúzkuuta",
		"abkuuta",
		"elúlkuuta"
	],
	"timeFormat-short": "H.mm",
	"dateFormatItem-H": "H",
	"quarters-format-abbr": [
		"1. nelj.",
		"2. nelj.",
		"3. nelj.",
		"4. nelj."
	],
	"days-format-abbr": [
		"su",
		"ma",
		"ti",
		"ke",
		"to",
		"pe",
		"la"
	],
	"dateFormatItem-MEd": "E d.M.",
	"months-format-narrow": [
		"T",
		"H",
		"K",
		"T",
		"S",
		"A",
		"A",
		"N",
		"I",
		"S",
		"T",
		"A",
		"E"
	],
	"dateFormatItem-hm": "h.mm a",
	"months-standAlone-wide-leap": "adárkuu II",
	"dayPeriods-format-abbr-pm": "ip.",
	"dateFormat-short": "d.M.yyyy",
	"dateFormatItem-yMMMEd": "EEE d. MMM y",
	"dateFormat-full": "cccc d. MMMM y",
	"dateFormatItem-Md": "d.M.",
	"dateFormatItem-yMEd": "EEE d.M.yyyy",
	"months-format-wide": [
		"tišríkuuta",
		"hešvánkuuta",
		"kislévkuuta",
		"tevétkuuta",
		"ševátkuuta",
		"adárkuuta I",
		"adárkuuta",
		"nisánkuuta",
		"ijjárkuuta",
		"sivánkuuta",
		"tammúzkuuta",
		"abkuuta",
		"elúlkuuta"
	],
	"dayPeriods-format-abbr-am": "ap.",
	"quarters-format-wide": [
		"1. neljännes",
		"2. neljännes",
		"3. neljännes",
		"4. neljännes"
	],
	"months-format-wide-leap": "adárkuuta II",
	"days-format-wide": [
		"sunnuntaina",
		"maanantaina",
		"tiistaina",
		"keskiviikkona",
		"torstaina",
		"perjantaina",
		"lauantaina"
	]
}
//end v1.x content
);