//>>built
define(
//begin v1.x content
({
	iframeEditTitle: 'modifica area',  // primary title for editable IFRAME, for screen readers when focus is in the editing area
	iframeFocusTitle: 'modifica frame area'  // secondary title for editable IFRAME when focus is on outer container
									 //  to let user know that focus has moved out of editing area and to the
									 //  parent element of the editing area
})
//end v1.x content
);
