//>>built
define(
//begin v1.x content
({
	createLinkTitle: "Propiedades del enlace",
	insertImageTitle: "Propiedades de la imagen",
	url: "URL:",
	text: "Descripción:",
	target: "Destino:",
	set: "Establecer",
	currentWindow: "Ventana actual",
	parentWindow: "Ventana padre",
	topWindow: "Ventana superior",
	newWindow: "Nueva ventana"
})

//end v1.x content
);
