//>>built
define(
//begin v1.x content
({
	'bold': 'Полужирный',
	'copy': 'Копировать',
	'cut': 'Вырезать',
	'delete': 'Удалить',
	'indent': 'Отступ',
	'insertHorizontalRule': 'Горизонтальная линейка',
	'insertOrderedList': 'Нумерованный список',
	'insertUnorderedList': 'Список с маркерами',
	'italic': 'Курсив',
	'justifyCenter': 'По центру',
	'justifyFull': 'По ширине',
	'justifyLeft': 'По левому краю',
	'justifyRight': 'По правому краю',
	'outdent': 'Втяжка',
	'paste': 'Вставить',
	'redo': 'Повторить',
	'removeFormat': 'Удалить формат',
	'selectAll': 'Выбрать все',
	'strikethrough': 'Перечеркивание',
	'subscript': 'Нижний индекс',
	'superscript': 'Верхний индекс',
	'underline': 'Подчеркивание',
	'undo': 'Отменить',
	'unlink': 'Удалить ссылку',
	'createLink': 'Создать ссылку',
	'toggleDir': 'Изменить направление',
	'insertImage': 'Вставить изображение',
	'insertTable': 'Вставить/изменить таблицу',
	'toggleTableBorder': 'Переключить рамку таблицы',
	'deleteTable': 'Удалить таблицу',
	'tableProp': 'Свойства таблицы',
	'htmlToggle': 'Код HTML',
	'foreColor': 'Цвет текста',
	'hiliteColor': 'Цвет фона',
	'plainFormatBlock': 'Стиль абзаца',
	'formatBlock': 'Стиль абзаца',
	'fontSize': 'Размер шрифта',
	'fontName': 'Название шрифта',
	'tabIndent': 'Табуляция',
	"fullScreen": "Переключить полноэкранный режим",
	"viewSource": "Показать исходный код HTML",
	"print": "Печать",
	"newPage": "Создать страницу",
	/* Error messages */
	'systemShortcut': 'Действие "${0}" можно выполнить в браузере только путем нажатия клавиш ${1}.'
})

//end v1.x content
);
