//>>built
define(
//begin v1.x content
({
	displayOptions: "[mostra opcions]",
	title: "Títol",
	authors: "Autors",
	contributors: "Col·laboradors",
	id: "ID",
	close: "[tanca]",
	updated: "Actualitzat",
	summary: "Resum",
	content: "Contingut"
})
//end v1.x content
);
