//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yMEd": "EEE, d/M/y",
	"dateFormatItem-yyyyMMMM": "MMMM y",
	"dateFormatItem-MEd": "E, d/M",
	"dateFormat-medium": "dd/MM/yyyy",
	"dateFormatItem-MMdd": "dd/MM",
	"dateFormatItem-yyyyMM": "MM/yyyy",
	"dateFormat-full": "EEEE, d MMMM y",
	"dateFormatItem-MMMMd": "d MMMM",
	"dateFormat-short": "d/MM/yy",
	"dateFormat-long": "d MMMM y",
	"dateFormatItem-MMMEd": "E, d MMM"
}
//end v1.x content
);