//>>built
define(
//begin v1.x content
({
	'bold': 'عري~ض',
	'copy': 'نسخ',
	'cut': 'قص',
	'delete': 'حذف',
	'indent': 'ازاحة للداخل',
	'insertHorizontalRule': 'مسطرة أفقية',
	'insertOrderedList': '‏كشف مرقم‏',
	'insertUnorderedList': 'كشف نقطي',
	'italic': '~مائل',
	'justifyCenter': 'محاذاة في الوسط',
	'justifyFull': 'ضبط',
	'justifyLeft': 'محاذاة الى اليسار',
	'justifyRight': 'محاذاة الى اليمين',
	'outdent': 'ازاحة للخارج',
	'paste': 'لصق',
	'redo': '‏اعادة‏',
	'removeFormat': 'ازالة النسق',
	'selectAll': '‏اختيار كل‏',
	'strikethrough': 'تشطيب',
	'subscript': 'رمز سفلي',
	'superscript': 'رمز علوي',
	'underline': '~تسطير',
	'undo': 'تراجع',
	'unlink': 'ازالة وصلة',
	'createLink': 'تكوين وصلة',
	'toggleDir': 'تبديل الاتجاه',
	'insertImage': 'ادراج صورة',
	'insertTable': 'ادراج/تحرير جدول',
	'toggleTableBorder': 'تبديل حدود الجدول',
	'deleteTable': 'حذف جدول',
	'tableProp': 'خصائص الجدول',
	'htmlToggle': 'مصدر HTML',
	'foreColor': 'لون الواجهة الأمامية',
	'hiliteColor': '‏لون الخلفية‏',
	'plainFormatBlock': 'نمط الفقرة',
	'formatBlock': 'نمط الفقرة',
	'fontSize': 'حجم طاقم الطباعة',
	'fontName': 'اسم طاقم الطباعة',
	'tabIndent': 'ازاحة علامة الجدولة للداخل',
	"fullScreen": "تبديل  الشاشة الكاملة",
	"viewSource": "مشاهدة مصدر HTML",
	"print": "طباعة",
	"newPage": "صفحة جديدة",
	/* Error messages */
	'systemShortcut': 'يكون التصرف "${0}" متاحا فقط ببرنامج الاستعراض الخاص بك باستخدام المسار المختصر للوحة المفاتيح.  استخدم ${1}.',
	'ctrlKey':'ctrl+${0}',
	'appleKey':'\u2318${0}' // "command" or open-apple key on Macintosh
})

//end v1.x content
);
