//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "M.y G",
	"dateFormatItem-yyMMdd": "dd.MM.y G",
	"dateFormatItem-yQ": "Q y G",
	"dayPeriods-format-wide-pm": "nachm.",
	"dateFormatItem-MMMEd": "E, d. MMM",
	"dateFormatItem-yQQQ": "QQQ y G",
	"dateFormatItem-MMdd": "dd.MM.",
	"dateFormatItem-MMM": "LLL",
	"months-standAlone-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dayPeriods-format-wide-am": "vorm.",
	"dateFormatItem-y": "y G",
	"dateFormatItem-yyyy": "y G",
	"months-standAlone-abbr": [
		"Jan",
		"Feb",
		"Mär",
		"Apr",
		"Mai",
		"Jun",
		"Jul",
		"Aug",
		"Sep",
		"Okt",
		"Nov",
		"Dez"
	],
	"dateFormatItem-Ed": "E d.",
	"dateFormatItem-yMMM": "MMM y G",
	"days-standAlone-narrow": [
		"S",
		"M",
		"D",
		"M",
		"D",
		"F",
		"S"
	],
	"dateFormatItem-yyyyMMMM": "MMMM y G",
	"dateFormat-long": "d. MMMM y G",
	"dateFormatItem-Hm": "HH:mm",
	"dateFormatItem-MMd": "d.MM.",
	"dateFormatItem-yyMM": "MM.y G",
	"dateFormat-medium": "d. MMM y G",
	"dateFormatItem-Hms": "HH:mm:ss",
	"dateFormatItem-yyMMM": "MMM y G",
	"dateFormatItem-yyQQQQ": "QQQQ y G",
	"dateFormatItem-ms": "mm:ss",
	"dateFormatItem-MMMd": "d. MMM",
	"dateFormatItem-yyQ": "Q y G",
	"months-format-abbr": [
		"Jan",
		"Feb",
		"Mär",
		"Apr",
		"Mai",
		"Jun",
		"Jul",
		"Aug",
		"Sep",
		"Okt",
		"Nov",
		"Dez"
	],
	"dateFormatItem-H": "HH 'Uhr'",
	"days-format-abbr": [
		"So.",
		"Mo.",
		"Di.",
		"Mi.",
		"Do.",
		"Fr.",
		"Sa."
	],
	"dateFormatItem-MMMMdd": "dd. MMMM",
	"dateFormatItem-M": "L",
	"dateFormatItem-MEd": "E, d.M.",
	"dateFormat-short": "d.M.yyyy",
	"dateFormatItem-yMMMEd": "EEE, d. MMM y G",
	"dateFormat-full": "EEEE d. MMMM y G",
	"dateFormatItem-Md": "d.M.",
	"dateFormatItem-yMEd": "EEE, d.M.y G",
	"months-format-wide": [
		"Januar",
		"Februar",
		"März",
		"April",
		"Mai",
		"Juni",
		"Juli",
		"August",
		"September",
		"Oktober",
		"November",
		"Dezember"
	],
	"dateFormatItem-d": "d",
	"quarters-format-wide": [
		"1. Quartal",
		"2. Quartal",
		"3. Quartal",
		"4. Quartal"
	],
	"days-format-wide": [
		"Sonntag",
		"Montag",
		"Dienstag",
		"Mittwoch",
		"Donnerstag",
		"Freitag",
		"Samstag"
	]
}
//end v1.x content
);