//>>built
define(
//begin v1.x content
({
	displayOptions: "[אפשרויות הצגה]",
	title: "כותרת",
	authors: "מחברים",
	contributors: "תורמים",
	id: "זיהוי",
	close: "[סגירה]",
	updated: "עודכן",
	summary: "סיכום",
	content: "תוכן"
})
//end v1.x content
);
