//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yMMMEd": "EEE, d MMM, y",
	"dateFormatItem-yyMMM": "MMM-yy",
	"dateFormatItem-Md": "M-d",
	"dateFormatItem-yMEd": "EEE, y-M-d",
	"dateFormatItem-MEd": "E, M-d",
	"dateFormatItem-MMMMEd": "E, d MMMM",
	"dateFormat-medium": "yyyy-MM-dd",
	"dateFormatItem-MMMd": "d MMM",
	"dateFormatItem-MMdd": "MM-dd",
	"dateFormat-full": "EEEE, d MMMM, y",
	"dateFormat-short": "yy-MM-dd",
	"dateFormatItem-MMMMd": "d MMMM",
	"dateFormat-long": "d MMMM, y",
	"dateFormatItem-MMMEd": "E, d MMM"
}
//end v1.x content
);