
.dijitRtl .soria .dijitEditorIcon {
	background-image: url('../../icons/images/editorIconsEnabled_rtl.png'); 
}
.dijitRtl .soria .dijitDisabled .dijitEditorIcon {
	background-image: url('../../icons/images/editorIconsDisabled_rtl.png'); 
}
