//>>built
define(
//begin v1.x content
({
	fontSize: "גודל",
	fontName: "גופן",
	formatBlock: "עיצוב",

	serif: "serif",
	"sans-serif": "sans-serif",
	monospace: "monospace",
	cursive: "cursive",
	fantasy: "fantasy",

	noFormat: "ללא ",
	p: "פיסקה",
	h1: "כותרת",
	h2: "תת-כותרת",
	h3: "תת-תת-כותרת",
	pre: "מעוצב מראש",

	1: "קטן ביות",
	2: "קטן מאוד",
	3: "קטן",
	4: "בינוני",
	5: "גדול",
	6: "גדול מאוד",
	7: "גדול ביותר"
})
//end v1.x content
);
