//>>built
define(
//begin v1.x content
{
	"months-format-narrow": [
		"g",
		"f",
		"m",
		"a",
		"m",
		"j",
		"j",
		"a",
		"s",
		"o",
		"n",
		"d"
	],
	"field-weekday": "dia de la setmana",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-yMEd": "E d/M/yyyy",
	"dateFormatItem-MMMEd": "E d MMM",
	"eraNarrow": [
		"aC",
		"dC"
	],
	"dateFormat-long": "d MMMM 'de' y",
	"months-format-wide": [
		"de gener",
		"de febrer",
		"de març",
		"d’abril",
		"de maig",
		"de juny",
		"de juliol",
		"d’agost",
		"de setembre",
		"d’octubre",
		"de novembre",
		"de desembre"
	],
	"dateFormatItem-EEEd": "EEE d",
	"dayPeriods-format-wide-pm": "p.m.",
	"dateFormat-full": "EEEE d MMMM 'de' y",
	"dateFormatItem-Md": "d/M",
	"field-era": "era",
	"dateFormatItem-yM": "M/yyyy",
	"months-standAlone-wide": [
		"gener",
		"febrer",
		"març",
		"abril",
		"maig",
		"juny",
		"juliol",
		"agost",
		"setembre",
		"octubre",
		"novembre",
		"desembre"
	],
	"timeFormat-short": "H:mm",
	"quarters-format-wide": [
		"1r trimestre",
		"2n trimestre",
		"3r trimestre",
		"4t trimestre"
	],
	"timeFormat-long": "H:mm:ss z",
	"field-year": "any",
	"dateFormatItem-yMMM": "LLL y",
	"dateFormatItem-yQ": "Q yyyy",
	"field-hour": "hora",
	"months-format-abbr": [
		"de gen.",
		"de febr.",
		"de març",
		"d’abr.",
		"de maig",
		"de juny",
		"de jul.",
		"d’ag.",
		"de set.",
		"d’oct.",
		"de nov.",
		"de des."
	],
	"dateFormatItem-yyQ": "Q yy",
	"timeFormat-full": "H:mm:ss zzzz",
	"field-day-relative+0": "avui",
	"field-day-relative+1": "demà",
	"field-day-relative+2": "demà passat",
	"dateFormatItem-H": "H",
	"field-day-relative+3": "d'aquí a tres dies",
	"months-standAlone-abbr": [
		"gen.",
		"febr.",
		"març",
		"abr.",
		"maig",
		"juny",
		"jul.",
		"ag.",
		"set.",
		"oct.",
		"nov.",
		"des."
	],
	"quarters-format-abbr": [
		"1T",
		"2T",
		"3T",
		"4T"
	],
	"quarters-standAlone-wide": [
		"1r trimestre",
		"2n trimestre",
		"3r trimestre",
		"4t trimestre"
	],
	"dateFormatItem-M": "L",
	"days-standAlone-wide": [
		"diumenge",
		"dilluns",
		"dimarts",
		"dimecres",
		"dijous",
		"divendres",
		"dissabte"
	],
	"dateFormatItem-MMMMd": "d MMMM",
	"timeFormat-medium": "H:mm:ss",
	"dateFormatItem-Hm": "H:mm",
	"quarters-standAlone-abbr": [
		"1T",
		"2T",
		"3T",
		"4T"
	],
	"eraAbbr": [
		"aC",
		"dC"
	],
	"field-minute": "minut",
	"field-dayperiod": "a.m./p.m.",
	"days-standAlone-abbr": [
		"dg",
		"dl",
		"dt",
		"dc",
		"dj",
		"dv",
		"ds"
	],
	"dateFormatItem-d": "d",
	"dateFormatItem-ms": "mm:ss",
	"field-day-relative+-1": "ahir",
	"field-day-relative+-2": "abans d'ahir",
	"field-day-relative+-3": "fa tres dies",
	"dateFormatItem-MMMd": "d MMM",
	"dateFormatItem-MEd": "E d/M",
	"dateFormatItem-yMMMM": "LLLL 'del' y",
	"field-day": "dia",
	"days-format-wide": [
		"diumenge",
		"dilluns",
		"dimarts",
		"dimecres",
		"dijous",
		"divendres",
		"dissabte"
	],
	"field-zone": "zona",
	"dateFormatItem-yyyyMM": "MM/yyyy",
	"dateFormatItem-y": "y",
	"months-standAlone-narrow": [
		"g",
		"f",
		"m",
		"a",
		"m",
		"j",
		"j",
		"a",
		"s",
		"o",
		"n",
		"d"
	],
	"dateFormatItem-hm": "h:mm a",
	"days-format-abbr": [
		"dg.",
		"dl.",
		"dt.",
		"dc.",
		"dj.",
		"dv.",
		"ds."
	],
	"eraNames": [
		"aC",
		"dC"
	],
	"days-format-narrow": [
		"g",
		"l",
		"t",
		"c",
		"j",
		"v",
		"s"
	],
	"field-month": "mes",
	"days-standAlone-narrow": [
		"g",
		"l",
		"t",
		"c",
		"j",
		"v",
		"s"
	],
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "a.m.",
	"dateFormatItem-MMMMEd": "E d MMMM",
	"dateFormat-short": "dd/MM/yy",
	"field-second": "segon",
	"dateFormatItem-yMMMEd": "EEE d MMM y",
	"field-week": "setmana",
	"dateFormat-medium": "dd/MM/yyyy",
	"dateFormatItem-mmss": "mm:ss",
	"dateFormatItem-Hms": "H:mm:ss",
	"dateFormatItem-hms": "h:mm:ss a"
}
//end v1.x content
);