
.dijitIconSave,
.dijitIconPrint,
.dijitIconCut,
.dijitIconCopy,
.dijitIconClear,
.dijitIconDelete,
.dijitIconUndo,
.dijitIconEdit,
.dijitIconNewTask,
.dijitIconEditTask,
.dijitIconEditProperty,
.dijitIconTask,
.dijitIconFilter,
.dijitIconConfigure,
.dijitIconSearch,
.dijitIconApplication,
.dijitIconBookmark,
.dijitIconChart,
.dijitIconConnector,
.dijitIconDatabase,
.dijitIconDocuments,
.dijitIconMail,
.dijitLeaf,
.dijitIconFile,
.dijitIconFunction,
.dijitIconKey,
.dijitIconPackage,
.dijitIconSample,
.dijitIconTable,
.dijitIconUsers,
.dijitFolderClosed,
.dijitIconFolderClosed,
.dijitFolderOpened,
.dijitIconFolderOpen,
.dijitIconError {
	background-image: url('images/commonIconsObjActEnabled.png'); 
	width: 16px;
	height: 16px;
}
.dj_ie6 .dijitIconSave,
.dj_ie6 .dijitIconPrint,
.dj_ie6 .dijitIconCut,
.dj_ie6 .dijitIconCopy,
.dj_ie6 .dijitIconClear,
.dj_ie6 .dijitIconDelete,
.dj_ie6 .dijitIconUndo,
.dj_ie6 .dijitIconEdit,
.dj_ie6 .dijitIconNewTask,
.dj_ie6 .dijitIconEditTask,
.dj_ie6 .dijitIconEditProperty,
.dj_ie6 .dijitIconTask,
.dj_ie6 .dijitIconFilter,
.dj_ie6 .dijitIconConfigure,
.dj_ie6 .dijitIconSearch,
.dj_ie6 .dijitIconApplication,
.dj_ie6 .dijitIconBookmark,
.dj_ie6 .dijitIconChart,
.dj_ie6 .dijitIconConnector,
.dj_ie6 .dijitIconDatabase,
.dj_ie6 .dijitIconDocuments,
.dj_ie6 .dijitIconMail,
.dj_ie6 .dijitLeaf,
.dj_ie6 .dijitIconFile,
.dj_ie6 .dijitIconFunction,
.dj_ie6 .dijitIconKey,
.dj_ie6 .dijitIconPackage,
.dj_ie6 .dijitIconSample,
.dj_ie6 .dijitIconTable,
.dj_ie6 .dijitIconUsers,
.dj_ie6 .dijitFolderClosed,
.dj_ie6 .dijitIconFolderClosed,
.dj_ie6 .dijitFolderOpened,
.dj_ie6 .dijitIconFolderOpen,
.dj_ie6 .dijitIconError {
	background-image: url('images/commonIconsObjActEnabled8bit.png');
}
.dijitDisabled .dijitIconSave,
.dijitDisabled .dijitIconPrint,
.dijitDisabled .dijitIconCut,
.dijitDisabled .dijitIconCopy,
.dijitDisabled .dijitIconClear,
.dijitDisabled .dijitIconDelete,
.dijitDisabled .dijitIconUndo,
.dijitDisabled .dijitIconEdit,
.dijitDisabled .dijitIconNewTask,
.dijitDisabled .dijitIconEditTask,
.dijitDisabled .dijitIconEditProperty,
.dijitDisabled .dijitIconTask,
.dijitDisabled .dijitIconFilter,
.dijitDisabled .dijitIconConfigure,
.dijitDisabled .dijitIconSearch,
.dijitDisabled .dijitIconApplication,
.dijitDisabled .dijitIconBookmark,
.dijitDisabled .dijitIconChart,
.dijitDisabled .dijitIconConnector,
.dijitDisabled .dijitIconDatabase,
.dijitDisabled .dijitIconDocuments,
.dijitDisabled .dijitIconMail,
.dijitDisabled .dijitLeaf,
.dijitDisabled .dijitIconFile,
.dijitDisabled .dijitIconFunction,
.dijitDisabled .dijitIconKey,
.dijitDisabled .dijitIconPackage,
.dijitDisabled .dijitIconSample,
.dijitDisabled .dijitIconTable,
.dijitDisabled .dijitIconUsers,
.dijitDisabled .dijitFolderClosed,
.dijitDisabled .dijitIconFolderClosed,
.dijitDisabled .dijitFolderOpened,
.dijitDisabled .dijitIconFolderOpen,
.dijitDisabled .dijitIconError {
	background-image: url('images/commonIconsObjActDisabled.png'); 
}
.dijitIconSave { background-position: 0; }
.dijitIconPrint { background-position: -16px; }
.dijitIconCut { background-position: -32px; }
.dijitIconCopy { background-position: -48px; }
.dijitIconClear { background-position: -64px; }
.dijitIconDelete { background-position: -80px; }
.dijitIconUndo { background-position: -96px; }
.dijitIconEdit { background-position: -112px; }
.dijitIconNewTask { background-position: -128px; }
.dijitIconEditTask { background-position: -144px; }
.dijitIconEditProperty { background-position: -166px; }
.dijitIconTask { background-position: -176px; }
.dijitIconFilter { background-position: -192px; }
.dijitIconConfigure { background-position: -208px; }
.dijitIconSearch { background-position: -224px; }
.dijitIconError { background-position: -496px; } 
.dijitIconApplication { background-position: -240px; }
.dijitIconBookmark { background-position: -256px; }
.dijitIconChart { background-position: -272px; }
.dijitIconConnector { background-position: -288px; }
.dijitIconDatabase { background-position: -304px; }
.dijitIconDocuments { background-position: -320px; }
.dijitIconMail { background-position: -336px; }
.dijitIconFile, .dijitLeaf { background-position: -352px; }
.dijitIconFunction { background-position: -368px; }
.dijitIconKey { background-position: -384px; }
.dijitIconPackage{ background-position: -400px; }
.dijitIconSample { background-position: -416px; }
.dijitIconTable { background-position: -432px; }
.dijitIconUsers { background-position: -448px; }
.dijitIconFolderClosed, .dijitFolderClosed { background-position: -464px; }
.dijitIconFolderOpen, .dijitFolderOpened { background-position: -480px; }
.dijitIconLoading {
	background:url('images/loadingAnimation_rtl.gif') no-repeat;
	height: 20px;
	width: 20px;
}
