//>>built
define(
//begin v1.x content
{
	"dateFormat-medium": "dd/MM/yyyy",
	"dateFormatItem-MMMEd": "E, d בMMM",
	"dateFormatItem-yMEd": "EEE, d.M.yyyy",
	"eraNarrow": [
		"לבה״ע"
	],
	"dateFormatItem-Md": "d/M",
	"months-standAlone-wide": [
		"תשרי",
		"חשון",
		"כסלו",
		"טבת",
		"שבט",
		"אדר א׳",
		"אדר",
		"ניסן",
		"אייר",
		"סיון",
		"תמוז",
		"אב",
		"אלול"
	],
	"months-format-wide-leap": "אדר ב׳",
	"dateFormatItem-EEEd": "EEE ה-d",
	"eraNames": [
		"לבה״ע"
	],
	"days-standAlone-narrow": [
		"א",
		"ב",
		"ג",
		"ד",
		"ה",
		"ו",
		"ש"
	],
	"dateFormatItem-MMMMEd": "E, d בMMMM",
	"dayPeriods-format-wide-pm": "אחה״צ",
	"months-standAlone-abbr": [
		"תשרי",
		"חשון",
		"כסלו",
		"טבת",
		"שבט",
		"אדר א׳",
		"אדר",
		"ניסן",
		"אייר",
		"סיון",
		"תמוז",
		"אב",
		"אלול"
	],
	"dayPeriods-format-wide-am": "לפנה״צ",
	"dateFormat-long": "d בMMMM y",
	"dateFormat-short": "dd/MM/yy",
	"dateFormatItem-yMMMEd": "EEE, d בMMM y",
	"months-format-wide": [
		"תשרי",
		"חשון",
		"כסלו",
		"טבת",
		"שבט",
		"אדר א׳",
		"אדר",
		"ניסן",
		"אייר",
		"סיון",
		"תמוז",
		"אב",
		"אלול"
	],
	"dateFormatItem-yM": "M.yyyy",
	"months-format-abbr": [
		"תשרי",
		"חשון",
		"כסלו",
		"טבת",
		"שבט",
		"אדר א׳",
		"אדר",
		"ניסן",
		"אייר",
		"סיון",
		"תמוז",
		"אב",
		"אלול"
	],
	"eraAbbr": [
		"לבה״ע"
	],
	"days-format-wide": [
		"יום ראשון",
		"יום שני",
		"יום שלישי",
		"יום רביעי",
		"יום חמישי",
		"יום שישי",
		"יום שבת"
	],
	"dateFormatItem-yQ": "yyyy Q",
	"dateFormatItem-yMMM": "MMM y",
	"quarters-format-wide": [
		"רבעון 1",
		"רבעון 2",
		"רבעון 3",
		"רבעון 4"
	],
	"dateFormat-full": "EEEE, d בMMMM y",
	"dateFormatItem-MMMd": "d בMMM",
	"days-format-abbr": [
		"יום א׳",
		"יום ב׳",
		"יום ג׳",
		"יום ד׳",
		"יום ה׳",
		"יום ו׳",
		"שבת"
	]
}
//end v1.x content
);