//>>built
define(
//begin v1.x content
{
	"quarters-standAlone-wide": [
		"第1季",
		"第2季",
		"第3季",
		"第4季"
	],
	"quarters-format-abbr": [
		"第1季",
		"第2季",
		"第3季",
		"第4季"
	],
	"dateFormat-medium": "yyyy/M/d",
	"field-second": "秒",
	"quarters-standAlone-abbr": [
		"第1季",
		"第2季",
		"第3季",
		"第4季"
	],
	"dateFormatItem-MMdd": "MM/dd",
	"dateFormatItem-MEd": "M/d（E）",
	"dateFormatItem-yMEd": "yyyy/M/d（EEE）",
	"field-week": "週",
	"dateFormatItem-H": "H時",
	"eraNarrow": [
		"西元前",
		"西元"
	],
	"field-day-relative+-3": "大前天",
	"timeFormat-full": "zzzzah時mm分ss秒",
	"dateFormatItem-Md": "M/d",
	"months-standAlone-narrow": [
		"1",
		"2",
		"3",
		"4",
		"5",
		"6",
		"7",
		"8",
		"9",
		"10",
		"11",
		"12"
	],
	"eraNames": [
		"西元前",
		"西元"
	],
	"field-minute": "分鐘",
	"field-hour": "小時",
	"field-day-relative+2": "後天",
	"field-day-relative+3": "大後天",
	"dateFormat-short": "yy/M/d",
	"dateFormatItem-yMMMEd": "y年M月d日EEE",
	"field-era": "年代",
	"dateFormatItem-yM": "yyyy/M",
	"timeFormat-long": "zah時mm分ss秒",
	"eraAbbr": [
		"西元前",
		"西元"
	],
	"dateFormatItem-h": "ah時",
	"dateFormatItem-yMMM": "y年M月",
	"quarters-format-wide": [
		"第1季",
		"第2季",
		"第3季",
		"第4季"
	],
	"field-weekday": "週天",
	"field-zone": "區域",
	"dateFormatItem-Ed": "d日(E)"
}
//end v1.x content
);