//>>built
define(
//begin v1.x content
({
	displayOptions: "[显示选项]",
	title: "标题",
	authors: "作者",
	contributors: "内容添加者",
	id: "标识",
	close: "[关闭]",
	updated: "更新时间",
	summary: "摘要",
	content: "内容"
})
//end v1.x content
);
