
.claro .dijitComboButtonRtl .dijitStretch {
  -moz-border-radius: 0 4px 4px 0;
  border-radius: 0 4px 4px 0;
}
.claro .dijitComboButtonRtl .dijitArrowButton {
  -moz-border-radius: 4px 0 0 4px;
  border-radius: 4px 0 0 4px;
  padding: 3px 0 4px;
  border-left-width: 1px;
  border-right-width: 0;
}
