//>>built
define(
//begin v1.x content
({
	createLinkTitle: "링크 등록 정보",
	insertImageTitle: "이미지 등록 정보",
	url: "URL:",
	text: "설명:",
	target: "대상",
	set: "설정",
	currentWindow: "현재 창",
	parentWindow: "상위 창",
	topWindow: "최상위 창",
	newWindow: "새 창"
})

//end v1.x content
);
