//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "M‏/yyyy",
	"field-dayperiod": "ص/م",
	"dateFormatItem-yQ": "yyyy Q",
	"dayPeriods-format-wide-pm": "م",
	"field-minute": "الدقائق",
	"eraNames": [
		"قبل الميلاد",
		"ميلادي"
	],
	"dateFormatItem-MMMEd": "E d MMM",
	"field-day-relative+-1": "أمس",
	"field-weekday": "اليوم",
	"dateFormatItem-yQQQ": "y QQQ",
	"dateFormatItem-MMdd": "dd‏/MM",
	"days-standAlone-wide": [
		"الأحد",
		"الإثنين",
		"الثلاثاء",
		"الأربعاء",
		"الخميس",
		"الجمعة",
		"السبت"
	],
	"dateFormatItem-MMM": "LLL",
	"months-standAlone-narrow": [
		"ي",
		"ف",
		"م",
		"أ",
		"و",
		"ن",
		"ل",
		"غ",
		"س",
		"ك",
		"ب",
		"د"
	],
	"field-era": "العصر",
	"field-hour": "الساعات",
	"dayPeriods-format-wide-am": "ص",
	"quarters-standAlone-abbr": [
		"الربع الأول",
		"الربع الثاني",
		"الربع الثالث",
		"الربع الرابع"
	],
	"dateFormatItem-y": "y",
	"timeFormat-full": "zzzz h:mm:ss a",
	"months-standAlone-abbr": [
		"يناير",
		"فبراير",
		"مارس",
		"أبريل",
		"مايو",
		"يونيو",
		"يوليو",
		"أغسطس",
		"سبتمبر",
		"أكتوبر",
		"نوفمبر",
		"ديسمبر"
	],
	"dateFormatItem-Ed": "E، d",
	"dateFormatItem-yMMM": "MMM y",
	"field-day-relative+0": "اليوم",
	"field-day-relative+1": "غدًا",
	"days-standAlone-narrow": [
		"ح",
		"ن",
		"ث",
		"ر",
		"خ",
		"ج",
		"س"
	],
	"eraAbbr": [
		"ق.م",
		"م"
	],
	"field-day-relative+2": "بعد الغد",
	"dateFormatItem-yyyyMM": "MM‏/yyyy",
	"dateFormatItem-yyyyMMMM": "MMMM، y",
	"dateFormat-long": "d MMMM، y",
	"timeFormat-medium": "h:mm:ss a",
	"field-zone": "التوقيت",
	"dateFormatItem-Hm": "HH:mm",
	"dateFormat-medium": "dd‏/MM‏/yyyy",
	"quarters-standAlone-wide": [
		"الربع الأول",
		"الربع الثاني",
		"الربع الثالث",
		"الربع الرابع"
	],
	"dateFormatItem-yMMMM": "MMMM y",
	"dateFormatItem-ms": "mm:ss",
	"field-year": "السنة",
	"quarters-standAlone-narrow": [
		"١",
		"٢",
		"٣",
		"٤"
	],
	"field-week": "الأسبوع",
	"months-standAlone-wide": [
		"يناير",
		"فبراير",
		"مارس",
		"أبريل",
		"مايو",
		"يونيو",
		"يوليو",
		"أغسطس",
		"سبتمبر",
		"أكتوبر",
		"نوفمبر",
		"ديسمبر"
	],
	"dateFormatItem-MMMMEd": "E d MMMM",
	"dateFormatItem-MMMd": "d MMM",
	"quarters-format-narrow": [
		"١",
		"٢",
		"٣",
		"٤"
	],
	"dateFormatItem-yyQ": "Q yy",
	"timeFormat-long": "z h:mm:ss a",
	"months-format-abbr": [
		"يناير",
		"فبراير",
		"مارس",
		"أبريل",
		"مايو",
		"يونيو",
		"يوليو",
		"أغسطس",
		"سبتمبر",
		"أكتوبر",
		"نوفمبر",
		"ديسمبر"
	],
	"timeFormat-short": "h:mm a",
	"field-month": "الشهر",
	"dateFormatItem-MMMMd": "d MMMM",
	"quarters-format-abbr": [
		"الربع الأول",
		"الربع الثاني",
		"الربع الثالث",
		"الربع الرابع"
	],
	"days-format-abbr": [
		"أحد",
		"إثنين",
		"ثلاثاء",
		"أربعاء",
		"خميس",
		"جمعة",
		"سبت"
	],
	"dateFormatItem-M": "L",
	"days-format-narrow": [
		"ح",
		"ن",
		"ث",
		"ر",
		"خ",
		"ج",
		"س"
	],
	"field-second": "الثواني",
	"field-day": "يوم",
	"months-format-narrow": [
		"ي",
		"ف",
		"م",
		"أ",
		"و",
		"ن",
		"ل",
		"غ",
		"س",
		"ك",
		"ب",
		"د"
	],
	"days-standAlone-abbr": [
		"أحد",
		"إثنين",
		"ثلاثاء",
		"أربعاء",
		"خميس",
		"جمعة",
		"سبت"
	],
	"dateFormat-short": "d‏/M‏/yyyy",
	"dateFormatItem-yMMMEd": "EEE، d MMMM y",
	"dateFormat-full": "EEEE، d MMMM، y",
	"dateFormatItem-Md": "d/‏M",
	"dateFormatItem-yMEd": "EEE، d/‏M/‏yyyy",
	"months-format-wide": [
		"يناير",
		"فبراير",
		"مارس",
		"أبريل",
		"مايو",
		"يونيو",
		"يوليو",
		"أغسطس",
		"سبتمبر",
		"أكتوبر",
		"نوفمبر",
		"ديسمبر"
	],
	"dateFormatItem-d": "d",
	"quarters-format-wide": [
		"الربع الأول",
		"الربع الثاني",
		"الربع الثالث",
		"الربع الرابع"
	],
	"days-format-wide": [
		"الأحد",
		"الإثنين",
		"الثلاثاء",
		"الأربعاء",
		"الخميس",
		"الجمعة",
		"السبت"
	],
	"eraNarrow": [
		"ق.م",
		"م"
	]
}
//end v1.x content
);