<div class="dijitReset dijitInline dojoxCalc"
><table class="dijitReset dijitInline dojoxCalcLayout" data-dojo-attach-point="calcTable" rules="none" cellspacing=0 cellpadding=0 border=0>
	<tr
		><td colspan="4" class="dojoxCalcInputContainer"
			><input data-dojo-type="dijit.form.TextBox" data-dojo-attach-event="onBlur:onBlur,onKeyPress:onKeyPress" data-dojo-attach-point='textboxWidget'
		/></td
	></tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="seven" label="7" value='7' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="eight" label="8" value='8' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="nine" label="9" value='9' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="divide" label="/" value='/' data-dojo-attach-event='onClick:insertOperator' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="four" label="4" value='4' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="five" label="5" value='5' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="six" label="6" value='6' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="multiply" label="*" value='*' data-dojo-attach-event='onClick:insertOperator' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="one" label="1" value='1' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="two" label="2" value='2' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="three" label="3" value='3' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="add" label="+" value='+' data-dojo-attach-event='onClick:insertOperator' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="decimal" label="." value='.' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="zero" label="0" value='0' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="equals" label="x=y" value='=' data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcMinusButtonContainer">
			<span data-dojo-type="dijit.form.ComboButton" data-dojo-attach-point="subtract" label='-' value='-' data-dojo-attach-event='onClick:insertOperator'>

				<div data-dojo-type="dijit.Menu" style="display:none;">
					<div data-dojo-type="dijit.MenuItem" data-dojo-attach-event="onClick:insertMinus">
						(-)
					</div>
				</div>
			</span>
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="clear" label="Clear" data-dojo-attach-event='onClick:clearText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="sqrt" label="&#x221A;" value="&#x221A;" data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="power" label="^" value="^" data-dojo-attach-event='onClick:insertOperator' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="comma" label="," value=',' data-dojo-attach-event='onClick:insertText' />
		</td>
	</tr>
	<tr>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="AnsButton" label="Ans" value="Ans" data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="LeftParenButton" label="(" value="(" data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="RightParenButton" label=")" value=")" data-dojo-attach-event='onClick:insertText' />
		</td>
		<td class="dojoxCalcButtonContainer">
			<button data-dojo-type="dijit.form.Button" data-dojo-attach-point="enter" label="Enter" data-dojo-attach-event='onClick:parseTextbox' />
		</td>
	</tr>
</table>
<span data-dojo-attach-point="executor" data-dojo-type="dojox.calc._Executor" data-dojo-attach-event="onLoad:executorLoaded"></span>
</div>
