//>>built
define(
//begin v1.x content
{
	"months-format-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"field-weekday": "Dag van de week",
	"dateFormatItem-yyQQQQ": "QQQQ yy",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-yMEd": "EEE d-M-y",
	"dateFormatItem-MMMEd": "E d MMM",
	"eraNarrow": [
		"v. Chr.",
		"n. Chr."
	],
	"dateFormat-long": "d MMMM y",
	"months-format-wide": [
		"januari",
		"februari",
		"maart",
		"april",
		"mei",
		"juni",
		"juli",
		"augustus",
		"september",
		"oktober",
		"november",
		"december"
	],
	"dayPeriods-format-wide-pm": "PM",
	"dateFormat-full": "EEEE d MMMM y",
	"dateFormatItem-Md": "d-M",
	"field-era": "Tijdperk",
	"dateFormatItem-yM": "M-y",
	"months-standAlone-wide": [
		"januari",
		"februari",
		"maart",
		"april",
		"mei",
		"juni",
		"juli",
		"augustus",
		"september",
		"oktober",
		"november",
		"december"
	],
	"timeFormat-short": "HH:mm",
	"quarters-format-wide": [
		"1e kwartaal",
		"2e kwartaal",
		"3e kwartaal",
		"4e kwartaal"
	],
	"timeFormat-long": "HH:mm:ss z",
	"field-year": "Jaar",
	"dateFormatItem-yMMM": "MMM y",
	"dateFormatItem-yQ": "Q yyyy",
	"dateFormatItem-yyyyMMMM": "MMMM y",
	"field-hour": "Uur",
	"dateFormatItem-MMdd": "dd-MM",
	"months-format-abbr": [
		"jan.",
		"feb.",
		"mrt.",
		"apr.",
		"mei",
		"jun.",
		"jul.",
		"aug.",
		"sep.",
		"okt.",
		"nov.",
		"dec."
	],
	"dateFormatItem-yyQ": "Q yy",
	"timeFormat-full": "HH:mm:ss zzzz",
	"field-day-relative+0": "vandaag",
	"field-day-relative+1": "morgen",
	"field-day-relative+2": "overmorgen",
	"field-day-relative+3": "overovermorgen",
	"months-standAlone-abbr": [
		"jan.",
		"feb.",
		"mrt.",
		"apr.",
		"mei",
		"jun.",
		"jul.",
		"aug.",
		"sep.",
		"okt.",
		"nov.",
		"dec."
	],
	"quarters-format-abbr": [
		"K1",
		"K2",
		"K3",
		"K4"
	],
	"quarters-standAlone-wide": [
		"1e kwartaal",
		"2e kwartaal",
		"3e kwartaal",
		"4e kwartaal"
	],
	"dateFormatItem-M": "L",
	"days-standAlone-wide": [
		"zondag",
		"maandag",
		"dinsdag",
		"woensdag",
		"donderdag",
		"vrijdag",
		"zaterdag"
	],
	"dateFormatItem-MMMMd": "d MMMM",
	"dateFormatItem-yyMMM": "MMM yy",
	"timeFormat-medium": "HH:mm:ss",
	"dateFormatItem-Hm": "HH:mm",
	"quarters-standAlone-abbr": [
		"K1",
		"K2",
		"K3",
		"K4"
	],
	"eraAbbr": [
		"v. Chr.",
		"n. Chr."
	],
	"field-minute": "Minuut",
	"field-dayperiod": "AM/PM",
	"days-standAlone-abbr": [
		"zo",
		"ma",
		"di",
		"wo",
		"do",
		"vr",
		"za"
	],
	"dateFormatItem-d": "d",
	"dateFormatItem-ms": "mm:ss",
	"field-day-relative+-1": "gisteren",
	"field-day-relative+-2": "eergisteren",
	"field-day-relative+-3": "eereergisteren",
	"dateFormatItem-MMMd": "d-MMM",
	"dateFormatItem-MEd": "E d-M",
	"field-day": "Dag",
	"days-format-wide": [
		"zondag",
		"maandag",
		"dinsdag",
		"woensdag",
		"donderdag",
		"vrijdag",
		"zaterdag"
	],
	"field-zone": "Zone",
	"dateFormatItem-y": "y",
	"months-standAlone-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dateFormatItem-yyMM": "MM-yy",
	"days-format-abbr": [
		"zo",
		"ma",
		"di",
		"wo",
		"do",
		"vr",
		"za"
	],
	"eraNames": [
		"Voor Christus",
		"na Christus"
	],
	"days-format-narrow": [
		"Z",
		"M",
		"D",
		"W",
		"D",
		"V",
		"Z"
	],
	"field-month": "Maand",
	"days-standAlone-narrow": [
		"Z",
		"M",
		"D",
		"W",
		"D",
		"V",
		"Z"
	],
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "AM",
	"dateFormat-short": "dd-MM-yy",
	"dateFormatItem-MMd": "d-MM",
	"field-second": "Seconde",
	"dateFormatItem-yMMMEd": "EEE d MMM y",
	"dateFormatItem-Ed": "E d",
	"field-week": "Week",
	"dateFormat-medium": "d MMM y"
}
//end v1.x content
);