//>>built
define(
//begin v1.x content
({
	"text" : "Yazı:",
	"insertImageTitle" : "Şəkil başlığı əlavə et",
	"set" : "Yönəlt",
	"newWindow" : "Yeni pəncərə",
	"topWindow" : "Üst pəncərə",
	"target" : "Hədəf:",
	"createLinkTitle" : "Köprü başlığı yarat",
	"parentWindow" : "Ana pəncərə",
	"currentWindow" : "Hazırki pəncərə",
	"url" : "URL:"
})
//end v1.x content
);
