//>>built
define({ root:
//begin v1.x content
({
// local representation of all CSS3 named colors, companion to dojo.colors.  To be used where descriptive information
// is required for each color, such as a palette widget, and not for specifying color programatically.

//Note: due to the SVG 1.0 spec additions, some of these are alternate spellings for the same color e.g. gray vs. gray.
//TODO: should we be using unique rgb values as keys instead and avoid these duplicates, or rely on the caller to do the reverse mapping?
aliceblue: "alice blue",
antiquewhite: "antique white",
aqua: "aqua",
aquamarine: "aquamarine",
azure: "azure",
beige: "beige",
bisque: "bisque",
black: "black",
blanchedalmond: "blanched almond",
blue: "blue",
blueviolet: "blue-violet",
brown: "brown",
burlywood: "burlywood",
cadetblue: "cadet blue",
chartreuse: "chartreuse",
chocolate: "chocolate",
coral: "coral",
cornflowerblue: "cornflower blue",
cornsilk: "cornsilk",
crimson: "crimson",
cyan: "cyan",
darkblue: "dark blue",
darkcyan: "dark cyan",
darkgoldenrod: "dark goldenrod",
darkgray: "dark gray",
darkgreen: "dark green",
darkgrey: "dark gray", // same as darkgray
darkkhaki: "dark khaki",
darkmagenta: "dark magenta",
darkolivegreen: "dark olive green",
darkorange: "dark orange",
darkorchid: "dark orchid",
darkred: "dark red",
darksalmon: "dark salmon",
darkseagreen: "dark sea green",
darkslateblue: "dark slate blue",
darkslategray: "dark slate gray",
darkslategrey: "dark slate gray", // same as darkslategray
darkturquoise: "dark turquoise",
darkviolet: "dark violet",
deeppink: "deep pink",
deepskyblue: "deep sky blue",
dimgray: "dim gray",
dimgrey: "dim gray", // same as dimgray
dodgerblue: "dodger blue",
firebrick: "fire brick",
floralwhite: "floral white",
forestgreen: "forest green",
fuchsia: "fuchsia",
gainsboro: "gainsboro",
ghostwhite: "ghost white",
gold: "gold",
goldenrod: "goldenrod",
gray: "gray",
green: "green",
greenyellow: "green-yellow",
grey: "gray", // same as gray
honeydew: "honeydew",
hotpink: "hot pink",
indianred: "indian red",
indigo: "indigo",
ivory: "ivory",
khaki: "khaki",
lavender: "lavender",
lavenderblush: "lavender blush",
lawngreen: "lawn green",
lemonchiffon: "lemon chiffon",
lightblue: "light blue",
lightcoral: "light coral",
lightcyan: "light cyan",
lightgoldenrodyellow: "light goldenrod yellow",
lightgray: "light gray",
lightgreen: "light green",
lightgrey: "light gray", // same as lightgray
lightpink: "light pink",
lightsalmon: "light salmon",
lightseagreen: "light sea green",
lightskyblue: "light sky blue",
lightslategray: "light slate gray",
lightslategrey: "light slate gray", // same as lightslategray
lightsteelblue: "light steel blue",
lightyellow: "light yellow",
lime: "lime",
limegreen: "lime green",
linen: "linen",
magenta: "magenta",
maroon: "maroon",
mediumaquamarine: "medium aquamarine",
mediumblue: "medium blue",
mediumorchid: "medium orchid",
mediumpurple: "medium purple",
mediumseagreen: "medium sea green",
mediumslateblue: "medium slate blue",
mediumspringgreen: "medium spring green",
mediumturquoise: "medium turquoise",
mediumvioletred: "medium violet-red",
midnightblue: "midnight blue",
mintcream: "mint cream",
mistyrose: "misty rose",
moccasin: "moccasin",
navajowhite: "navajo white",
navy: "navy",
oldlace: "old lace",
olive: "olive",
olivedrab: "olive drab",
orange: "orange",
orangered: "orange red",
orchid: "orchid",
palegoldenrod: "pale goldenrod",
palegreen: "pale green",
paleturquoise: "pale turquoise",
palevioletred: "pale violet-red",
papayawhip: "papaya whip",
peachpuff: "peach puff",
peru: "peru",
pink: "pink",
plum: "plum",
powderblue: "powder blue",
purple: "purple",
red: "red",
rosybrown: "rosy brown",
royalblue: "royal blue",
saddlebrown: "saddle brown",
salmon: "salmon",
sandybrown: "sandy brown",
seagreen: "sea green",
seashell: "seashell",
sienna: "sienna",
silver: "silver",
skyblue: "sky blue",
slateblue: "slate blue",
slategray: "slate gray",
slategrey: "slate gray", // same as slategray
snow: "snow",
springgreen: "spring green",
steelblue: "steel blue",
tan: "tan",
teal: "teal",
thistle: "thistle",
tomato: "tomato",
transparent: "transparent",
turquoise: "turquoise",
violet: "violet",
wheat: "wheat",
white: "white",
whitesmoke: "white smoke",
yellow: "yellow",
yellowgreen: "yellow green"
})
//end v1.x content
,
"zh": true,
"zh-tw": true,
"tr": true,
"th": true,
"sv": true,
"sl": true,
"sk": true,
"ru": true,
"ro": true,
"pt": true,
"pt-pt": true,
"pl": true,
"nl": true,
"nb": true,
"ko": true,
"kk": true,
"ja": true,
"it": true,
"hu": true,
"he": true,
"fr": true,
"fi": true,
"es": true,
"el": true,
"de": true,
"da": true,
"cs": true,
"ca": true,
"ar": true
});
