//>>built
define(
//begin v1.x content
{
	"group": "٬",
	"percentSign": "٪",
	"exponential": "اس",
	"list": "؛",
	"infinity": "∞",
	"minusSign": "-",
	"decimal": "٫",
	"nan": "ليس رقم",
	"perMille": "؉",
	"decimalFormat": "#,##0.###;#,##0.###-",
	"currencyFormat": "¤ #,##0.00;¤ #,##0.00-",
	"plusSign": "+"
}
//end v1.x content
);