
.claro  .dijitColorPalette {
  border: 1px solid #b5bcc7;
  background: #ffffff;
  -moz-border-radius: 0;
  border-radius: 0;
}
.claro .dijitColorPalette .dijitPaletteImg {
  
  border: 1px solid #d3d3d3;
}
.claro .dijitColorPalette .dijitPaletteCell:hover .dijitPaletteImg {
  border: 1px solid #000000;
}
.claro .dijitColorPalette .dijitPaletteCell:active .dijitPaletteImg, .claro .dijitColorPalette .dijitPaletteTable .dijitPaletteCellSelected .dijitPaletteImg {
  border: 2px solid #000000;
}
