//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "M.y",
	"dateFormatItem-yQ": "Q yyyy",
	"dayPeriods-format-wide-pm": "odp.",
	"eraNames": [
		"př.Kr.",
		"po Kr."
	],
	"dateFormatItem-MMMEd": "E, d. MMM",
	"field-day-relative+-1": "Včera",
	"dateFormatItem-yQQQ": "QQQ y",
	"field-day-relative+-2": "Předevčírem",
	"days-standAlone-wide": [
		"neděle",
		"pondělí",
		"úterý",
		"středa",
		"čtvrtek",
		"pátek",
		"sobota"
	],
	"months-standAlone-narrow": [
		"l",
		"ú",
		"b",
		"d",
		"k",
		"č",
		"č",
		"s",
		"z",
		"ř",
		"l",
		"p"
	],
	"dayPeriods-format-wide-am": "dop.",
	"quarters-standAlone-abbr": [
		"1. čtvrtletí",
		"2. čtvrtletí",
		"3. čtvrtletí",
		"4. čtvrtletí"
	],
	"timeFormat-full": "H:mm:ss zzzz",
	"dateFormatItem-yyyy": "y",
	"months-standAlone-abbr": [
		"1.",
		"2.",
		"3.",
		"4.",
		"5.",
		"6.",
		"7.",
		"8.",
		"9.",
		"10.",
		"11.",
		"12."
	],
	"dateFormatItem-yMMM": "LLL y",
	"field-day-relative+0": "Dnes",
	"field-day-relative+1": "Zítra",
	"days-standAlone-narrow": [
		"N",
		"P",
		"Ú",
		"S",
		"Č",
		"P",
		"S"
	],
	"eraAbbr": [
		"př.Kr.",
		"po Kr."
	],
	"field-day-relative+2": "Pozítří",
	"dateFormatItem-yyyyMMMM": "LLLL y",
	"dateFormat-long": "d. MMMM y",
	"timeFormat-medium": "H:mm:ss",
	"dateFormatItem-EEEd": "EEE, d.",
	"dateFormatItem-Hm": "H:mm",
	"dateFormat-medium": "d.M.yyyy",
	"dateFormatItem-Hms": "H:mm:ss",
	"dateFormatItem-yMd": "d.M.y",
	"quarters-standAlone-wide": [
		"1. čtvrtletí",
		"2. čtvrtletí",
		"3. čtvrtletí",
		"4. čtvrtletí"
	],
	"months-standAlone-wide": [
		"leden",
		"únor",
		"březen",
		"duben",
		"květen",
		"červen",
		"červenec",
		"srpen",
		"září",
		"říjen",
		"listopad",
		"prosinec"
	],
	"dateFormatItem-MMMd": "d. MMM",
	"dateFormatItem-yyQ": "Q yy",
	"timeFormat-long": "H:mm:ss z",
	"months-format-abbr": [
		"ledna",
		"února",
		"března",
		"dubna",
		"května",
		"června",
		"července",
		"srpna",
		"září",
		"října",
		"listopadu",
		"prosince"
	],
	"timeFormat-short": "H:mm",
	"dateFormatItem-H": "H",
	"quarters-format-abbr": [
		"1. čtvrtletí",
		"2. čtvrtletí",
		"3. čtvrtletí",
		"4. čtvrtletí"
	],
	"days-format-abbr": [
		"ne",
		"po",
		"út",
		"st",
		"čt",
		"pá",
		"so"
	],
	"days-format-narrow": [
		"N",
		"P",
		"Ú",
		"S",
		"Č",
		"P",
		"S"
	],
	"dateFormatItem-MEd": "E, d.M",
	"months-format-narrow": [
		"l",
		"ú",
		"b",
		"d",
		"k",
		"č",
		"č",
		"s",
		"z",
		"ř",
		"l",
		"p"
	],
	"days-standAlone-abbr": [
		"ne",
		"po",
		"út",
		"st",
		"čt",
		"pá",
		"so"
	],
	"dateFormat-short": "d.M.yy",
	"dateFormatItem-yyyyM": "M.yyyy",
	"dateFormatItem-yMMMEd": "EEE, d. MMM y",
	"dateFormat-full": "EEEE, d. MMMM y",
	"dateFormatItem-Md": "d.M",
	"dateFormatItem-yMEd": "EEE, d.M.y",
	"months-format-wide": [
		"ledna",
		"února",
		"března",
		"dubna",
		"května",
		"června",
		"července",
		"srpna",
		"září",
		"října",
		"listopadu",
		"prosince"
	],
	"dateFormatItem-d": "d.",
	"quarters-format-wide": [
		"1. čtvrtletí",
		"2. čtvrtletí",
		"3. čtvrtletí",
		"4. čtvrtletí"
	],
	"days-format-wide": [
		"neděle",
		"pondělí",
		"úterý",
		"středa",
		"čtvrtek",
		"pátek",
		"sobota"
	],
	"eraNarrow": [
		"př.Kr.",
		"po Kr."
	]
}
//end v1.x content
);