
.dijitReset {
	
	margin:0;
	border:0;
	padding:0;
	line-height:normal;
	font: inherit;
	color: inherit;
}
.dijit_a11y .dijitReset {
	-moz-appearance: none; 
}
.dijitInline {
	
	display:inline-block;			
	#zoom: 1; 
	#display:inline; 
	border:0;
	padding:0;
	vertical-align:middle;
	#vertical-align: auto;	
}
table.dijitInline {
	
	display:inline-table;
	box-sizing: content-box; -moz-box-sizing: content-box;
}
.dijitHidden {
	
	display: none !important;
}
.dijitVisible {
	
	display: block !important;	
	position: relative;			
}
.dijitInputContainer {
	
	#zoom: 1;
	overflow: hidden;
	float: none !important; 
	position: relative;
	vertical-align: middle;
	#display: inline;
}
.dj_ie input.dijitTextBox,
.dj_ie .dijitTextBox input {
	font-size: 100%;
}
.dijitTextBox .dijitSpinnerButtonContainer,
.dijitTextBox .dijitArrowButtonContainer,
.dijitTextBox .dijitValidationContainer {
	float: right;
	text-align: center;
}
.dijitTextBox input.dijitInputField {
	
	padding-left: 0 !important;
	padding-right: 0 !important;
}
.dijitTextBox .dijitValidationContainer {
	display: none;
}
.dijitTeeny {
	font-size:1px;
	line-height:1px;
}
.dijitOffScreen {
	position: absolute;
	left: 50%;
	top: -10000px;
}
.dijitPopup {
	position: absolute;
	background-color: transparent;
	margin: 0;
	border: 0;
	padding: 0;
}
.dijitPositionOnly {
	
	padding: 0 !important;
	border: 0 !important;
	background-color: transparent !important;
	background-image: none !important;
	height: auto !important;
	width: auto !important;
}
.dijitNonPositionOnly {
	
	float: none !important;
	position: static !important;
	margin: 0 0 0 0 !important;
	vertical-align: middle !important;
}
.dijitBackgroundIframe {
	
	position: absolute;
	left: 0;
	top: 0;
	width: 100%;
	height: 100%;
	z-index: -1;
	border: 0;
	padding: 0;
	margin: 0;
}
.dijitDisplayNone {
	
	display:none !important;
}
.dijitContainer {
	
	overflow: hidden;	
}
.dijit_a11y .dijitIcon,
.dijit_a11y div.dijitArrowButtonInner, 
.dijit_a11y span.dijitArrowButtonInner,
.dijit_a11y img.dijitArrowButtonInner,
.dijit_a11y .dijitCalendarIncrementControl,
.dijit_a11y .dijitTreeExpando {
	
	display: none;
}
.dijitSpinner div.dijitArrowButtonInner {
	display: block; 
}
.dijit_a11y .dijitA11ySideArrow {
	display: inline !important; 
	cursor: pointer;
}
.dijit_a11y .dijitCalendarDateLabel {
	padding: 1px;
	border: 0px !important;
}
.dijit_a11y .dijitCalendarSelectedDate .dijitCalendarDateLabel {
	border-style: solid !important;
	border-width: 1px !important;
	padding: 0;
}
.dijit_a11y .dijitCalendarDateTemplate {
	padding-bottom: 0.1em !important;	
	border: 0px !important;
}
.dijit_a11y .dijitButtonNode {
	border: black outset medium !important;
	
	padding: 0 !important;
}
.dijit_a11y .dijitButtonContents{
	margin: 0.15em; 
}
.dijit_a11y .dijitTextBoxReadOnly .dijitInputField,
.dijit_a11y .dijitTextBoxReadOnly .dijitButtonNode {
	border-style: outset!important;
	border-width: medium!important;
	border-color: #999 !important;
	color:#999 !important;
}
.dijitButtonNode * {
	vertical-align: middle;
}
.dijitButtonNode .dijitArrowButtonInner {
	
	background: no-repeat center;
	width: 12px;
	height: 12px;
	direction: ltr; 
}
.dijitLeft {
	
	background-position:left top;
	background-repeat:no-repeat;
}
.dijitStretch {
	
	white-space:nowrap;			
	background-repeat:repeat-x;
}
.dijitRight {
	
	#display:inline;				
	background-position:right top;
	background-repeat:no-repeat;
}
.dj_gecko .dijit_a11y .dijitButtonDisabled .dijitButtonNode {
	opacity: 0.5;
}
.dijitToggleButton,
.dijitButton,
.dijitDropDownButton,
.dijitComboButton {
	
	margin: 0.2em;
	vertical-align: middle;
}
.dijitButtonContents {
	display: block;		
}
td.dijitButtonContents {
	display: table-cell;	
}
.dijitButtonNode img {
	
	vertical-align:middle;
	
}
.dijitToolbar .dijitComboButton {
	
	border-collapse: separate;
}
.dijitToolbar .dijitToggleButton,
.dijitToolbar .dijitButton,
.dijitToolbar .dijitDropDownButton,
.dijitToolbar .dijitComboButton {
	margin: 0;
}
.dijitToolbar .dijitButtonContents {
	
	padding: 1px 2px;
}
.dj_webkit .dijitToolbar .dijitDropDownButton {
	padding-left: 0.3em;
}
.dj_gecko .dijitToolbar .dijitButtonNode::-moz-focus-inner {
	padding:0;
}
.dijitButtonNode {
	
	border:1px solid gray;
	margin:0;
	line-height:normal;
	vertical-align: middle;
	#vertical-align: auto;
	text-align:center;
	white-space: nowrap;
}
.dj_webkit .dijitSpinner .dijitSpinnerButtonContainer {
	
	line-height:inherit;
}
.dijitTextBox .dijitButtonNode {
	border-width: 0;
}
.dijitButtonNode,
.dijitButtonNode * {
	cursor: pointer;
}
.dj_ie .dijitButtonNode {
	
	zoom: 1;
}
.dj_ie .dijitButtonNode button {
	
	overflow: visible;
}
div.dijitArrowButton {
	float: right;
}
.dijitTextBox {
	border: solid black 1px;
	#overflow: hidden; 
	width: 15em;	
	vertical-align: middle;
}
.dijitTextBoxReadOnly,
.dijitTextBoxDisabled {
	color: gray;
}
.dj_webkit .dijitTextBoxDisabled input {
	color: #eee; 
}
.dj_webkit textarea.dijitTextAreaDisabled {
	color: #333; 
}
.dj_gecko .dijitTextBoxReadOnly input.dijitInputField, 
.dj_gecko .dijitTextBoxDisabled input {
	-moz-user-input: none; 
}
.dijitPlaceHolder {
	
	color: #AAAAAA;
	font-style: italic;
	position: absolute;
	top: 0;
	left: 0;
	#filter: ""; 
}
.dijitTimeTextBox {
	width: 8em;
}
.dijitTextBox input:focus {
	outline: none;	
}
.dijitTextBoxFocused {
	outline: 5px -webkit-focus-ring-color;
}
.dijitTextBox input {
	float: left; 
}
.dijitInputInner {
	
	border:0 !important;
	vertical-align:middle !important;
	background-color:transparent !important;
	width:100% !important;
	
	padding-left: 0 !important;
	padding-right: 0 !important;
	margin-left: 0 !important;
	margin-right: 0 !important;
}
.dijit_a11y .dijitTextBox input {
	margin: 0 !important;
}
.dijitTextBoxError input.dijitValidationInner,
.dijitTextBox input.dijitArrowButtonInner {
	
	text-indent: -1em !important;
	direction: ltr !important;
	text-align: left !important;
	height: auto !important;
	#text-indent: 0 !important;
	#letter-spacing: -5em !important;
	#text-align: right !important;
}
.dj_ie .dijitTextBox input,
.dj_ie input.dijitTextBox {
	overflow-y: visible; 
	line-height: normal; 
}
.dj_ie7 .dijitTextBox input.dijitValidationInner,
.dj_ie7 .dijitTextBox input.dijitArrowButtonInner {
	line-height: 86%; 
}
.dj_ie6 .dijitTextBox input,
.dj_ie6 input.dijitTextBox,
.dj_iequirks .dijitTextBox input.dijitValidationInner,
.dj_iequirks .dijitTextBox input.dijitArrowButtonInner,
.dj_iequirks .dijitTextBox input.dijitSpinnerButtonInner,
.dj_iequirks .dijitTextBox input.dijitInputInner,
.dj_iequirks input.dijitTextBox {
	line-height: 100%; 
}
.dijit_a11y input.dijitValidationInner,
.dijit_a11y input.dijitArrowButtonInner {
	
	text-indent: 0 !important;
	width: 1em !important;
	#text-align: left !important;
}
.dijitTextBoxError .dijitValidationContainer {
	display: inline;
	cursor: default;
}
.dijitSpinner .dijitSpinnerButtonContainer,
.dijitComboBox .dijitArrowButtonContainer {
	
	border-width: 0 0 0 1px !important; 
}
.dijitToolbar .dijitComboBox .dijitArrowButtonContainer {
	
	border-width: 0 !important;
}
.dijitComboBoxMenu {
	
	list-style-type: none;
}
.dijitSpinner .dijitSpinnerButtonContainer .dijitButtonNode {
	
	border-width: 0;
}
.dj_ie .dijit_a11y .dijitSpinner .dijitSpinnerButtonContainer .dijitButtonNode {
	clear: both; 
}
.dj_ie .dijitToolbar .dijitComboBox {
	
	vertical-align: middle;
}
.dijitTextBox .dijitSpinnerButtonContainer {
	width: 1em;
	position: relative !important;
	overflow: hidden;
}
.dijitSpinner .dijitSpinnerButtonInner {
	width:1em;
	visibility:hidden !important; 
	overflow-x:hidden;
}
.dijitComboBox .dijitButtonNode,
.dijitSpinnerButtonContainer .dijitButtonNode {
	border-width: 0;
}
.dijit_a11y .dijitSpinnerButtonContainer .dijitButtonNode {
	border: 0 none !important;
}
.dijit_a11y .dijitTextBox .dijitSpinnerButtonContainer,
.dijit_a11y .dijitSpinner .dijitArrowButtonInner,
.dijit_a11y .dijitSpinnerButtonContainer input {
	width: 1em !important;
}
.dijit_a11y .dijitSpinner .dijitArrowButtonInner {
	margin: 0 auto !important; 
}
.dj_ie .dijit_a11y .dijitSpinner .dijitArrowButtonInner .dijitInputField {
	padding-left: 0.3em !important;
	padding-right: 0.3em !important;
	margin-left: 0.3em !important;
	margin-right: 0.3em !important;
	width: 1.4em !important;
}
.dj_ie7 .dijit_a11y .dijitSpinner .dijitArrowButtonInner .dijitInputField {
	padding-left: 0 !important; 
	padding-right: 0 !important;
	width: 1em !important;
}
.dj_ie6 .dijit_a11y .dijitSpinner .dijitArrowButtonInner .dijitInputField {
	margin-left: 0.1em !important;
	margin-right: 0.1em !important;
	width: 1em !important;
}
.dj_iequirks .dijit_a11y .dijitSpinner .dijitArrowButtonInner .dijitInputField {
	margin-left: 0 !important;
	margin-right: 0 !important;
	width: 2em !important;
}
.dijitSpinner .dijitSpinnerButtonContainer .dijitArrowButton {
	
	padding: 0;
	position: absolute !important;
	right: 0;
	float: none;
	height: 50%;
	width: 100%;
	bottom: auto;
	left: 0;
	right: auto;
}
.dj_iequirks .dijitSpinner .dijitSpinnerButtonContainer .dijitArrowButton {
	width: auto;
}
.dijit_a11y .dijitSpinnerButtonContainer .dijitArrowButton {
	overflow: visible !important;
}
.dijitSpinner .dijitSpinnerButtonContainer .dijitDownArrowButton {
	top: 50%;
	border-top-width: 1px !important;
}
.dijitSpinner .dijitSpinnerButtonContainer .dijitUpArrowButton {
	#bottom: 50%;	
	top: 0;
}
.dijitSpinner .dijitArrowButtonInner {
	margin: auto;
	overflow-x: hidden;
	height: 100% !important;
}
.dj_iequirks .dijitSpinner .dijitArrowButtonInner {
	height: auto !important;
}
.dijitSpinner .dijitArrowButtonInner .dijitInputField {
	-moz-transform: scale(0.5);
	-moz-transform-origin: center top;
	-webkit-transform: scale(0.5);
	-webkit-transform-origin: center top;
	-o-transform: scale(0.5);
	-o-transform-origin: center top;
	transform: scale(0.5);
	transform-origin: left top;
	padding-top: 0;
	padding-bottom: 0;
	padding-left: 0 !important;
	padding-right: 0 !important;
	width: 100%;
}
.dj_ie .dijitSpinner .dijitArrowButtonInner .dijitInputField {
	zoom: 50%; 
}
.dijitSpinner .dijitSpinnerButtonContainer .dijitArrowButtonInner {
	overflow: hidden;
}
.dijit_a11y .dijitSpinner .dijitSpinnerButtonContainer .dijitArrowButton {
	width: 100%;
}
.dj_iequirks .dijit_a11y .dijitSpinner .dijitSpinnerButtonContainer .dijitArrowButton {
	width: 1em; 
}
.dijitSpinner .dijitArrowButtonInner .dijitInputField {
	visibility: hidden;
}
.dijit_a11y .dijitSpinner .dijitArrowButtonInner .dijitInputField {
	vertical-align:top;
	visibility: visible;
}
.dijit_a11y .dijitSpinnerButtonContainer {
	width: 1em;
}
.dijit_a11y .dijitSpinnerButtonContainer .dijitButtonNode {
	border-width: 1px 0 0 0;
	border-style: solid !important;
}
.dijitCheckBox,
.dijitRadio,
.dijitCheckBoxInput {
	padding: 0;
	border: 0;
	width: 16px;
	height: 16px;
	background-position:center center;
	background-repeat:no-repeat;
	overflow: hidden;
}
.dijitCheckBox input,
.dijitRadio input {
	margin: 0;
	padding: 0;
	display: block;
}
.dijitCheckBoxInput {
	
	opacity: 0.01;
}
.dj_ie .dijitCheckBoxInput {
	filter: alpha(opacity=0);
}
.dijit_a11y .dijitCheckBox,
.dijit_a11y .dijitRadio {
	
	width: auto !important;
	height: auto !important;
}
.dijit_a11y .dijitCheckBoxInput {
	opacity: 1;
	filter: none;
	width: auto;
	height: auto;
}
.dijit_a11y .dijitFocusedLabel {
	
	border: 1px dotted;
	outline: 0px !important;
}
.dijitProgressBarEmpty {
	
	position:relative;overflow:hidden;
	border:1px solid black; 	
	z-index:0;			
}
.dijitProgressBarFull {
	
	position:absolute;
	overflow:hidden;
	z-index:-1;
	top:0;
	width:100%;
}
.dj_ie6 .dijitProgressBarFull {
	height:1.6em;
}
.dijitProgressBarTile {
	
	position:absolute;
	overflow:hidden;
	top:0;
	left:0;
	bottom:0;
	right:0;
	margin:0;
	padding:0;
	width: 100%;    
	height:auto;
	background-color:#aaa;
	background-attachment: fixed;
}
.dijit_a11y .dijitProgressBarTile {
	
	border-width:2px;
	border-style:solid;
	background-color:transparent !important;
}
.dj_ie6 .dijitProgressBarTile {
	
	position:static;
	
	height:1.6em;
}
.dijitProgressBarIndeterminate .dijitProgressBarTile {
	
}
.dijitProgressBarIndeterminateHighContrastImage {
	display:none;
}
.dijit_a11y .dijitProgressBarIndeterminate .dijitProgressBarIndeterminateHighContrastImage {
	display:block;
	position:absolute;
	top:0;
	bottom:0;
	margin:0;
	padding:0;
	width:100%;
	height:auto;
}
.dijitProgressBarLabel {
	display:block;
	position:static;
	width:100%;
	text-align:center;
	background-color:transparent !important;
}
.dijitTooltip {
	position: absolute;
	z-index: 2000;
	display: block;
	
	left: 0;
	top: -10000px;
	overflow: visible;
}
.dijitTooltipContainer {
	border: solid black 2px;
	background: #b8b5b5;
	color: black;
	font-size: small;
}
.dijitTooltipFocusNode {
	padding: 2px 2px 2px 2px;
}
.dijitTooltipConnector {
	position: absolute;
}
.dijit_a11y .dijitTooltipConnector {
	display: none;	
}
.dijitTooltipData {
	display:none;
}
.dijitLayoutContainer {
	position: relative;
	display: block;
	overflow: hidden;
}
.dijitAlignTop,
.dijitAlignBottom,
.dijitAlignLeft,
.dijitAlignRight {
	position: absolute;
	overflow: hidden;
}
body .dijitAlignClient { position: absolute; }
.dijitBorderContainer, .dijitBorderContainerNoGutter {
	position:relative;
	overflow: hidden;
}
.dijitBorderContainerPane,
.dijitBorderContainerNoGutterPane {
	position: absolute !important;	
	z-index: 2;		
}
.dijitBorderContainer > .dijitTextArea {
	
	resize: none;
}
.dijitGutter {
	
	position: absolute;
	font-size: 1px;		
}
.dijitSplitter {
	position: absolute;
	overflow: hidden;
	z-index: 10;		
	background-color: #fff;
	border-color: gray;
	border-style: solid;
	border-width: 0;
}
.dj_ie .dijitSplitter {
	z-index: 1;	
}
.dijitSplitterActive {
	z-index: 11 !important;
}
.dijitSplitterCover {
	position:absolute;
	z-index:-1;
	top:0;
	left:0;
	width:100%;
	height:100%;
}
.dijitSplitterCoverActive {
	z-index:3 !important;
}
.dj_ie .dijitSplitterCover {
	background: white;
	filter: alpha(opacity=0);
}
.dijitSplitterH {
	height: 7px;
	border-top:1px;
	border-bottom:1px;
	cursor: row-resize;
}
.dijitSplitterV {
	width: 7px;
	border-left:1px;
	border-right:1px;
	cursor: col-resize;
}
.dijitSplitContainer {
	position: relative;
	overflow: hidden;
	display: block;
}
.dj_ff3 .dijit_a11y div.dijitSplitter:focus {
	outline-style:dotted;
	outline-width: 2px;
}
.dijitSplitPane {
	position: absolute;
}
.dijitSplitContainerSizerH,
.dijitSplitContainerSizerV {
	position:absolute;
	font-size: 1px;
	background-color: ThreeDFace;
	border: 1px solid;
	border-color: ThreeDHighlight ThreeDShadow ThreeDShadow ThreeDHighlight;
	margin: 0;
}
.dijitSplitContainerSizerH .thumb, .dijitSplitterV .dijitSplitterThumb {
	overflow:hidden;
	position:absolute;
	top:49%;
}
.dijitSplitContainerSizerV .thumb, .dijitSplitterH .dijitSplitterThumb {
	position:absolute;
	left:49%;
}
.dijitSplitterShadow,
.dijitSplitContainerVirtualSizerH,
.dijitSplitContainerVirtualSizerV {
	font-size: 1px;
	background-color: ThreeDShadow;
	-moz-opacity: 0.5;
	opacity: 0.5;
	filter: Alpha(Opacity=50);
	margin: 0;
}
.dijitSplitContainerSizerH, .dijitSplitContainerVirtualSizerH {
	cursor: col-resize;
}
.dijitSplitContainerSizerV, .dijitSplitContainerVirtualSizerV {
	cursor: row-resize;
}
.dijit_a11y .dijitSplitterH {
	border-top:1px solid #d3d3d3 !important;
	border-bottom:1px solid #d3d3d3 !important;
}
.dijit_a11y .dijitSplitterV {
	border-left:1px solid #d3d3d3 !important;
	border-right:1px solid #d3d3d3 !important;
}
.dijitContentPane {
	display: block;
	overflow: auto;	
}
.dijitContentPaneSingleChild {
	
	overflow: hidden;
}
.dijitContentPaneLoading .dijitIconLoading,
.dijitContentPaneError .dijitIconError {
	margin-right: 9px;
}
.dijitTitlePane {
	display: block;
	overflow: hidden;
}
.dijitTitlePaneTitle {
	cursor: pointer;
}
.dijitFixedOpen, .dijitFixedClosed {
	
	cursor: default;
}
.dijitTitlePaneTitle * {
	vertical-align: middle;
}
.dijitTitlePane .dijitArrowNodeInner {
	
	display: none;
}
.dijit_a11y .dijitTitlePane .dijitArrowNodeInner {
	
	display:inline !important;
	font-family: monospace;		
}
.dijit_a11y .dijitTitlePane .dijitArrowNode {
	
	display:none;
}
.dj_ie6 .dijitTitlePaneContentOuter,
.dj_ie6 .dijitTitlePane .dijitTitlePaneTitle {
	
	zoom: 1;
}
.dijitColorPalette {
	border: 1px solid #999;
	background: #fff;
	position: relative;
}
.dijitColorPalette .dijitPaletteTable {
	
	padding: 2px 3px 3px 3px;
	position: relative;
	overflow: hidden;
	outline: 0;
	border-collapse: separate;
}
.dj_ie6 .dijitColorPalette .dijitPaletteTable,
.dj_ie7 .dijitColorPalette .dijitPaletteTable,
.dj_iequirks .dijitColorPalette .dijitPaletteTable {
	
	padding: 0;
	margin: 2px 3px 3px 3px;
}
.dijitColorPalette .dijitPaletteCell {
	
	font-size: 1px;
	vertical-align: middle;
	text-align: center;
	background: none;
}
.dijitColorPalette .dijitPaletteImg {
	
	padding: 1px;		
	border: 1px solid #999;
	margin: 2px 1px;
	cursor: default;
	font-size: 1px;		
}
.dj_gecko .dijitColorPalette .dijitPaletteImg {
	padding-bottom: 0;	
}
.dijitColorPalette .dijitColorPaletteSwatch {
	
	width: 14px;
	height: 12px;
}
.dijitPaletteTable td {
		padding: 0;
}
.dijitColorPalette .dijitPaletteCell:hover .dijitPaletteImg {
	
	border: 1px solid #000;
}
.dijitColorPalette .dijitPaletteCell:active .dijitPaletteImg,
.dijitColorPalette .dijitPaletteTable .dijitPaletteCellSelected .dijitPaletteImg {
	border: 2px solid #000;
	margin: 1px 0;	
}
.dijit_a11y .dijitColorPalette .dijitPaletteTable,
.dijit_a11y .dijitColorPalette .dijitPaletteTable * {
	
	background-color: transparent !important;
}
.dijitAccordionContainer {
	border:1px solid #b7b7b7;
	border-top:0 !important;
}
.dijitAccordionTitle {
	cursor: pointer;
}
.dijitAccordionTitleSelected {
	cursor: default;
}
.dijitAccordionTitle .arrowTextUp,
.dijitAccordionTitle .arrowTextDown {
	display: none;
	font-size: 0.65em;
	font-weight: normal !important;
}
.dijit_a11y .dijitAccordionTitle .arrowTextUp,
.dijit_a11y .dijitAccordionTitleSelected .arrowTextDown {
	display: inline;
}
.dijit_a11y .dijitAccordionTitleSelected .arrowTextUp {
	display: none;
}
.dijitAccordionChildWrapper {
	
	overflow: hidden;
}
.dijitCalendarContainer {
	width: auto;	
}
.dijitCalendarContainer th, .dijitCalendarContainer td {
	padding: 0;
	vertical-align: middle;
}
.dijitCalendarYearLabel {
    white-space: nowrap;    
}
.dijitCalendarNextYear {
	margin:0 0 0 0.55em;
}
.dijitCalendarPreviousYear {
	margin:0 0.55em 0 0;
}
.dijitCalendarIncrementControl {
	vertical-align: middle;
}
.dijitCalendarIncrementControl,
.dijitCalendarDateTemplate,
.dijitCalendarMonthLabel,
.dijitCalendarPreviousYear,
.dijitCalendarNextYear {
	cursor: pointer;
}
.dijitCalendarDisabledDate {
	color: gray;
	text-decoration: line-through;
	cursor: default;
}
.dijitSpacer {
	
  	position: relative;
  	height: 1px;
  	overflow: hidden;
  	visibility: hidden;
}
.dijitCalendarMonthMenu .dijitCalendarMonthLabel {
	text-align:center;
}
.dijitMenu {
	border:1px solid black;
	background-color:white;
}
.dijitMenuTable {
	border-collapse:collapse;
	border-width:0;
	background-color:white;
}
.dj_webkit .dijitMenuTable td[colspan="2"]{
	border-right:hidden;
}
.dijitMenuItem {
	text-align: left;
	white-space: nowrap;
	padding:.1em .2em;
	cursor:pointer;
}
.dijitMenuPassive .dijitMenuItemHover,
.dijitMenuItemSelected {
	
	background-color:black;
	color:white;
}
.dijitMenuItemIcon, .dijitMenuExpand {
	background-repeat: no-repeat;
}
.dijitMenuItemDisabled * {
	
	opacity:0.5;
	cursor:default;
}
.dj_ie .dijit_a11y .dijitMenuItemDisabled,
.dj_ie .dijit_a11y .dijitMenuItemDisabled td,
.dj_ie .dijitMenuItemDisabled *,
.dj_ie .dijitMenuItemDisabled td {
	color:gray !important;
	filter: alpha(opacity=35);
}
.dijitMenuItemLabel {
	position: relative;
	vertical-align: middle;
}
.dijit_a11y .dijitMenuItemSelected {
	border: 1px dotted black !important;
}
.dj_ff3 .dijit_a11y .dijitMenuItem td {
	padding: 0 !important;
	background:none !important;
}
.dijit_a11y .dijitMenuItemSelected .dijitMenuItemLabel {
	border-width: 1px;
	border-style: solid;
}
.dj_ie8 .dijit_a11y .dijitMenuItemLabel {
	position:static;
}
.dijitMenuExpandA11y {
	display: none;
}
.dijit_a11y .dijitMenuExpandA11y {
	display: inline;
}
.dijitMenuSeparator td {
	border: 0;
	padding: 0;
}
.dijitMenuSeparatorTop {
	height: 50%;
	margin: 0;
	margin-top:3px;
	font-size: 1px;
}
.dijitMenuSeparatorBottom {
	height: 50%;
	margin: 0;
	margin-bottom:3px;
	font-size: 1px;
}
.dijitCheckedMenuItemIconChar {
	vertical-align: middle;
	visibility:hidden;
}
.dijitCheckedMenuItemChecked .dijitCheckedMenuItemIconChar {
	visibility: visible;
}
.dijit_a11y .dijitCheckedMenuItemIconChar {
	display:inline !important;
}
.dijit_a11y .dijitCheckedMenuItemIcon {
	display: none;
}
.dj_ie .dijit_a11y .dijitMenuBar .dijitMenuItem {
	
	margin: 0;
}
.dijitStackController .dijitToggleButtonChecked * {
	cursor: default;	
}
.dijitTabContainerNoLayout {
	width: 100%;	
}
.dijitTabContainerBottom-tabs,
.dijitTabContainerTop-tabs,
.dijitTabContainerLeft-tabs,
.dijitTabContainerRight-tabs {
	overflow: visible !important;  
}
.dijitTabContainerBottom-container,
.dijitTabContainerTop-container,
.dijitTabContainerLeft-container,
.dijitTabContainerRight-container {
	z-index:0;
	overflow: hidden;
	border: 1px solid black;
}
.nowrapTabStrip {
	width: 50000px;
	display: block;
	position: relative;
    text-align: left;  
}
.dijitTabListWrapper {
	overflow: hidden;
}
.dijit_a11y .tabStripButton img {
	
	display: none;
}
.dijitTabContainerTop-tabs {
	border-bottom: 1px solid black;
}
.dijitTabContainerTop-container {
	border-top: 0;
}
.dijitTabContainerLeft-tabs {
	border-right: 1px solid black;
	float: left;
}
.dijitTabContainerLeft-container {
	border-left: 0;
}
.dijitTabContainerBottom-tabs {
	border-top: 1px solid black;
}
.dijitTabContainerBottom-container {
	border-bottom: 0;
}
.dijitTabContainerRight-tabs {
	border-left: 1px solid black;
	float: left;
}
.dijitTabContainerRight-container {
	border-right: 0;
}
div.dijitTabDisabled, .dj_ie div.dijitTabDisabled {
	cursor: auto;
}
.dijitTab {
	position:relative;
	cursor:pointer;
	white-space:nowrap;
	z-index:3;
}
.dijitTab * {
	
	vertical-align: middle;
}
.dijitTabChecked {
	cursor: default;	
}
.dijitTabContainerTop-tabs .dijitTab {
	top: 1px;	
}
.dijitTabContainerBottom-tabs .dijitTab {
	top: -1px;	
}
.dijitTabContainerLeft-tabs .dijitTab {
	left: 1px;	
}
.dijitTabContainerRight-tabs .dijitTab {
	left: -1px;	
}
.dijitTabContainerTop-tabs .dijitTab,
.dijitTabContainerBottom-tabs .dijitTab {
	
	display:inline-block;			
	#zoom: 1; 
	#display:inline; 
}
.dijitTabInnerDiv {
	position:relative;
}
.tabStripButton {
	z-index: 12;
}
.dijitTabButtonDisabled .tabStripButton {
	display: none;
}
.dijitTabCloseButton {
	margin-left: 1em;
}
.dijitTabCloseText {
	display:none;
}
.dijitTab .tabLabel {
	
	min-height: 15px;
	display: inline-block;
}
.dijitNoIcon {
	
	display: none;
}
.dj_ie6 .dijitTab .dijitNoIcon {
	
	display: inline;
	height: 15px;
	width: 1px;
}
.dijit_a11y .dijitTabCloseButton {
	background-image: none !important;
	width: auto !important;
	height: auto !important;
}
.dijit_a11y .dijitTabCloseText {
	display: inline;
}
.dijitTabPane,
.dijitStackContainer-child,
.dijitAccordionContainer-child {
	
    border: none !important;
}
.dijitInlineEditBoxDisplayMode {
	border: 1px solid transparent;	
	cursor: text;
}
.dijit_a11y .dijitInlineEditBoxDisplayMode,
.dj_ie6 .dijitInlineEditBoxDisplayMode {
	
	border: none;
}
.dijitInlineEditBoxDisplayModeHover,
.dijit_a11y .dijitInlineEditBoxDisplayModeHover,
.dj_ie6 .dijitInlineEditBoxDisplayModeHover {
	
	background-color: #e2ebf2;
	border: solid 1px black;
}
.dijitInlineEditBoxDisplayModeDisabled {
	cursor: default;
}
.dijitTree {
	overflow: auto;	
}
.dijitTreeIndent {
	
	width: 19px;
}
.dijitTreeRow, .dijitTreeContent {
	white-space: nowrap;
}
.dijitTreeRow img {
	
	vertical-align: middle;
}
.dijitTreeContent {
    cursor: default;
}
.dijitExpandoText {
	display: none;
}
.dijit_a11y .dijitExpandoText {
	display: inline;
	padding-left: 10px;
	padding-right: 10px;
	font-family: monospace;
	border-style: solid;
	border-width: thin;
	cursor: pointer;
}
.dijitTreeLabel {
	margin: 0 4px;
}
.dijitDialog {
	position: absolute;
	z-index: 999;
	overflow: hidden;       
}
.dijitDialogTitleBar {
	cursor: move;
}
.dijitDialogFixed .dijitDialogTitleBar {
	cursor:default;
}
.dijitDialogCloseIcon {
	cursor: pointer;
}
.dijitDialogUnderlayWrapper {
	position: absolute;
	left: 0;
	top: 0;
	z-index: 998;
	display: none;
	background: transparent !important;
}
.dijitDialogUnderlay {
	background: #eee;
	opacity: 0.5;
}
.dj_ie .dijitDialogUnderlay {
	filter: alpha(opacity=50);
}
.dijit_a11y .dijitSpinnerButtonContainer,
.dijit_a11y .dijitDialog {
	opacity: 1 !important;
	background-color: white !important;
}
.dijitDialog .closeText {
	display:none;
	
	position:absolute;
}
.dijit_a11y .dijitDialog .closeText {
	display:inline;
}
.dijitSliderMoveable {
	z-index:99;
	position:absolute !important;
	display:block;
	vertical-align:middle;
}
.dijitSliderMoveableH {
	right:0;
}
.dijitSliderMoveableV {
	right:50%;
}
.dijit_a11y div.dijitSliderImageHandle,
.dijitSliderImageHandle {
	margin:0;
	padding:0;
	position:relative !important;
	border:8px solid gray;
	width:0;
	height:0;
	cursor: pointer;
}
.dj_iequirks .dijit_a11y .dijitSliderImageHandle {
	font-size: 0;
}
.dj_ie7 .dijitSliderImageHandle {
	overflow: hidden; 
}
.dj_ie7 .dijit_a11y .dijitSliderImageHandle {
	overflow: visible; 
}
.dijit_a11y .dijitSliderFocused .dijitSliderImageHandle {
	border:4px solid #000;
	height:8px;
	width:8px;
}
.dijitSliderImageHandleV {
	top:-8px;
	right: -50%;
}
.dijitSliderImageHandleH {
	left:50%;
	top:-5px;
	vertical-align:top;
}
.dijitSliderBar {
	border-style:solid;
	border-color:black;
	cursor: pointer;
}
.dijitSliderBarContainerV {
	position:relative;
	height:100%;
	z-index:1;
}
.dijitSliderBarContainerH {
	position:relative;
	z-index:1;
}
.dijitSliderBarH {
	height:4px;
	border-width:1px 0;
}
.dijitSliderBarV {
	width:4px;
	border-width:0 1px;
}
.dijitSliderProgressBar {
	background-color:red;
	z-index:1;
}
.dijitSliderProgressBarV {
	position:static !important;
	height:0;
	vertical-align:top;
	text-align:left;
}
.dijitSliderProgressBarH {
	position:absolute !important;
	width:0;
	vertical-align:middle;
	overflow:visible;
}
.dijitSliderRemainingBar {
	overflow:hidden;
	background-color:transparent;
	z-index:1;
}
.dijitSliderRemainingBarV {
	height:100%;
	text-align:left;
}
.dijitSliderRemainingBarH {
	width:100% !important;
}
.dijitSliderBumper {
	overflow:hidden;
	z-index:1;
}
.dijitSliderBumperV {
	width:4px;
	height:8px;
	border-width:0 1px;
}
.dijitSliderBumperH {
	width:8px;
	height:4px;
	border-width:1px 0;
}
.dijitSliderBottomBumper,
.dijitSliderLeftBumper {
	background-color:red;
}
.dijitSliderTopBumper,
.dijitSliderRightBumper {
	background-color:transparent;
}
.dijitSliderDecoration {
	text-align:center;
}
.dijitSliderDecorationC,
.dijitSliderDecorationV {
	position: relative; 
}
.dijitSliderDecorationH {
	width: 100%;
}
.dijitSliderDecorationV {
	height: 100%;
}
.dijitSliderButton {
	font-family:monospace;
	margin:0;
	padding:0;
	display:block;
}
.dijit_a11y .dijitSliderButtonInner {
	visibility:visible !important;
}
.dijitSliderButtonContainer {
	text-align:center;
	height:0;	
}
.dijitSliderButtonContainer * {
	cursor: pointer;
}
.dijitSlider .dijitButtonNode {
	padding:0;
	display:block;
}
.dijitRuleContainer {
	position:relative;
	overflow:visible;
}
.dijitRuleContainerV {
	height:100%;
	line-height:0;
	float:left;
	text-align:left;
}
.dj_opera .dijitRuleContainerV {
	line-height:2%;
}
.dj_ie .dijitRuleContainerV {
	line-height:normal;
}
.dj_gecko .dijitRuleContainerV {
	margin:0 0 1px 0; 
}
.dijitRuleMark {
	position:absolute;
	border:1px solid black;
	line-height:0;
	height:100%;
}
.dijitRuleMarkH {
	width:0;
	border-top-width:0 !important;
	border-bottom-width:0 !important;
	border-left-width:0 !important;
}
.dijitRuleLabelContainer {
	position:absolute;
}
.dijitRuleLabelContainerH {
	text-align:center;
	display:inline-block;
}
.dijitRuleLabelH {
	position:relative;
	left:-50%;
}
.dijitRuleLabelV {
	
	text-overflow: ellipsis;
	white-space: nowrap;
	overflow: hidden;
}
.dijitRuleMarkV {
	height:0;
	border-right-width:0 !important;
	border-bottom-width:0 !important;
	border-left-width:0 !important;
	width:100%;
	left:0;
}
.dj_ie .dijitRuleLabelContainerV {
	margin-top:-.55em;
}
.dijit_a11y .dijitSliderReadOnly,
.dijit_a11y .dijitSliderDisabled {
	opacity:0.6;
}
.dj_ie .dijit_a11y .dijitSliderReadOnly .dijitSliderBar,
.dj_ie .dijit_a11y .dijitSliderDisabled .dijitSliderBar {
	filter: alpha(opacity=40);
}
.dijit_a11y .dijitSlider .dijitSliderButtonContainer div {
	font-family: monospace; 
	font-size: 1em;
	line-height: 1em;
	height: auto;
	width: auto;
	margin: 0 4px;
}
.dijit_a11y .dijitButtonContents .dijitButtonText,
.dijit_a11y .dijitTab .tabLabel {
	display: inline !important;
}
.dijitTextArea {
	width:100%;
	overflow-y: auto;	
}
.dijitTextArea[cols] {
	width:auto; 
}
.dj_ie .dijitTextAreaCols {
	width:auto;
}
.dijitExpandingTextArea {
	
	resize: none;
}
.dijitToolbarSeparator {
	height: 18px;
	width: 5px;
	padding: 0 1px;
	margin: 0;
}
.dijitIEFixedToolbar {
	position:absolute;
	
	top: expression(eval((document.documentElement||document.body).scrollTop));
}
.dijitEditor {
	display: block;	
}
.dijitEditorDisabled,
.dijitEditorReadOnly {
	color: gray;
}
.dijitTimePickerItemInner {
	text-align:center;
	border:0;
	padding:2px 8px 2px 8px;
}
.dijitTimePickerTick,
.dijitTimePickerMarker {
	border-bottom:1px solid gray;
}
.dijitTimePicker .dijitDownArrowButton {
	border-top: none !important;
}
.dijitTimePickerTick {
	color:#CCC;
}
.dijitTimePickerMarker {
	color:black;
	background-color:#CCC;
}
.dijitTimePickerItemSelected {
	font-weight:bold;
	color:#333;
	background-color:#b7cdee;
}
.dijitTimePickerItemHover {
	background-color:gray;
	color:white;
	cursor:pointer;
}
.dijit_a11y .dijitTimePickerItemSelected .dijitTimePickerItemInner {
	border: solid 4px black;
}
.dijit_a11y .dijitTimePickerItemHover .dijitTimePickerItemInner {
	border: dashed 4px black;
}
.dijitToggleButtonIconChar {
	
	display:none !important;
}
.dijit_a11y .dijitToggleButton .dijitToggleButtonIconChar {
	display:inline !important;
	visibility:hidden;
}
.dj_ie6 .dijitToggleButtonIconChar, .dj_ie6 .tabStripButton .dijitButtonText {
	font-family: "Arial Unicode MS";	
}
.dijit_a11y .dijitToggleButtonChecked .dijitToggleButtonIconChar {
	display: inline !important; 
	visibility:visible !important;
}
.dijitArrowButtonChar {
        display:none !important;
}
.dijit_a11y .dijitArrowButtonChar {
        display:inline !important;
}
.dijit_a11y .dijitDropDownButton .dijitArrowButtonInner,
.dijit_a11y .dijitComboButton .dijitArrowButtonInner {
	display:none !important;
}
.dijitSelect {
	margin: 0.2em;
	border-collapse: collapse;
}
.dj_ie .dijitSelect,
.dj_ie7 .dijitSelect,
.dj_iequirks .dijitSelect {
	vertical-align: middle; 
}
.dj_ie8 .dijitSelect .dijitButtonText {
	vertical-align: top;
}
.dijitToolbar .dijitSelect {
	margin: 0;
}
.dj_webkit .dijitToolbar .dijitSelect {
	padding-left: 0.3em;
}
.dijit_a11y .dijitSelectDisabled .dijitButtonNode {
	border-style: outset!important;
	border-width: medium!important;
	border-color: #999 !important;
	color:#999 !important;
}
.dijitSelect .dijitButtonContents {
	padding: 0;
	background: transparent none;
	white-space: nowrap;
	text-align: left;
}
.dijitSelectFixedWidth .dijitButtonContents {
	width: 100%;
}
.dijitSelectMenu .dijitMenuItemIcon {
	
	display:none;
}
.dj_ie6 .dijitSelectMenu .dijitMenuItemLabel,
.dj_ie7 .dijitSelectMenu .dijitMenuItemLabel {
	
	position: static;
}
.dijitSelectLabel *
{
	vertical-align: baseline;
}
.dijitSelectSelectedOption * {
	font-weight: bold;
}
.dijitSelectMenu {
	border-width: 1px;
}
.dijitSelectMenu .dijitMenuTable {
	margin: 0;
	background-color: transparent;
}
.dijitForceStatic {
	position: static !important;
}
.dijitReadOnly *,
.dijitDisabled *,
.dijitReadOnly,
.dijitDisabled {
	
	cursor: default;
}
