//>>built
define(
//begin v1.x content
{
	"months-format-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"field-weekday": "jour de la semaine",
	"dateFormatItem-yyQQQQ": "QQQQ yy",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-yMEd": "EEE d/M/yyyy",
	"dateFormatItem-MMMEd": "E d MMM",
	"eraNarrow": [
		"av. J.-C.",
		"ap. J.-C."
	],
	"dayPeriods-format-wide-morning": "matin",
	"dateFormatItem-MMMdd": "dd MMM",
	"dateFormat-long": "d MMMM y",
	"months-format-wide": [
		"janvier",
		"février",
		"mars",
		"avril",
		"mai",
		"juin",
		"juillet",
		"août",
		"septembre",
		"octobre",
		"novembre",
		"décembre"
	],
	"dayPeriods-format-wide-pm": "PM",
	"dateFormat-full": "EEEE d MMMM y",
	"dateFormatItem-Md": "d/M",
	"dayPeriods-format-wide-noon": "midi",
	"field-era": "ère",
	"dateFormatItem-yM": "M/yyyy",
	"months-standAlone-wide": [
		"janvier",
		"février",
		"mars",
		"avril",
		"mai",
		"juin",
		"juillet",
		"août",
		"septembre",
		"octobre",
		"novembre",
		"décembre"
	],
	"timeFormat-short": "HH:mm",
	"quarters-format-wide": [
		"1er trimestre",
		"2e trimestre",
		"3e trimestre",
		"4e trimestre"
	],
	"timeFormat-long": "HH:mm:ss z",
	"field-year": "année",
	"dateFormatItem-yMMM": "MMM y",
	"dateFormatItem-yQ": "'T'Q y",
	"dateFormatItem-yyyyMMMM": "MMMM y",
	"field-hour": "heure",
	"dateFormatItem-MMdd": "dd/MM",
	"months-format-abbr": [
		"janv.",
		"févr.",
		"mars",
		"avr.",
		"mai",
		"juin",
		"juil.",
		"août",
		"sept.",
		"oct.",
		"nov.",
		"déc."
	],
	"dateFormatItem-yyQ": "'T'Q yy",
	"timeFormat-full": "HH:mm:ss zzzz",
	"field-day-relative+0": "aujourd’hui",
	"field-day-relative+1": "demain",
	"field-day-relative+2": "après-demain",
	"field-day-relative+3": "après-après-demain",
	"months-standAlone-abbr": [
		"janv.",
		"févr.",
		"mars",
		"avr.",
		"mai",
		"juin",
		"juil.",
		"août",
		"sept.",
		"oct.",
		"nov.",
		"déc."
	],
	"quarters-format-abbr": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"quarters-standAlone-wide": [
		"1er trimestre",
		"2e trimestre",
		"3e trimestre",
		"4e trimestre"
	],
	"dateFormatItem-M": "L",
	"days-standAlone-wide": [
		"dimanche",
		"lundi",
		"mardi",
		"mercredi",
		"jeudi",
		"vendredi",
		"samedi"
	],
	"dateFormatItem-yyMMMEEEd": "EEE d MMM yy",
	"dateFormatItem-yyMMM": "MMM yy",
	"timeFormat-medium": "HH:mm:ss",
	"dateFormatItem-Hm": "HH:mm",
	"quarters-standAlone-abbr": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"eraAbbr": [
		"av. J.-C.",
		"ap. J.-C."
	],
	"field-minute": "minute",
	"field-dayperiod": "cadran",
	"days-standAlone-abbr": [
		"dim.",
		"lun.",
		"mar.",
		"mer.",
		"jeu.",
		"ven.",
		"sam."
	],
	"dayPeriods-format-wide-night": "soir",
	"dateFormatItem-yyMMMd": "d MMM yy",
	"dateFormatItem-d": "d",
	"dateFormatItem-ms": "mm:ss",
	"quarters-format-narrow": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"field-day-relative+-1": "hier",
	"field-day-relative+-2": "avant-hier",
	"field-day-relative+-3": "avant-avant-hier",
	"dateFormatItem-MMMd": "d MMM",
	"dateFormatItem-MEd": "EEE d/M",
	"field-day": "jour",
	"days-format-wide": [
		"dimanche",
		"lundi",
		"mardi",
		"mercredi",
		"jeudi",
		"vendredi",
		"samedi"
	],
	"field-zone": "fuseau horaire",
	"dateFormatItem-y": "y",
	"months-standAlone-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dateFormatItem-yyMM": "MM/yy",
	"days-format-abbr": [
		"dim.",
		"lun.",
		"mar.",
		"mer.",
		"jeu.",
		"ven.",
		"sam."
	],
	"eraNames": [
		"avant Jésus-Christ",
		"après Jésus-Christ"
	],
	"days-format-narrow": [
		"D",
		"L",
		"M",
		"M",
		"J",
		"V",
		"S"
	],
	"field-month": "mois",
	"days-standAlone-narrow": [
		"D",
		"L",
		"M",
		"M",
		"J",
		"V",
		"S"
	],
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "AM",
	"dateFormatItem-MMMMEd": "EEE d MMMM",
	"dateFormat-short": "dd/MM/yy",
	"dateFormatItem-MMd": "d/MM",
	"dayPeriods-format-wide-afternoon": "après-midi",
	"field-second": "seconde",
	"dateFormatItem-yMMMEd": "EEE d MMM y",
	"dateFormatItem-Ed": "E d",
	"field-week": "semaine",
	"dateFormat-medium": "d MMM y",
	"dateFormatItem-Hms": "HH:mm:ss"
}
//end v1.x content
);