//>>built
define(
//begin v1.x content
({
	invalidMessage: "El valor especificado no es válido.",
	missingMessage: "Este valor es necesario.",
	rangeMessage: "Este valor está fuera del intervalo."
})
//end v1.x content
);
