
.claro .dijitEditorIFrameContainer {
  padding: 3px 3px 1px 10px;
}
.claro .dijitEditorIFrame {
  background-color: #ffffff;
}
.claro .dijitEditor {
  border: 1px solid #b5bcc7;
}
.claro .dijitEditor .dijitEditorIFrameContainer {
  background-color: #ffffff;
  background-image: url("form/images/textBox_back.png");
  background-repeat: repeat-x;
}
.dj_ie6 .claro .dijitEditor .dijitEditorIFrameContainer {
  background-image: none;
}
.claro .dijitEditorHover .dijitEditorIFrameContainer, .claro .dijitEditorHover .dijitEditorIFrameContainer .dijitEditorIFrame {
  background-color: #e5f2fe;
}
.claro .dijitEditorFocused .dijitEditorIFrameContainer, .claro .dijitEditorFocused .dijitEditorIFrameContainer .dijitEditorIFrame {
  
  background-color: #ffffff;
}
.claro .dijitEditorDisabled {
  border: 1px solid #d3d3d3;
  color: #818181;
}
.claro .dijitDisabled .dijitEditorIFrame, .claro .dijitDisabled .dijitEditorIFrameContainer, .claro .dijitDisabled .dijitEditorIFrameContainer .dijitEditorIFrame {
  background-color: #efefef;
  background-image: none;
}
