/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/

//>>built
define("dojo/require",["./_base/loader"],function(_1){
return {dynamic:0,normalize:function(id){
return id;
},load:_1.require};
});
