//>>built
define(
//begin v1.x content
({
	displayOptions: "[表示オプション]",
	title: "タイトル",
	authors: "作成者",
	contributors: "貢献者",
	id: "ID",
	close: "[閉じる]",
	updated: "更新",
	summary: "要約",
	content: "内容"
})
//end v1.x content
);
