//>>built
define(
//begin v1.x content
{
	"dateFormat-medium": "MMM d, y G",
	"dateFormatItem-MMMEd": "E, MMM d",
	"dateFormatItem-MEd": "E, M/d",
	"dateFormatItem-yMEd": "EEE, M/d/y G",
	"dateFormatItem-Hm": "HH:mm",
	"dateFormatItem-y": "y G",
	"timeFormat-full": "h:mm:ss a zzzz",
	"dateFormatItem-hm": "h:mm a",
	"dateFormatItem-Md": "M/d",
	"months-standAlone-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dateFormatItem-EEEd": "d EEE",
	"dateFormatItem-M": "L",
	"days-standAlone-narrow": [
		"S",
		"M",
		"T",
		"W",
		"T",
		"F",
		"S"
	],
	"dateFormatItem-yQQQ": "QQQ y G",
	"timeFormat-medium": "h:mm:ss a",
	"dateFormatItem-Hms": "HH:mm:ss",
	"dateFormat-long": "MMMM d, y G",
	"dateFormatItem-ms": "mm:ss",
	"dateFormat-short": "M/d/yy G",
	"dateFormatItem-yMMMEd": "EEE, MMM d, y G",
	"months-format-wide": [
		"January",
		"February",
		"March",
		"April",
		"May",
		"June",
		"July",
		"August",
		"September",
		"October",
		"November",
		"December"
	],
	"dateFormatItem-d": "d",
	"dateFormatItem-yM": "M/y G",
	"timeFormat-short": "h:mm a",
	"months-format-abbr": [
		"Jan",
		"Feb",
		"Mar",
		"Apr",
		"May",
		"Jun",
		"Jul",
		"Aug",
		"Sep",
		"Oct",
		"Nov",
		"Dec"
	],
	"timeFormat-long": "h:mm:ss a z",
	"days-format-wide": [
		"Sunday",
		"Monday",
		"Tuesday",
		"Wednesday",
		"Thursday",
		"Friday",
		"Saturday"
	],
	"dateFormatItem-hms": "h:mm:ss a",
	"dateFormatItem-yQ": "Q y G",
	"dateFormatItem-MMM": "LLL",
	"dateFormatItem-yMMM": "MMM y G",
	"quarters-format-wide": [
		"1st quarter",
		"2nd quarter",
		"3rd quarter",
		"4th quarter"
	],
	"dateFormat-full": "EEEE, MMMM d, y G",
	"dateFormatItem-MMMd": "MMM d",
	"days-format-abbr": [
		"Sun",
		"Mon",
		"Tue",
		"Wed",
		"Thu",
		"Fri",
		"Sat"
	]
}
//end v1.x content
);