
.tundra .dijitTreeNodeRtl,
.tundra .dijitTreeNodeRtl .dijitTreeExpandoLeaf {
	
    background-image: none;
}
.tundra .dijitTreeNodeRtl .dijitTreeExpandoOpened {
	
	background-image: url('images/treeExpand_minus_rtl.gif');
}
.tundra .dijitTreeNodeRtl .dijitTreeExpandoClosed {
	background-image: url('images/treeExpand_plus_rtl.gif');
}
