.claro .dijitSliderRtl .dijitSliderProgressBarH,
.claro .dijitSliderRtl .dijitSliderRemainingBarH,
.claro .dijitSliderRtl .dijitSliderLeftBumper,
.claro .dijitSliderRtl .dijitSliderRightBumper,
.claro .dijitSliderRtl .dijitSliderTopBumper {
  background-position: top right;
}
.claro .dijitSliderRtl .dijitSliderProgressBarV, .claro .dijitSliderRtl .dijitSliderRemainingBarV, .claro .dijitSliderRtl .dijitSliderBottomBumper {
  background-position: bottom right;
}
.claro .dijitSliderRtl .dijitSliderLeftBumper {
  border-left-width: 0;
  border-right-width: 1px;
}
.claro .dijitSliderRtl .dijitSliderRightBumper {
  border-left-width: 1px;
  border-right-width: 0;
}
.claro .dijitSliderRtl .dijitSliderIncrementIconH {
  background-position: -357px 50%;
}
.claro .dijitSliderRtl .dijitSliderDecrementIconH {
  background-position: -251px 50%;
}
