.tundra .dijitTitlePaneTitle {
	background: #cccccc;
	background:#fff url("images/titleBar.png") repeat-x bottom left;
	border:1px solid #bfbfbf;
	padding:3px 4px;
}
.tundra .dijitTitlePaneTitleHover  {
	background: #f8fafd url("images/accordionItemHover.gif") bottom repeat-x;
}
.tundra .dijitTitlePane .dijitArrowNode {
	background-image: url('images/spriteArrows.png');
	background-repeat: no-repeat;
	background-position: 0 0;
	height: 7px;
	width: 7px;
}
.dj_ie6 .tundra .dijitTitlePane .dijitArrowNode {
	background-image: url('images/spriteArrows.gif');
}
.tundra .dijitTitlePane .dijitClosed .dijitArrowNode {
	background-position: -14px 0;
}
.tundra .dijitTitlePaneContentOuter {
	background: #ffffff;
	border:1px solid #bfbfbf;
	border-top: 0;
}
.tundra .dijitTitlePaneContentInner {
	padding:10px;
}
.tundra .dijitTitlePaneTextNode {
	margin-left: 4px;
	margin-right: 4px;
}