/*
   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
*/

#ifndef TRANSID_AI_HPP
#define TRANSID_AI_HPP

#include "SignalData.hpp"

#define JAM_FILE_ID 192


class TransIdAI {
  /**
   * Sender(s)
   */
  friend class Dbtup;
  
  /**
   * Receiver(s)
   */
  friend class NdbTransaction;
  friend class Dbtc;
  friend class Dbutil;
  friend class Dblqh;
  friend class Suma;

  friend bool printTRANSID_AI(FILE *, const Uint32 *, Uint32, Uint16);
  
public:
  STATIC_CONST( HeaderLength = 3 );
  STATIC_CONST( DataLength = 22 );

  // Public methods
public:
 Uint32* getData() const;

public:
  Uint32 connectPtr;
  Uint32 transId[2];
  Uint32 attrData[DataLength];
};

inline
Uint32* TransIdAI::getData() const
{
  return (Uint32*)&attrData[0];
}


#undef JAM_FILE_ID

#endif
