//>>built
define(
//begin v1.x content
{
	"months-format-narrow": [
		"E",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"field-weekday": "día de la semana",
	"dateFormatItem-yyQQQQ": "QQQQ 'de' yy",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-yMEd": "EEE d/M/y",
	"dateFormatItem-MMMEd": "E d MMM",
	"eraNarrow": [
		"a.C.",
		"d.C."
	],
	"dateFormatItem-MMMdd": "dd-MMM",
	"dateFormat-long": "d 'de' MMMM 'de' y",
	"months-format-wide": [
		"enero",
		"febrero",
		"marzo",
		"abril",
		"mayo",
		"junio",
		"julio",
		"agosto",
		"septiembre",
		"octubre",
		"noviembre",
		"diciembre"
	],
	"dateFormatItem-EEEd": "EEE d",
	"dayPeriods-format-wide-pm": "p.m.",
	"dateFormat-full": "EEEE d 'de' MMMM 'de' y",
	"dateFormatItem-Md": "d/M",
	"field-era": "era",
	"dateFormatItem-yM": "M/y",
	"months-standAlone-wide": [
		"enero",
		"febrero",
		"marzo",
		"abril",
		"mayo",
		"junio",
		"julio",
		"agosto",
		"septiembre",
		"octubre",
		"noviembre",
		"diciembre"
	],
	"timeFormat-short": "HH:mm",
	"quarters-format-wide": [
		"1er trimestre",
		"2º trimestre",
		"3er trimestre",
		"4º trimestre"
	],
	"timeFormat-long": "HH:mm:ss z",
	"field-year": "año",
	"dateFormatItem-yMMM": "MMM y",
	"dateFormatItem-yQ": "Q y",
	"field-hour": "hora",
	"months-format-abbr": [
		"ene",
		"feb",
		"mar",
		"abr",
		"may",
		"jun",
		"jul",
		"ago",
		"sep",
		"oct",
		"nov",
		"dic"
	],
	"dateFormatItem-yyQ": "Q yy",
	"timeFormat-full": "HH:mm:ss zzzz",
	"field-day-relative+0": "hoy",
	"field-day-relative+1": "mañana",
	"field-day-relative+2": "pasado mañana",
	"field-day-relative+3": "Dentro de tres días",
	"months-standAlone-abbr": [
		"ene",
		"feb",
		"mar",
		"abr",
		"may",
		"jun",
		"jul",
		"ago",
		"sep",
		"oct",
		"nov",
		"dic"
	],
	"quarters-format-abbr": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"quarters-standAlone-wide": [
		"1er trimestre",
		"2º trimestre",
		"3er trimestre",
		"4º trimestre"
	],
	"dateFormatItem-M": "L",
	"days-standAlone-wide": [
		"domingo",
		"lunes",
		"martes",
		"miércoles",
		"jueves",
		"viernes",
		"sábado"
	],
	"dateFormatItem-MMMMd": "d 'de' MMMM",
	"dateFormatItem-yyMMM": "MMM-yy",
	"timeFormat-medium": "HH:mm:ss",
	"dateFormatItem-Hm": "HH:mm",
	"quarters-standAlone-abbr": [
		"T1",
		"T2",
		"T3",
		"T4"
	],
	"eraAbbr": [
		"a.C.",
		"d.C."
	],
	"field-minute": "minuto",
	"field-dayperiod": "periodo del día",
	"days-standAlone-abbr": [
		"dom",
		"lun",
		"mar",
		"mié",
		"jue",
		"vie",
		"sáb"
	],
	"dateFormatItem-d": "d",
	"dateFormatItem-ms": "mm:ss",
	"field-day-relative+-1": "ayer",
	"dateFormatItem-h": "hh a",
	"field-day-relative+-2": "antes de ayer",
	"field-day-relative+-3": "Hace tres días",
	"dateFormatItem-MMMd": "d MMM",
	"dateFormatItem-MEd": "E, d/M",
	"dateFormatItem-yMMMM": "MMMM 'de' y",
	"field-day": "día",
	"days-format-wide": [
		"domingo",
		"lunes",
		"martes",
		"miércoles",
		"jueves",
		"viernes",
		"sábado"
	],
	"field-zone": "zona",
	"dateFormatItem-yyyyMM": "MM/yyyy",
	"dateFormatItem-y": "y",
	"months-standAlone-narrow": [
		"E",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dateFormatItem-yyMM": "MM/yy",
	"dateFormatItem-hm": "hh:mm a",
	"days-format-abbr": [
		"dom",
		"lun",
		"mar",
		"mié",
		"jue",
		"vie",
		"sáb"
	],
	"eraNames": [
		"antes de Cristo",
		"anno Dómini"
	],
	"days-format-narrow": [
		"D",
		"L",
		"M",
		"M",
		"J",
		"V",
		"S"
	],
	"field-month": "mes",
	"days-standAlone-narrow": [
		"D",
		"L",
		"M",
		"M",
		"J",
		"V",
		"S"
	],
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "a.m.",
	"dateFormat-short": "dd/MM/yy",
	"dateFormatItem-MMd": "d/MM",
	"field-second": "segundo",
	"dateFormatItem-yMMMEd": "EEE, d MMM y",
	"field-week": "semana",
	"dateFormat-medium": "dd/MM/yyyy",
	"dateFormatItem-Hms": "HH:mm:ss",
	"dateFormatItem-hms": "hh:mm:ss a"
}
//end v1.x content
);