//>>built
define(
//begin v1.x content
{
	"HKD_displayName": "Hongkongse dollar",
	"CHF_displayName": "Zwitserse franc",
	"CAD_displayName": "Canadese dollar",
	"CNY_displayName": "Chinese yuan renminbi",
	"AUD_displayName": "Australische dollar",
	"JPY_displayName": "Japanse yen",
	"USD_displayName": "Amerikaanse dollar",
	"GBP_displayName": "Brits pond sterling",
	"EUR_displayName": "Euro"
}
//end v1.x content
);