
.dj_ie .claro .dijitTimePickerRtl .dijitTimePickerTickHover,
.dj_ie .claro .dijitTimePickerRtl .dijitTimePickerMarkerHover,
.dj_ie .claro .dijitTimePickerRtl .dijitTimePickerMarkerSelected,
.dj_ie .claro .dijitTimePickerRtl .dijitTimePickerTickSelected {
  margin-left: -6px;
  margin-right: -8px;
  width: 114%;
}
