//>>built
define(
//begin v1.x content
({
// local representation of all CSS3 named colors, companion to dojo.colors.  To be used where descriptive information
// is required for each color, such as a palette widget, and not for specifying color programatically.

//Note: due to the SVG 1.0 spec additions, some of these are alternate spellings for the same color e.g. gray vs. gray.
//TODO: should we be using unique rgb values as keys instead and avoid these duplicates, or rely on the caller to do the reverse mapping?
aliceblue: "babyblå",
antiquewhite: "antikhvid",
aqua: "akvablå",
aquamarine: "akvamarin",
azure: "azurblå",
beige: "beige",
bisque: "gulgrå",
black: "sort",
blanchedalmond: "blanceret mandel",
blue: "blå",
blueviolet: "blåviolet",
brown: "brun",
burlywood: "tobak",
cadetblue: "kadetblå",
chartreuse: "chartreuse",
chocolate: "rust",
coral: "koralrød",
cornflowerblue: "kornblomstblå",
cornsilk: "majs",
crimson: "blodrød",
cyan: "cyan",
darkblue: "mørkeblå",
darkcyan: "mørk cyan",
darkgoldenrod: "mørk gyldenris",
darkgray: "mørkegrå",
darkgreen: "mørkegrøn",
darkgrey: "mørkegrå", // same as darkgray
darkkhaki: "mørk khaki",
darkmagenta: "mørk magenta",
darkolivegreen: "mørk olivengrøn",
darkorange: "mørk orange",
darkorchid: "mørk orkide",
darkred: "mørkerød",
darksalmon: "mørk laksefarvet",
darkseagreen: "mørk havgrøn",
darkslateblue: "mørk skiferblå",
darkslategray: "mørk skifergrå",
darkslategrey: "mørk skifergrå", // same as darkslategray
darkturquoise: "mørk turkis",
darkviolet: "mørkelilla",
deeppink: "dyb pink",
deepskyblue: "dyb himmelblå",
dimgray: "svag grå",
dimgrey: "svag grå", // same as dimgray
dodgerblue: "dodgerblå",
firebrick: "chamottesten",
floralwhite: "blomsterhvid",
forestgreen: "skovgrøn",
fuchsia: "lyslilla",
gainsboro: "gainsboro",
ghostwhite: "spøgelseshvid",
gold: "guld",
goldenrod: "gyldenris",
gray: "grå",
green: "grøn",
greenyellow: "grøngul",
grey: "grå", // same as gray
honeydew: "honningdug",
hotpink: "mørk rosa",
indianred: "lys rødbrun",
indigo: "indigo",
ivory: "elfenben",
khaki: "khaki",
lavender: "lysviolet",
lavenderblush: "lavendelrød",
lawngreen: "græsgrøn",
lemonchiffon: "citronfromage",
lightblue: "lyseblå",
lightcoral: "lys koralrød",
lightcyan: "lys cyan",
lightgoldenrodyellow: "lys gyldenrisgul",
lightgray: "lysegrå",
lightgreen: "lysegrøn",
lightgrey: "lysegrå", // same as lightgray
lightpink: "lys pink",
lightsalmon: "lys laksefarvet",
lightseagreen: "lys havgrøn",
lightskyblue: "lys himmelblå",
lightslategray: "lys skifergrå",
lightslategrey: "lys skifergrå", // same as lightslategray
lightsteelblue: "lys stålblå",
lightyellow: "lysegul",
lime: "lime",
limegreen: "limegrøn",
linen: "lærred",
magenta: "magenta",
maroon: "rødbrun",
mediumaquamarine: "mellem akvamarin",
mediumblue: "mellemblå",
mediumorchid: "mellem orkide",
mediumpurple: "mellemlilla",
mediumseagreen: "mellemhavgrøn",
mediumslateblue: "mellemskiferblå",
mediumspringgreen: "mellemforårsgrøn",
mediumturquoise: "mellemturkis",
mediumvioletred: "mellemviolet",
midnightblue: "midnatsblå",
mintcream: "pebermyntecreme",
mistyrose: "blegrosa",
moccasin: "fruesko",
navajowhite: "navajo-hvid",
navy: "marineblå",
oldlace: "kniplingshvid",
olive: "olivengrøn",
olivedrab: "brungrøn",
orange: "orange",
orangered: "orangerød",
orchid: "orkide",
palegoldenrod: "bleg gyldenris",
palegreen: "bleggrøn",
paleturquoise: "bleg turkis",
palevioletred: "blegviolet",
papayawhip: "papaya",
peachpuff: "fersken",
peru: "peru",
pink: "pink",
plum: "blomme",
powderblue: "pudderblå",
purple: "lilla",
red: "rød",
rosybrown: "rosabrun",
royalblue: "kongeblå",
saddlebrown: "saddelbrun",
salmon: "laksefarvet",
sandybrown: "sandbrun",
seagreen: "havgrøn",
seashell: "muslingeskal",
sienna: "sienna",
silver: "sølv",
skyblue: "himmelblå",
slateblue: "skiferblå",
slategray: "skifergrå",
slategrey: "skifergrå", // same as slategray
snow: "sne",
springgreen: "forårsgrøn",
steelblue: "metalblå",
tan: "tan",
teal: "blågrøn",
thistle: "tidsel",
tomato: "tomat",
turquoise: "turkis",
violet: "lilla",
wheat: "korngul",
white: "hvid",
whitesmoke: "hvid røg",
yellow: "gul",
yellowgreen: "gulgrøn"
})
//end v1.x content
);
