//>>built
define(
//begin v1.x content
({
// local representation of all CSS3 named colors, companion to dojo.colors.  To be used where descriptive information
// is required for each color, such as a palette widget, and not for specifying color programatically.

//Note: due to the SVG 1.0 spec additions, some of these are alternate spellings for the same color e.g. gray vs. gray.
//TODO: should we be using unique rgb values as keys instead and avoid these duplicates, or rely on the caller to do the reverse mapping?
aliceblue: "alice blue modra",
antiquewhite: "antično bela",
aqua: "akva",
aquamarine: "akvamarin",
azure: "azurno modra",
beige: "bež",
bisque: "porcelanasta",
black: "črna",
blanchedalmond: "obledelo mandljeva",
blue: "modra",
blueviolet: "modro vijolična",
brown: "rjava",
burlywood: "peščeno sivo-rjava",
cadetblue: "kadetsko modra",
chartreuse: "chartreuse",
chocolate: "čokoladna",
coral: "koralna",
cornflowerblue: "plavičasto modra",
cornsilk: "koruzna",
crimson: "karminasta",
cyan: "cijan",
darkblue: "temno modra",
darkcyan: "temno cijan",
darkgoldenrod: "temna zlata rozga",
darkgray: "temno siva",
darkgreen: "temno zelena",
darkgrey: "temno siva", // same as darkgray
darkkhaki: "temno kaki",
darkmagenta: "temna magenta",
darkolivegreen: "temna olivno zelena",
darkorange: "temno oranžna",
darkorchid: "temno orhidejasta",
darkred: "temno rdeča",
darksalmon: "temno lososova",
darkseagreen: "temno morsko zelena",
darkslateblue: "temno skrilasto modra",
darkslategray: "temno skrilasto siva",
darkslategrey: "temno skrilasto siva", // same as darkslategray
darkturquoise: "temno turkizna",
darkviolet: "temno vijolična",
deeppink: "temno rožnata",
deepskyblue: "temno nebeško modra",
dimgray: "pepelnato siva",
dimgrey: "pepelnato siva", // same as dimgray
dodgerblue: "dodgersko modra",
firebrick: "opečnata",
floralwhite: "cvetno bela",
forestgreen: "gozdno zelena",
fuchsia: "fuksija",
gainsboro: "gainsboro",
ghostwhite: "senčnato bela",
gold: "zlata",
goldenrod: "zlata rozga",
gray: "siva",
green: "zelena",
greenyellow: "zeleno-rumena",
grey: "siva", // same as gray
honeydew: "medena rosa",
hotpink: "kričeče rožnata",
indianred: "indijansko rdeča",
indigo: "indigo",
ivory: "slonokoščena",
khaki: "kaki",
lavender: "sivka",
lavenderblush: "rožnato sivka",
lawngreen: "travniško zelena",
lemonchiffon: "limonast šifon",
lightblue: "svetlo modra",
lightcoral: "svetlo koralna",
lightcyan: "svetlo cijan",
lightgoldenrodyellow: "svetlo rumena zlata rozga",
lightgray: "svetlo siva",
lightgreen: "svetlo zelena",
lightgrey: "svetlo siva", // same as lightgray
lightpink: "svetlo rožnata",
lightsalmon: "svetlo lososova",
lightseagreen: "svetlo morsko zelena",
lightskyblue: "svetlo nebeško modra",
lightslategray: "svetlo skrilasto siva",
lightslategrey: "svetlo skrilasto siva", // same as lightslategray
lightsteelblue: "svetlo kovinsko modra",
lightyellow: "svetlo rumena",
lime: "limetasta",
limegreen: "apneno zelena",
linen: "lanena",
magenta: "magenta",
maroon: "kostanjeva",
mediumaquamarine: "srednji akvamarin",
mediumblue: "srednje modra",
mediumorchid: "srednje orhidejasta",
mediumpurple: "srednje škrlatna",
mediumseagreen: "srednje morsko zelena",
mediumslateblue: "srednje skrilasto modra",
mediumspringgreen: "srednje pomladno zelena",
mediumturquoise: "srednje turkizna",
mediumvioletred: "srednje vijolično rdeča",
midnightblue: "polnočno modra",
mintcream: "metina krema",
mistyrose: "megleno rožnata",
moccasin: "mokasinasta",
navajowhite: "navajo bela",
navy: "mornarska",
oldlace: "stara čipka",
olive: "olivna",
olivedrab: "umazano olivna",
orange: "oranžna",
orangered: "oranžno-rdeča",
orchid: "orhidejasta",
palegoldenrod: "bleda zlata rozga",
palegreen: "bledo zelena",
paleturquoise: "bledo turkizna",
palevioletred: "bledo vijolično-rdeča",
papayawhip: "papaja",
peachpuff: "breskova",
peru: "perujska",
pink: "rožnata",
plum: "slivova",
powderblue: "kobaltovo modra",
purple: "škrlatna",
red: "rdeča",
rosybrown: "rožnato rjava",
royalblue: "kraljevsko modra",
saddlebrown: "sedlasto rjava",
salmon: "lososova",
sandybrown: "peščeno rjava",
seagreen: "morsko zelena",
seashell: "morska lupina",
sienna: "sienna",
silver: "srebrna",
skyblue: "nebeško modra",
slateblue: "skrilasto modra",
slategray: "skrilasto siva",
slategrey: "skrilasto siva", // same as slategray
snow: "snežena",
springgreen: "pomladno zelena",
steelblue: "kovinsko modra",
tan: "rumeno-rjava",
teal: "modrozelena",
thistle: "osatna",
tomato: "paradižnikova",
turquoise: "turkizna",
violet: "vijolična",
wheat: "pšenična",
white: "bela",
whitesmoke: "megleno bela",
yellow: "rumena",
yellowgreen: "rumeno-zelena"
})
//end v1.x content
);
