//>>built
define(
//begin v1.x content
({
	"lightsteelblue":"açıq metal mavi",
	"orangered" :"narıncı qırmızı",
	"midnightblue" :"gecə mavisi",
	"cadetblue" :"dəniz mavisi" ,
	"seashell" :"dəniz səthi",
	"slategrey" :"boz şifer rəngi",
	"coral" :"mərcan",
	"darkturquoise" :"tünd firuzəyi",
	"antiquewhite" :"antik ağ",
	"mediumspringgreen" :"orta bahar yaşılı",
	"salmon" :"somon",
	"darkgrey" :"tünd boz",
	"ivory" :"fil dişi",
	"greenyellow" :"yaşıl-sarı",
	"mistyrose" :"gül qurusu",
	"lightsalmon" :"açıq somon",
	"silver" :"gümüşü",
	"dimgrey" :"açıq boz",
	"orange" :"narıncı",
	"white" :"ağ",
	"navajowhite" :"navajo ağı",
	"royalblue" :"parlaq tünd mavi" ,
	"deeppink" :"tünd çəhrayı",
	"lime" :"lomon yaşılı",
	"oldlace" :"köhnə krujeva",
	"chartreuse" :"chartreuse",
	"darkcyan" :"tünd firuzəyi",
	"yellow" :"sarı",
	"linen" :"kətan",
	"olive" :"zeytun",
	"gold" :"qızıl",
	"lawngreen" :"çəmən yaşılı",
	"lightyellow" :"açıq sarı",
	"tan" :"günəş yanığı",
	"darkviolet" :"tünd bənövşəyi",
	"lightslategrey" :"tünd şifer bozu",
	"grey" :"boz",
	"darkkhaki" :"tünd haki",
	"green" :"yaşıl",
	"deepskyblue" :"tünd səma mavisi",
	"aqua" :"dəniz mavisi",
	"sienna" :"tünd qəhvəyi",
	"mintcream" :"nanəli krem",
	"rosybrown" :"çəhrayımsı qəhvəyi",
	"mediumslateblue" :"orta şıfer bozu",
	"magenta" :"magenta",
	"lightseagreen" :"açıq dəniz yaşılı",
	"cyan" :"firuzəyi",
	"olivedrab" :"əsgər yaşılı",
	"darkgoldenrod" :"tünd sarı",
	"slateblue" :"şifer mavisi",
	"mediumaquamarine" :"orta akvamarin",
	"lavender" :"lavanta",
	"mediumseagreen" :"orta dəniz yaşılı",
	"maroon" :"tünd qırmızı",
	"darkslategray" :"tünd şifer bozu",
	"mediumturquoise" :"orta firuzəyi",
	"ghostwhite" :"ala",
	"darkblue" :"tünd mavi",
	"mediumvioletred" :"orta bənövşəyi-qırmızı",
	"brown" :"qəhvəyi",
	"lightgray" :"açıq boz",
	"sandybrown" :"qum rəngi",
	"pink" :"çəhrayı",
	"firebrick" :"yanmış kərpic rəngi",
	"indigo" :"indigo",
	"snow" :"qar",
	"darkorchid" :"tünd orkide",
	"turquoise" :"firuzəyi",
	"chocolate" :"şokolad",
	"springgreen" :"bahar yaşılı",
	"moccasin" :"mokosen",
	"navy" :"tünd göy",
	"lemonchiffon" :"limon rəngi",
	"teal" :"teal mavi",
	"floralwhite" :"çiçək ağı",
	"cornflowerblue" :"peyğəmbər çiçək mavisi",
	"paleturquoise" :"solğun firuzəyi",
	"purple" :"tünd qırmızı",
	"gainsboro" :"gainsboro",
	"plum" :"gavalı",
	"red" :"qırmızı",
	"blue" :"göy",
	"forestgreen" :"tünd dəniz yaşılı",
	"darkgreen" :"tünd yaşıl",
	"honeydew" :"bal saqqızı",
	"darkseagreen" :"tünd dəniz yaşılı",
	"lightcoral" :"açıq mərcan",
	"palevioletred" :"solğun bənövşəyi-qırmızı",
	"mediumpurple" :"orta tünd qırmızı",
	"saddlebrown" :"açıq qəhvəyi",
	"darkmagenta" :"tünd magenta",
	"thistle" :"dəvə tikanı",
	"whitesmoke" :"ağ duman",
	"wheat" :"buğdayı",
	"violet" :"bənövşəyi",
	"lightskyblue" :"açıq səma mavisi" ,
	"goldenrod" :"sapsarı",
	"mediumblue" :"orta göy",
	"skyblue" :"göy mavisi",
	"crimson" :"crimson",
	"darksalmon" :"tünd somon",
	"darkred" :"tünd qırmızı",
	"darkslategrey" :"tünd şifer bozu",
	"peru" :"peru",
	"lightgrey" :"açıq boz",
	"lightgoldenrodyellow" :"açıq qızılı",
	"blanchedalmond" :"solğun badamı",
	"aliceblue" :"alice mavisi",
	"bisque" :"biskvit",
	"slategray" :"şifer bozu",
	"palegoldenrod" :"açıq qızılı",
	"darkorange" :"tünd narıncı",
	"aquamarine" :"akvamarin",
	"lightgreen" :"açıq yaşıl",
	"burlywood" :"sarımsı qəhvə rəngi",
	"dodgerblue" :"toz mavisi",
	"darkgray" :"tünd boz",
	"lightcyan" :"açıq firuzəyi",
	"powderblue" :"pudra mavisi",
	"blueviolet" :"mavi-bənövşəyi",
	"orchid" :"orkide",
	"dimgray" :"solğun boz",
	"beige" :"bej",
	"fuchsia" :"fuşya",
	"lavenderblush" :"lavanta tünd qırmızısı",
	"hotpink" :"tünd çəhrayı",
	"steelblue" :"metal mavisi",
	"tomato" :"pomidor",
	"lightpink" :"açıq çəhrayı",
	"limegreen" :"əhəng yaşılı",
	"indianred" :"qızıldərili qırmızısı",
	"papayawhip" :"papaya qamçısı",
	"lightslategray" :"açıq şifer bozu",
	"gray" :"boz",
	"mediumorchid" :"orta orkide",
	"cornsilk" :"qarğıdalı rəngi",
	"black" :"qara",
	"seagreen" :"dəniz yaşılı",
	"darkslateblue" :"tünd şifer bozu",
	"khaki" :"haki",
	"lightblue" :"açıq mavi",
	"palegreen" :"solğun yaşıl",
	"azure" :"azur mavisi",
	"peachpuff" :"açıq şaftalı",
	"darkolivegreen" :"tünd zeytun yaşılı",
	"yellowgreen" :"sarı-yaşıl"
})
//end v1.x content
);