@import "../variables";

.claro .dijitTabContainerTop-tabs .dijitTabRtl,
.claro .dijitTabContainerBottom-tabs .dijitTabRtl {
	margin-right: 0;
	margin-left: 1px;
}
.claro .dijitTabRtl {
	-moz-box-orient:horizontal;
	text-align: right;
}
.dj_ie7 .claro .dijitTabRtl .dijitTabContent {
	display: block;
	left: 0;
}
.dj_ie7 .claro .dijitTabContainerTop-tabs .dijitTabRtl,
.dj_ie7 .claro .dijitTabContainerTop-tabs .dijitTabCheckedRtl {
	top: 1px;
}

// Note that .tabStripButtonRtl .dijitButtonContents needed for IE8 quirks but breaks IE6 quirks
.dj_ie7 .claro .tabStripButtonRtl .dijitButtonContents,
.dj_ie8 .claro .tabStripButtonRtl .dijitButtonContents,
.dj_ie6 .claro .dijitTabContainerTop-tabs .dijitTabRtl .dijitTabInnerDiv,
.dj_ie6 .claro .dijitTabContainerBottom-tabs .dijitTabRtl .dijitTabInnerDiv,
.dj_ie7 .claro .dijitTabContainerTop-tabs .dijitTabRtl .dijitTabInnerDiv,
.dj_ie7 .claro .dijitTabContainerBottom-tabs .dijitTabRtl .dijitTabInnerDiv,
.dj_iequirks .claro .dijitTabContainerTop-tabs .dijitTabRtl .dijitTabInnerDiv,
.dj_iequirks .claro .dijitTabContainerBottom-tabs .dijitTabRtl .dijitTabInnerDiv {
	float:left;
}
.dj_ie6 .claro .dijitTabRtl .tabLabel,
.dj_ie6 .claro .dijitTabContainerRight-tabs .dijitTabRtl,
.dj_ie6 .claro .dijitTabContainerLeft-tabs .dijitTabRtl,
.dj_ie7 .claro .dijitTabContainerRight-tabs .dijitTabRtl .dijitTabInnerDiv,
.dj_ie7 .claro .dijitTabContainerLeft-tabs .dijitTabRtl .dijitTabInnerDiv {
	/*Make the .tablabel have width in ie6 with haslayout property, fix Close icon position bug*/
	zoom:1;
}
.dj_ie6 .claro .dijitTabRtl .dijitTabCloseButton,
.dj_ie7 .claro .dijitTabRtl .dijitTabCloseButton,
.dj_iequirks .claro .dijitTabRtl .dijitTabCloseButton {
	margin-right:5px;
}

.dj_ie6 .claro .dijitTabContainerRightRtl .dijitTabContainerRight-tabs,
.dj_ie6 .claro .dijitTabContainerLeftRtl .dijitTabContainerLeft-tabs {
	width:1%;
}

.dj_ie6 .dijitTabContainerTopStrip,
.dj_ie6 .dijitTabContainerBottomStrip {
	position:absolute;
}
.dj_iequirks .claro .dijitTabContainerTopRtl .dijitTabContainerTopStrip {
	padding-top: 10px;
}
.dj_ie7 .claro .dijitTabContainerRight-tabs .dijitTabRtlChecked .dijitTabInnerDiv {
	background-position:-341px -179px;
}

.dj_ie6 .dijitTabContainerTopRtl .dijitTabStripIcon,
.dj_ie6 .dijitTabContainerBottomRtl .dijitTabStripIcon {
   position: relative;
}


.dj_ie6-rtl .claro .dijitTabContainerTop-tabs {	
	/* this strange rule prevents IE6 bug in themeTester.html?dir=rtl upon closing
	 * "Closable" tab, where the other tabs disappear
	 */
	padding-left: 3px;
}

.dj_iequirks-rtl .claro .dijitTabListWrapper {
	/* this strange rule prevents IE6 bug in themeTesterQuirk.html?dir=rtl upon closing
	 * "Closable" tab, where the other tabs disappear
	 */
	border-left: 1px solid @pane-background-color;
	border-right: 1px solid @pane-background-color;
}
