//>>built
define(
//begin v1.x content
{
	"decimalFormat": "#,##0.###",
	"group": ".",
	"scientificFormat": "#E0",
	"percentFormat": "#,##0%",
	"currencyFormat": "¤ #,##0.00",
	"decimal": ","
}
//end v1.x content
);