//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "yyyy/M",
	"dateFormatItem-yQ": "y年QQQ",
	"dayPeriods-format-wide-pm": "下午",
	"dateFormatItem-MMMEd": "MMMd日E",
	"dateTimeFormat-full": "{1}{0}",
	"dateFormatItem-hms": "ah:mm:ss",
	"dateFormatItem-yQQQ": "y年QQQ",
	"dateFormatItem-MMdd": "MM/dd",
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "上午",
	"dateFormatItem-y": "y年",
	"timeFormat-full": "zzzzah時mm分ss秒",
	"dateFormatItem-yyyy": "Gy年",
	"dateFormatItem-Ed": "d日(E)",
	"dateFormatItem-yMMM": "y年M月",
	"days-standAlone-narrow": [
		"日",
		"一",
		"二",
		"三",
		"四",
		"五",
		"六"
	],
	"dateFormat-long": "Gy年M月d日",
	"timeFormat-medium": "ah:mm:ss",
	"dateFormatItem-Hm": "H:mm",
	"dateFormat-medium": "Gy/M/d",
	"dateFormatItem-Hms": "H:mm:ss",
	"dateFormatItem-ms": "mm:ss",
	"dateTimeFormat-long": "{1}{0}",
	"dateFormatItem-yyyyMd": "Gy/M/d",
	"dateFormatItem-yyyyMMMd": "Gy年M月d日",
	"dateFormatItem-MMMd": "MMMd日",
	"timeFormat-long": "zah時mm分ss秒",
	"timeFormat-short": "ah:mm",
	"dateFormatItem-H": "H時",
	"quarters-format-abbr": [
		"1季",
		"2季",
		"3季",
		"4季"
	],
	"days-format-abbr": [
		"週日",
		"週一",
		"週二",
		"週三",
		"週四",
		"週五",
		"週六"
	],
	"dateFormatItem-MMMMdd": "MMMMdd日",
	"dateFormatItem-M": "M月",
	"dateFormatItem-MEd": "M/d（E）",
	"dateFormatItem-hm": "ah:mm",
	"dateFormat-short": "Gy/M/d",
	"dateFormatItem-yyyyM": "Gy/M",
	"dateFormatItem-yMMMEd": "y年M月d日EEE",
	"dateFormat-full": "Gy年M月d日EEEE",
	"dateFormatItem-Md": "M/d",
	"dateFormatItem-yyyyQ": "Gy年QQQ",
	"dateFormatItem-yMEd": "yyyy/M/d（EEE）",
	"dateFormatItem-yyyyMMM": "Gy年M月",
	"dateFormatItem-d": "d日",
	"quarters-format-wide": [
		"第1季",
		"第2季",
		"第3季",
		"第4季"
	],
	"days-format-wide": [
		"星期日",
		"星期一",
		"星期二",
		"星期三",
		"星期四",
		"星期五",
		"星期六"
	],
	"dateFormatItem-h": "ah時"
}
//end v1.x content
);