//>>built
define(
//begin v1.x content
({
	fontSize: "Grootte",
	fontName: "Lettertype",
	formatBlock: "Opmaak",

	serif: "serif",
	"sans-serif": "sans-serif",
	monospace: "monospace",
	cursive: "cursief",
	fantasy: "fantasy",

	noFormat: "Geen",
	p: "Alinea",
	h1: "Kop",
	h2: "Subkop",
	h3: "Sub-subkop",
	pre: "Vooraf opgemaakt",

	1: "xx-klein",
	2: "x-klein",
	3: "klein",
	4: "gemiddeld",
	5: "groot",
	6: "x-groot",
	7: "xx-groot"
})
//end v1.x content
);
