//>>built
define(
//begin v1.x content
{
	"dateFormatItem-yM": "Gy/M",
	"dateFormatItem-yQ": "Gy年QQQ",
	"dayPeriods-format-wide-pm": "下午",
	"dateFormatItem-MMMEd": "MMMd日E",
	"dateTimeFormat-full": "{1}{0}",
	"dateFormatItem-hms": "ah:mm:ss",
	"dateFormatItem-yQQQ": "Gy年QQQ",
	"dateFormatItem-MMdd": "MM/dd",
	"dateFormatItem-MMM": "LLL",
	"dayPeriods-format-wide-am": "上午",
	"dateFormatItem-y": "Gy年",
	"timeFormat-full": "zzzzah時mm分ss秒",
	"dateFormatItem-yyyy": "y年",
	"months-standAlone-abbr": [
		"一月",
		"二月",
		"三月",
		"四月",
		"五月",
		"六月",
		"七月",
		"八月",
		"九月",
		"十月",
		"十一月",
		"十二月"
	],
	"dateFormatItem-Ed": "d日(E)",
	"dateFormatItem-yMMM": "Gy年M月",
	"days-standAlone-narrow": [
		"日",
		"一",
		"二",
		"三",
		"四",
		"五",
		"六"
	],
	"dateFormatItem-yyyyMMMM": "y年MMMM",
	"dateFormat-long": "Gy年M月d日",
	"timeFormat-medium": "ah:mm:ss",
	"dateFormatItem-EEEd": "d EEE",
	"dateFormatItem-Hm": "H:mm",
	"dateFormatItem-yyMM": "Gyy/MM",
	"dateFormat-medium": "Gy/M/d",
	"dateFormatItem-Hms": "H:mm:ss",
	"dateFormatItem-yyMMM": "Gyy年MMM",
	"dateFormatItem-yMd": "Gy/M/d",
	"dateFormatItem-ms": "mm:ss",
	"dateTimeFormat-long": "{1}{0}",
	"months-standAlone-wide": [
		"一月",
		"二月",
		"三月",
		"四月",
		"五月",
		"六月",
		"七月",
		"八月",
		"九月",
		"十月",
		"十一月",
		"十二月"
	],
	"dateFormatItem-MMMd": "MMMd日",
	"dateFormatItem-yyQ": "Gyy年第Q季度",
	"timeFormat-long": "zah時mm分ss秒",
	"months-format-abbr": [
		"1月",
		"2月",
		"3月",
		"4月",
		"5月",
		"6月",
		"7月",
		"8月",
		"9月",
		"10月",
		"11月",
		"12月"
	],
	"dateFormatItem-H": "H時",
	"timeFormat-short": "ah:mm",
	"quarters-format-abbr": [
		"1季",
		"2季",
		"3季",
		"4季"
	],
	"dateFormatItem-MMMMdd": "MMMMdd日",
	"days-format-abbr": [
		"週日",
		"週一",
		"週二",
		"週三",
		"週四",
		"週五",
		"週六"
	],
	"dateFormatItem-M": "M月",
	"dateFormatItem-yMMMd": "Gy年MMMd日",
	"dateFormatItem-MEd": "M/d（E）",
	"dateFormatItem-hm": "ah:mm",
	"dateFormat-short": "Gy/M/d",
	"dateFormatItem-yyyyM": "y年M月",
	"dateFormatItem-yMMMEd": "Gy年M月d日EEE",
	"dateFormat-full": "Gy年M月d日EEEE",
	"dateFormatItem-Md": "M/d",
	"dateFormatItem-yMEd": "Gy/M/d（EEE）",
	"months-format-wide": [
		"1月",
		"2月",
		"3月",
		"4月",
		"5月",
		"6月",
		"7月",
		"8月",
		"9月",
		"10月",
		"11月",
		"12月"
	],
	"dateFormatItem-d": "d日",
	"quarters-format-wide": [
		"第1季",
		"第2季",
		"第3季",
		"第4季"
	],
	"days-format-wide": [
		"星期日",
		"星期一",
		"星期二",
		"星期三",
		"星期四",
		"星期五",
		"星期六"
	],
	"dateFormatItem-h": "ah時"
}
//end v1.x content
);