
.claro .dijitButtonNode {
  
  -webkit-transition-property: background-color;
  -moz-transition-property: background-color;
  transition-property: background-color;
  -webkit-transition-duration: 0.3s;
  -moz-transition-duration: 0.3s;
  transition-duration: 0.3s;
}
.claro .dijitButton .dijitButtonNode,
.claro .dijitDropDownButton .dijitButtonNode,
.claro .dijitComboButton .dijitButtonNode,
.claro .dijitToggleButton .dijitButtonNode {
  
  border: 1px solid #759dc0;
  padding: 2px 4px 4px 4px;
  background-image: url("../form/images/button.png");
  background-position: center top;
  background-repeat: repeat-x;
  background-color: #e5f2fe;
  color: #000000;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: 0 1px 1px rgba(0, 0, 0, 0.15);
  -moz-box-shadow: 0 1px 1px rgba(0, 0, 0, 0.15);
  box-shadow: 0 1px 1px rgba(0, 0, 0, 0.15);
}
.claro .dijitComboButton .dijitArrowButton {
  border-left-width: 0;
  padding: 4px 2px 4px 2px;
  
}
.claro .dijitArrowButtonInner {
  width: 15px;
  height: 15px;
  margin: 0 auto;
  background-image: url("../form/images/buttonArrows.png");
  background-repeat: no-repeat;
  background-position: -51px 53%;
}
.claro .dijitLeftArrowButton .dijitArrowButtonInner {
  background-position: -77px 53%;
}
.claro .dijitRightArrowButton .dijitArrowButtonInner {
  background-position: -26px 53%;
}
.claro .dijitUpArrowButton .dijitArrowButtonInner {
  background-position: 0 53%;
}
.claro .dijitDisabled .dijitArrowButtonInner {
  background-position: -151px 53%;
}
.claro .dijitDisabled .dijitLeftArrowButton .dijitArrowButtonInner {
  background-position: -177px 53%;
}
.claro .dijitDisabled .dijitRightArrowButton .dijitArrowButtonInner {
  background-position: -126px 53%;
}
.claro .dijitDisabled .dijitUpArrowButton .dijitArrowButtonInner {
  background-position: -100px 53%;
}
.claro .dijitButtonText {
  padding: 0 0.3em;
  text-align: center;
}
.claro .dijitButtonHover .dijitButtonNode,
.claro .dijitDropDownButtonHover .dijitButtonNode,
.claro .dijitComboButton .dijitButtonNodeHover,
.claro .dijitComboButton .dijitDownArrowButtonHover,
.claro .dijitToggleButtonHover .dijitButtonNode {
  background-color: #abd6ff;
  color: #000000;
  -webkit-transition-duration: 0.2s;
  -moz-transition-duration: 0.2s;
  transition-duration: 0.2s;
}
.claro .dijitButtonActive .dijitButtonNode,
.claro .dijitDropDownButtonActive .dijitButtonNode,
.claro .dijitComboButtonActive .dijitButtonNode,
.claro .dijitToggleButtonActive .dijitButtonNode,
.claro .dijitStackController .dijitToggleButtonChecked .dijitButtonNode {
  background-color: #abd6ff;
  -webkit-box-shadow: 0 0 0 rgba(0, 0, 0, 0);
  -moz-box-shadow: 0 0 0 rgba(0, 0, 0, 0);
  box-shadow: 0 0 0 rgba(0, 0, 0, 0);
  -webkit-transition-duration: 0.1s;
  -moz-transition-duration: 0.1s;
  transition-duration: 0.1s;
}
.claro .dijitButtonDisabled,
.claro .dijitDropDownButtonDisabled,
.claro .dijitComboButtonDisabled,
.claro .dijitToggleButtonDisabled {
  background-image: none;
  outline: none;
}
.claro .dijitButtonDisabled .dijitButtonNode,
.claro .dijitDropDownButtonDisabled .dijitButtonNode,
.claro .dijitComboButtonDisabled .dijitButtonNode,
.claro .dijitToggleButtonDisabled .dijitButtonNode {
  background-position: 0 -149px;
  background-color: #efefef;
  border: solid 1px #d3d3d3;
  color: #818181;
  -webkit-box-shadow: 0 0 0 rgba(0, 0, 0, 0);
  -moz-box-shadow: 0 0 0 rgba(0, 0, 0, 0);
  box-shadow: 0 0 0 rgba(0, 0, 0, 0);
}
.claro .dijitComboButtonDisabled .dijitArrowButton {
  border-left-width: 0;
}
.claro table.dijitComboButton {
  border-collapse: separate;
  
}
.dj_ie6 .claro .dijitButtonNode {
  background-image: none;
}
.claro .dijitComboButton .dijitStretch {
  -moz-border-radius: 4px 0 0 4px;
  border-radius: 4px 0 0 4px;
}
.claro .dijitComboButton .dijitArrowButton {
  -moz-border-radius: 0 4px 4px 0;
  border-radius: 0 4px 4px 0;
}
