
.tundra .dijitTimePickerTick,
.tundra .dijitTimePickerMarker {
	border-color: #ccc;
}
.tundra .dijitTimePickerTick {
	color:white;
}
.tundra .dijitTimePickerMarker {
	background:#d3d3d3 url("images/titleBar.png") repeat-x top;
	color:#293a4b;
	font-weight: bold;
}
.tundra .dijitTimePickerItemSelected {
	color: black;
	background: #bbc4d0 none;
}
.tundra .dijitTimePickerItemHover {
	background: #60a1ea none;
	color:white;
}
.tundra .dijitTimePickerItemHover,
.tundra .dijitTimePickerItemSelected {
	position: relative;
	z-index: 10;
}
.tundra .dijitTimePickerTick .dijitTimePickerItemInner {
	font-size:0.4em;
}
.tundra .dijitTimePickerItemHover .dijitTimePickerItemInner,
.tundra .dijitTimePickerItemSelected .dijitTimePickerItemInner {
	font-size:1em;
}
.tundra .dijitTimePickerMarkerHover {
	border-top: 1px solid #ccc;
}
.tundra .dijitTimePickerTickHover,
.tundra .dijitTimePickerTickSelected {
	margin-top:-0.3em;
	margin-bottom:-0.3em;
	border-bottom: none;
}
