//>>built
define(
//begin v1.x content
({
// local representation of all CSS3 named colors, companion to dojo.colors.  To be used where descriptive information
// is required for each color, such as a palette widget, and not for specifying color programatically.

//Note: due to the SVG 1.0 spec additions, some of these are alternate spellings for the same color e.g. gray vs. gray.
//TODO: should we be using unique rgb values as keys instead and avoid these duplicates, or rely on the caller to do the reverse mapping?
aliceblue: "Alice kék",
antiquewhite: "antik fehér",
aqua: "vízszín",
aquamarine: "akvamarin",
azure: "azúrkék",
beige: "bézs",
bisque: "porcelán",
black: "fekete",
blanchedalmond: "hámozott mandula",
blue: "kék",
blueviolet: "ibolyakék",
brown: "barna",
burlywood: "nyersfa",
cadetblue: "kadétkék",
chartreuse: "chartreuse",
chocolate: "csokoládé",
coral: "korall",
cornflowerblue: "búzavirágkék",
cornsilk: "kukoricahaj",
crimson: "karmazsinvörös",
cyan: "ciánkék",
darkblue: "sötétkék",
darkcyan: "sötét ciánkék",
darkgoldenrod: "sötét aranyvessző",
darkgray: "sötétszürke",
darkgreen: "sötétzöld",
darkgrey: "sötétszürke", // same as darkgray
darkkhaki: "sötét khakiszín",
darkmagenta: "sötétbíbor",
darkolivegreen: "sötét olajzöld",
darkorange: "sötét narancssárga",
darkorchid: "sötét orchidea",
darkred: "sötétvörös",
darksalmon: "sötét lazacszín",
darkseagreen: "sötét tengerzöld",
darkslateblue: "sötét palakék",
darkslategray: "sötét palaszürke",
darkslategrey: "sötét palaszürke", // same as darkslategray
darkturquoise: "sötét türkizkék",
darkviolet: "sötét ibolyaszín",
deeppink: "sötétrózsaszín",
deepskyblue: "sötét égszínkék",
dimgray: "halványszürke",
dimgrey: "halványszürke", // same as dimgray
dodgerblue: "dodger kék",
firebrick: "téglavörös",
floralwhite: "virágfehér",
forestgreen: "erdőzöld",
fuchsia: "fukszia",
gainsboro: "gainsboro",
ghostwhite: "szellemfehér",
gold: "arany",
goldenrod: "aranyvessző",
gray: "szürke",
green: "zöld",
greenyellow: "zöldessárga",
grey: "szürke", // same as gray
honeydew: "mézharmat",
hotpink: "meleg rózsaszín",
indianred: "indiánvörös",
indigo: "indigó",
ivory: "elefántcsont",
khaki: "khakiszín",
lavender: "levendula",
lavenderblush: "pirosas levendula",
lawngreen: "fűzöld",
lemonchiffon: "sárga műselyem",
lightblue: "világoskék",
lightcoral: "világos korall",
lightcyan: "világos ciánkék",
lightgoldenrodyellow: "világos aranyvessző sárga",
lightgray: "világosszürke",
lightgreen: "világoszöld",
lightgrey: "világosszürke", // same as lightgray
lightpink: "világos rózsaszín",
lightsalmon: "világos lazacszín",
lightseagreen: "világos tengerzöld",
lightskyblue: "világos égszínkék",
lightslategray: "világos palaszürke",
lightslategrey: "világos palaszürke", // same as lightslategray
lightsteelblue: "világos acélkék",
lightyellow: "világossárga",
lime: "lime",
limegreen: "limezöld",
linen: "vászonfehér",
magenta: "bíbor",
maroon: "gesztenyebarna",
mediumaquamarine: "közepes akvamarin",
mediumblue: "közepes kék",
mediumorchid: "közepes orchidea",
mediumpurple: "közepes lila",
mediumseagreen: "közepes tengerzöld",
mediumslateblue: "közepes palakék",
mediumspringgreen: "közepes tavaszzöld",
mediumturquoise: "közepes türkizkék",
mediumvioletred: "közepes ibolyavörös",
midnightblue: "éjkék",
mintcream: "mentaszósz",
mistyrose: "halvány rózsaszín",
moccasin: "mokkaszín",
navajowhite: "navajo fehér",
navy: "tengerészkék",
oldlace: "régi csipke",
olive: "olajzöld",
olivedrab: "olajzöld drapp",
orange: "narancssárga",
orangered: "narancsvörös",
orchid: "orchidea",
palegoldenrod: "halvány aranyvessző",
palegreen: "halványzöld",
paleturquoise: "halvány türkizkék",
palevioletred: "halvány ibolyavörös",
papayawhip: "papayahab",
peachpuff: "barackszín",
peru: "peru",
pink: "rózsaszín",
plum: "szilvakék",
powderblue: "púderkék",
purple: "lila",
red: "vörös",
rosybrown: "barnásrózsaszín",
royalblue: "királykék",
saddlebrown: "nyeregbarna",
salmon: "lazacszín",
sandybrown: "homokbarna",
seagreen: "tengerzöld",
seashell: "kagyló",
sienna: "vörösesbarna",
silver: "ezüst",
skyblue: "égszínkék",
slateblue: "palakék",
slategray: "palaszürke",
slategrey: "palaszürke", // same as slategray
snow: "hó",
springgreen: "tavaszzöld",
steelblue: "acélkék",
tan: "rozsdabarna",
teal: "pávakék",
thistle: "bogáncs",
tomato: "paradicsom",
turquoise: "türkizkék",
violet: "ibolyaszín",
wheat: "búza",
white: "fehér",
whitesmoke: "fehér füst",
yellow: "sárga",
yellowgreen: "sárgászöld"
})
//end v1.x content
);
