
.dijitRtl .soria .dijitTreeNode,
.dijitRtl .soria .dijitTreeExpandoLeaf {
	
    background-image: none;
}
.dijitRtl .soria .dijitTreeContent {
    padding-left: 0;
    padding-right: 1px;
}
.dijitRtl .soria .dijitTreeExpandoOpened {
	
	background: url('images/spriteTree_rtl.gif') no-repeat -18px top;
}
.dijitRtl .soria .dijitTreeExpandoClosed {
	
	background-image: url('images/spriteTree_rtl.gif');
}
