//>>built
define("dojox/charting/themes/Dollar", ["../Theme", "./common"], function(Theme, themes){
	
	themes.Dollar=new Theme({
		colors: [
			"#A4CE67",
			"#739363",
			"#6B824A",
			"#343434",
			"#636563"
		]
	});
	
	return themes.Dollar;
});
