//>>built
define(
//begin v1.x content
{
	"HKD_displayName": "dolar hongkoński",
	"CHF_displayName": "frank szwajcarski",
	"CAD_displayName": "dolar kanadyjski",
	"CNY_displayName": "juan renminbi",
	"AUD_displayName": "dolar australijski",
	"JPY_displayName": "jen japoński",
	"USD_displayName": "dolar amerykański ",
	"GBP_displayName": "funt szterling",
	"EUR_displayName": "euro"
}
//end v1.x content
);