//>>built
define(
//begin v1.x content
({
	fontSize: "Dimensione",
	fontName: "Carattere",
	formatBlock: "Formato",

	serif: "serif",
	"sans-serif": "sans-serif",
	monospace: "spaziatura fissa",
	cursive: "corsivo",
	fantasy: "fantasy",

	noFormat: "Nessuna",
	p: "Paragrafo",
	h1: "Intestazione",
	h2: "Sottointestazione",
	h3: "Sottointestazione secondaria",
	pre: "Preformattato",

	1: "xx-small",
	2: "x-small",
	3: "small",
	4: "medium",
	5: "large",
	6: "x-large",
	7: "xx-large"
})

//end v1.x content
);
