.dijitRtl .dijitPlaceHolder {
	left: auto;
	right: 0;
}
.dijitMenuItemRtl {
	text-align: right;
}
.dj_iequirks .dijitComboButtonRtl button {
	
	float:left;
}
.dijitTextBoxRtl .dijitValidationContainer,
.dijitTextBoxRtl .dijitSpinnerButtonContainer,
.dijitComboBoxRtl .dijitArrowButtonContainer {
	
	border-right-width: 1px !important;
	border-left-width: 0 !important;
}
.dijitSpinnerRtl .dijitSpinnerButtonContainer .dijitArrowButton {
	right: 0;
	left: auto;
}
.dj_ie7 .dijitInputContainer {
	
	display: inline-block;
}
.dijitTextBoxRtl .dijitSpinnerButtonContainer,
.dijitTextBoxRtl .dijitValidationContainer,
.dijitTextBoxRtl .dijitArrowButtonContainer {
	float: left;
}
.dijitCalendarRtl .dijitCalendarNextYear {
	margin:0 0.55em 0 0;
}
.dijitCalendarRtl .dijitCalendarPreviousYear {
	margin:0 0 0 0.55em;
}
.dijitSliderRtl .dijitSliderImageHandleV {
	left:auto;
}
.dijitSliderRtl .dijitSliderImageHandleH {
	left:-50%;
}
.dijitSliderRtl .dijitSliderMoveableH {
	right:auto;
	left:0;
}
.dijitSliderRtl .dijitRuleContainerV {
	float:right;
}
.dj_ie .dijitSliderRtl .dijitRuleContainerV {
	text-align:right;
}
.dj_ie .dijitSliderRtl .dijitRuleLabelV {
	text-align:left;
}
.dj_ie .dijitSliderRtl .dijitRuleLabelH {
	zoom:1;
}
.dijitSliderRtl .dijitSliderProgressBarH {
	
	float:right;
	right:0;
	left:auto;
}
.dijitRtl .dijitContentPaneLoading .dijitIconLoading,
.dijitRtl .dijitContentPaneError .dijitIconError {
	margin-right: 0;
	margin-left: 9px;
}
.dijitTabControllerRtl .nowrapTabStrip {
    text-align: right;  
}
.dijitTabRtl .dijitTabCloseButton {
	margin-left: 0;
	margin-right: 1em;
}
.dj_ie .dijitTimePickerRtl .dijitTimePickerItem {
	width:100%;
}
.dijitColorPaletteRtl .dijitColorPaletteUnder {
	
	left: auto;
	right: 0;
}
.dijitSelectRtl .dijitButtonContents {
	text-align: right;
}
