//>>built
define(
//begin v1.x content
{
	"HKD_displayName": "דולר הונג קונגי",
	"CHF_displayName": "פרנק שוויצרי",
	"CAD_displayName": "דולר קנדי",
	"CNY_displayName": "יואן רנמינבי סיני",
	"AUD_displayName": "דולר אוסטרלי",
	"JPY_displayName": "ין יפני",
	"USD_displayName": "דולר אמריקאי",
	"GBP_displayName": "לירה שטרלינג",
	"EUR_displayName": "אירו"
}
//end v1.x content
);