//>>built
define(
//begin v1.x content
({
	"1" : "xx-kiçik",
	"2" : "x-kiçik",
	"formatBlock" : "Format",
	"3" : "kiçik",
	"4" : "orta",
	"5" : "böyük",
	"6" : "çox-böyük",
	"7" : "ən böyük",
	"fantasy" : "fantaziya",
	"serif" : "serif",
	"p" : "Abzas",
	"pre" : "Əvvəldən düzəldilmiş",
	"sans-serif" : "sans-serif",
	"fontName" : "Şrift",
	"h1" : "Başlıq",
	"h2" : "Alt Başlıq",
	"h3" : "Alt Alt Başlıq",
	"monospace" : "Tək aralıqlı",
	"fontSize" : "Ölçü",
	"cursive" : "Əl yazısı",
	"noFormat" : "Heç biri"
})
//end v1.x content
);
