/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/

/*
	This is an optimized version of Dojo, built for deployment and not for
	development. To get sources and documentation, please visit:

		http://dojotoolkit.org
*/

//>>built
require({cache:{"dojo/uacss":function(){define(["./dom-geometry","./_base/lang","./ready","./_base/sniff","./_base/window"],function(_1,_2,_3,_4,_5){var _6=_5.doc.documentElement,ie=_4("ie"),_7=_4("opera"),_8=Math.floor,ff=_4("ff"),_9=_1.boxModel.replace(/-/,""),_a={"dj_ie":ie,"dj_ie6":_8(ie)==6,"dj_ie7":_8(ie)==7,"dj_ie8":_8(ie)==8,"dj_ie9":_8(ie)==9,"dj_quirks":_4("quirks"),"dj_iequirks":ie&&_4("quirks"),"dj_opera":_7,"dj_khtml":_4("khtml"),"dj_webkit":_4("webkit"),"dj_safari":_4("safari"),"dj_chrome":_4("chrome"),"dj_gecko":_4("mozilla"),"dj_ff3":_8(ff)==3};_a["dj_"+_9]=true;var _b="";for(var _c in _a){if(_a[_c]){_b+=_c+" ";}}_6.className=_2.trim(_6.className+" "+_b);_3(90,function(){if(!_1.isBodyLtr()){var _d="dj_rtl dijitRtl "+_b.replace(/ /g,"-rtl ");_6.className=_2.trim(_6.className+" "+_d+"dj_rtl dijitRtl "+_b.replace(/ /g,"-rtl "));}});return _4;});},"dojo/text":function(){define(["./_base/kernel","require","./has","./_base/xhr"],function(_e,_f,has,xhr){var _10;if(1){_10=function(url,_11,_12){xhr("GET",{url:url,sync:!!_11,load:_12});};}else{if(_f.getText){_10=_f.getText;}else{console.error("dojo/text plugin failed to load because loader does not support getText");}}var _13={},_14=function(_15){if(_15){_15=_15.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im,"");var _16=_15.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);if(_16){_15=_16[1];}}else{_15="";}return _15;},_17={},_18={},_19={dynamic:true,normalize:function(id,_1a){var _1b=id.split("!"),url=_1b[0];return (/^\./.test(url)?_1a(url):url)+(_1b[1]?"!"+_1b[1]:"");},load:function(id,_1c,_1d){var _1e=id.split("!"),_1f=_1e.length>1,_20=_1e[0],url=_1c.toUrl(_1e[0]),_21=_17,_22=function(_23){_1d(_1f?_14(_23):_23);};if(_20 in _13){_21=_13[_20];}else{if(url in _1c.cache){_21=_1c.cache[url];}else{if(url in _13){_21=_13[url];}}}if(_21===_17){if(_18[url]){_18[url].push(_22);}else{var _24=_18[url]=[_22];_10(url,!_1c.async,function(_25){_13[_20]=_13[url]=_25;for(var i=0;i<_24.length;){_24[i++](_25);}delete _18[url];});}}else{_22(_21);}}};_e.cache=function(_26,url,_27){var key;if(typeof _26=="string"){if(/\//.test(_26)){key=_26;_27=url;}else{key=_f.toUrl(_26.replace(/\./g,"/")+(url?("/"+url):""));}}else{key=_26+"";_27=url;}var val=(_27!=undefined&&typeof _27!="string")?_27.value:_27,_28=_27&&_27.sanitize;if(typeof val=="string"){_13[key]=val;return _28?_14(val):val;}else{if(val===null){delete _13[key];return null;}else{if(!(key in _13)){_10(key,true,function(_29){_13[key]=_29;});}return _28?_14(_13[key]):_13[key];}}};return _19;});},"dijit/hccss":function(){define("dijit/hccss",["require","dojo/_base/config","dojo/dom-class","dojo/dom-construct","dojo/dom-style","dojo/ready","dojo/_base/sniff","dojo/_base/window"],function(_2a,_2b,_2c,_2d,_2e,_2f,has,win){if(has("ie")||has("mozilla")){_2f(90,function(){var div=_2d.create("div",{id:"a11yTestNode",style:{cssText:"border: 1px solid;"+"border-color:red green;"+"position: absolute;"+"height: 5px;"+"top: -999px;"+"background-image: url(\""+(_2b.blankGif||_2a.toUrl("dojo/resources/blank.gif"))+"\");"}},win.body());var cs=_2e.getComputedStyle(div);if(cs){var _30=cs.backgroundImage;var _31=(cs.borderTopColor==cs.borderRightColor)||(_30!=null&&(_30=="none"||_30=="url(invalid-url:)"));if(_31){_2c.add(win.body(),"dijit_a11y");}if(has("ie")){div.outerHTML="";}else{win.body().removeChild(div);}}});}});},"dijit/_Contained":function(){define("dijit/_Contained",["dojo/_base/declare","./registry"],function(_32,_33){return _32("dijit._Contained",null,{_getSibling:function(_34){var _35=this.domNode;do{_35=_35[_34+"Sibling"];}while(_35&&_35.nodeType!=1);return _35&&_33.byNode(_35);},getPreviousSibling:function(){return this._getSibling("previous");},getNextSibling:function(){return this._getSibling("next");},getIndexInParent:function(){var p=this.getParent();if(!p||!p.getIndexOfChild){return -1;}return p.getIndexOfChild(this);}});});},"dojo/parser":function(){define(["./_base/kernel","./_base/lang","./_base/array","./_base/html","./_base/window","./_base/url","./_base/json","./aspect","./date/stamp","./query","./on","./ready"],function(_36,_37,_38,_39,_3a,_3b,_3c,_3d,_3e,_3f,don){new Date("X");var _40={"dom-attributes-explicit":document.createElement("div").attributes.length<40};function has(_41){return _40[_41];};_36.parser=new function(){var _42={};function _43(_44){var map={};for(var _45 in _44){if(_45.charAt(0)=="_"){continue;}map[_45.toLowerCase()]=_45;}return map;};_3d.after(_37,"extend",function(){_42={};},true);var _46={};this._functionFromScript=function(_47,_48){var _49="";var _4a="";var _4b=(_47.getAttribute(_48+"args")||_47.getAttribute("args"));if(_4b){_38.forEach(_4b.split(/\s*,\s*/),function(_4c,idx){_49+="var "+_4c+" = arguments["+idx+"]; ";});}var _4d=_47.getAttribute("with");if(_4d&&_4d.length){_38.forEach(_4d.split(/\s*,\s*/),function(_4e){_49+="with("+_4e+"){";_4a+="}";});}return new Function(_49+_47.innerHTML+_4a);};this.instantiate=function(_4f,_50,_51){var _52=[],_50=_50||{};_51=_51||{};var _53=(_51.scope||_36._scopeName)+"Type",_54="data-"+(_51.scope||_36._scopeName)+"-",_55=_54+"type",_56=_54+"props",_57=_54+"attach-point",_58=_54+"attach-event",_59=_54+"id";var _5a={};_38.forEach([_56,_55,_53,_59,"jsId",_57,_58,"dojoAttachPoint","dojoAttachEvent","class","style"],function(_5b){_5a[_5b.toLowerCase()]=_5b.replace(_51.scope,"dojo");});_38.forEach(_4f,function(obj){if(!obj){return;}var _5c=obj.node||obj,_5d=_53 in _50?_50[_53]:obj.node?obj.type:(_5c.getAttribute(_55)||_5c.getAttribute(_53)),_5e=_46[_5d]||(_46[_5d]=_37.getObject(_5d)),_5f=_5e&&_5e.prototype;if(!_5e){throw new Error("Could not load class '"+_5d);}var _60={};if(_51.defaults){_37.mixin(_60,_51.defaults);}if(obj.inherited){_37.mixin(_60,obj.inherited);}var _61;if(has("dom-attributes-explicit")){_61=_5c.attributes;}else{var _62=/^input$|^img$/i.test(_5c.nodeName)?_5c:_5c.cloneNode(false),_63=_62.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g,"").replace(/^\s*<[a-zA-Z0-9]*/,"").replace(/>.*$/,"");_61=_38.map(_63.split(/\s+/),function(_64){var _65=_64.toLowerCase();return {name:_64,value:(_5c.nodeName=="LI"&&_64=="value")||_65=="enctype"?_5c.getAttribute(_65):_5c.getAttributeNode(_65).value,specified:true};});}var i=0,_66;while(_66=_61[i++]){if(!_66||!_66.specified){continue;}var _67=_66.name,_68=_67.toLowerCase(),_69=_66.value;if(_68 in _5a){switch(_5a[_68]){case "data-dojo-props":var _6a=_69;break;case "data-dojo-id":case "jsId":var _6b=_69;break;case "data-dojo-attach-point":case "dojoAttachPoint":_60.dojoAttachPoint=_69;break;case "data-dojo-attach-event":case "dojoAttachEvent":_60.dojoAttachEvent=_69;break;case "class":_60["class"]=_5c.className;break;case "style":_60["style"]=_5c.style&&_5c.style.cssText;break;}}else{if(!(_67 in _5f)){var map=(_42[_5d]||(_42[_5d]=_43(_5f)));_67=map[_68]||_67;}if(_67 in _5f){switch(typeof _5f[_67]){case "string":_60[_67]=_69;break;case "number":_60[_67]=_69.length?Number(_69):NaN;break;case "boolean":_60[_67]=_69.toLowerCase()!="false";break;case "function":if(_69===""||_69.search(/[^\w\.]+/i)!=-1){_60[_67]=new Function(_69);}else{_60[_67]=_37.getObject(_69,false)||new Function(_69);}break;default:var _6c=_5f[_67];_60[_67]=(_6c&&"length" in _6c)?(_69?_69.split(/\s*,\s*/):[]):(_6c instanceof Date)?(_69==""?new Date(""):_69=="now"?new Date():_3e.fromISOString(_69)):(_6c instanceof _36._Url)?(_36.baseUrl+_69):_3c.fromJson(_69);}}else{_60[_67]=_69;}}}if(_6a){try{_6a=_3c.fromJson.call(_51.propsThis,"{"+_6a+"}");_37.mixin(_60,_6a);}catch(e){throw new Error(e.toString()+" in data-dojo-props='"+_6a+"'");}}_37.mixin(_60,_50);var _6d=obj.node?obj.scripts:(_5e&&(_5e._noScript||_5f._noScript)?[]:_3f("> script[type^='dojo/']",_5c));var _6e=[],_6f=[],_70=[],on=[];if(_6d){for(i=0;i<_6d.length;i++){var _71=_6d[i];_5c.removeChild(_71);var _72=(_71.getAttribute(_54+"event")||_71.getAttribute("event")),_73=_71.getAttribute(_54+"prop"),_5d=_71.getAttribute("type"),nf=this._functionFromScript(_71,_54);if(_72){if(_5d=="dojo/connect"){_6e.push({event:_72,func:nf});}else{if(_5d=="dojo/on"){on.push({event:_72,func:nf});}else{_60[_72]=nf;}}}else{if(_5d=="dojo/watch"){_70.push({prop:_73,func:nf});}else{_6f.push(nf);}}}}var _74=_5e.markupFactory||_5f.markupFactory;var _75=_74?_74(_60,_5c,_5e):new _5e(_60,_5c);_52.push(_75);if(_6b){_37.setObject(_6b,_75);}for(i=0;i<_6e.length;i++){_3d.after(_75,_6e[i].event,_36.hitch(_75,_6e[i].func),true);}for(i=0;i<_6f.length;i++){_6f[i].call(_75);}for(i=0;i<_70.length;i++){_75.watch(_70[i].prop,_70[i].func);}for(i=0;i<on.length;i++){don(_75,on[i].event,on[i].func);}},this);if(!_50._started){_38.forEach(_52,function(_76){if(!_51.noStart&&_76&&_37.isFunction(_76.startup)&&!_76._started){_76.startup();}});}return _52;};this.parse=function(_77,_78){var _79;if(!_78&&_77&&_77.rootNode){_78=_77;_79=_78.rootNode;}else{_79=_77;}_79=_79?_39.byId(_79):_3a.body();_78=_78||{};var _7a=(_78.scope||_36._scopeName)+"Type",_7b="data-"+(_78.scope||_36._scopeName)+"-",_7c=_7b+"type",_7d=_7b+"textdir";var _7e=[];var _7f=_79.firstChild;var _80=_78&&_78.inherited;if(!_80){function _81(_82,_83){return (_82.getAttribute&&_82.getAttribute(_83))||(_82!==_3a.doc&&_82!==_3a.doc.documentElement&&_82.parentNode?_81(_82.parentNode,_83):null);};_80={dir:_81(_79,"dir"),lang:_81(_79,"lang"),textDir:_81(_79,_7d)};for(var key in _80){if(!_80[key]){delete _80[key];}}}var _84={inherited:_80};var _85;var _86;function _87(_88){if(!_88.inherited){_88.inherited={};var _89=_88.node,_8a=_87(_88.parent);var _8b={dir:_89.getAttribute("dir")||_8a.dir,lang:_89.getAttribute("lang")||_8a.lang,textDir:_89.getAttribute(_7d)||_8a.textDir};for(var key in _8b){if(_8b[key]){_88.inherited[key]=_8b[key];}}}return _88.inherited;};while(true){if(!_7f){if(!_84||!_84.node){break;}_7f=_84.node.nextSibling;_85=_84.scripts;_86=false;_84=_84.parent;continue;}if(_7f.nodeType!=1){_7f=_7f.nextSibling;continue;}if(_85&&_7f.nodeName.toLowerCase()=="script"){_8c=_7f.getAttribute("type");if(_8c&&/^dojo\/\w/i.test(_8c)){_85.push(_7f);}_7f=_7f.nextSibling;continue;}if(_86){_7f=_7f.nextSibling;continue;}var _8c=_7f.getAttribute(_7c)||_7f.getAttribute(_7a);var _8d=_7f.firstChild;if(!_8c&&(!_8d||(_8d.nodeType==3&&!_8d.nextSibling))){_7f=_7f.nextSibling;continue;}var _8e={node:_7f,scripts:_85,parent:_84};var _8f=_8c&&(_46[_8c]||(_46[_8c]=_37.getObject(_8c))),_90=_8f&&!_8f.prototype._noScript?[]:null;if(_8c){_7e.push({"type":_8c,node:_7f,scripts:_90,inherited:_87(_8e)});}_7f=_8d;_85=_90;_86=_8f&&_8f.prototype.stopParser&&!(_78&&_78.template);_84=_8e;}var _91=_78&&_78.template?{template:true}:null;return this.instantiate(_7e,_91,_78);};}();if(_36.config.parseOnLoad){_36.ready(100,_36.parser,"parse");}return _36.parser;});},"dijit/_Container":function(){define("dijit/_Container",["dojo/_base/array","dojo/_base/declare","dojo/dom-construct","./registry"],function(_92,_93,_94,_95){return _93("dijit._Container",null,{buildRendering:function(){this.inherited(arguments);if(!this.containerNode){this.containerNode=this.domNode;}},addChild:function(_96,_97){var _98=this.containerNode;if(_97&&typeof _97=="number"){var _99=this.getChildren();if(_99&&_99.length>=_97){_98=_99[_97-1].domNode;_97="after";}}_94.place(_96.domNode,_98,_97);if(this._started&&!_96._started){_96.startup();}},removeChild:function(_9a){if(typeof _9a=="number"){_9a=this.getChildren()[_9a];}if(_9a){var _9b=_9a.domNode;if(_9b&&_9b.parentNode){_9b.parentNode.removeChild(_9b);}}},hasChildren:function(){return this.getChildren().length>0;},_getSiblingOfChild:function(_9c,dir){var _9d=_9c.domNode,_9e=(dir>0?"nextSibling":"previousSibling");do{_9d=_9d[_9e];}while(_9d&&(_9d.nodeType!=1||!_95.byNode(_9d)));return _9d&&_95.byNode(_9d);},getIndexOfChild:function(_9f){return _92.indexOf(this.getChildren(),_9f);}});});},"dijit/_base/scroll":function(){define("dijit/_base/scroll",["dojo/window",".."],function(_a0,_a1){_a1.scrollIntoView=function(_a2,pos){_a0.scrollIntoView(_a2,pos);};});},"dijit/layout/_LayoutWidget":function(){define("dijit/layout/_LayoutWidget",["dojo/_base/lang","../_Widget","../_Container","../_Contained","dojo/_base/declare","dojo/dom-class","dojo/dom-geometry","dojo/dom-style","dojo/_base/sniff","dojo/_base/window"],function(_a3,_a4,_a5,_a6,_a7,_a8,_a9,_aa,has,win){return _a7("dijit.layout._LayoutWidget",[_a4,_a5,_a6],{baseClass:"dijitLayoutContainer",isLayoutContainer:true,buildRendering:function(){this.inherited(arguments);_a8.add(this.domNode,"dijitContainer");},startup:function(){if(this._started){return;}this.inherited(arguments);var _ab=this.getParent&&this.getParent();if(!(_ab&&_ab.isLayoutContainer)){this.resize();this.connect(win.global,"onresize",function(){this.resize();});}},resize:function(_ac,_ad){var _ae=this.domNode;if(_ac){_a9.setMarginBox(_ae,_ac);}var mb=_ad||{};_a3.mixin(mb,_ac||{});if(!("h" in mb)||!("w" in mb)){mb=_a3.mixin(_a9.getMarginBox(_ae),mb);}var cs=_aa.getComputedStyle(_ae);var me=_a9.getMarginExtents(_ae,cs);var be=_a9.getBorderExtents(_ae,cs);var bb=(this._borderBox={w:mb.w-(me.w+be.w),h:mb.h-(me.h+be.h)});var pe=_a9.getPadExtents(_ae,cs);this._contentBox={l:_aa.toPixelValue(_ae,cs.paddingLeft),t:_aa.toPixelValue(_ae,cs.paddingTop),w:bb.w-pe.w,h:bb.h-pe.h};this.layout();},layout:function(){},_setupChild:function(_af){var cls=this.baseClass+"-child "+(_af.baseClass?this.baseClass+"-"+_af.baseClass:"");_a8.add(_af.domNode,cls);},addChild:function(_b0,_b1){this.inherited(arguments);if(this._started){this._setupChild(_b0);}},removeChild:function(_b2){var cls=this.baseClass+"-child"+(_b2.baseClass?" "+this.baseClass+"-"+_b2.baseClass:"");_a8.remove(_b2.domNode,cls);this.inherited(arguments);}});});},"dijit/_base":function(){define("dijit/_base",[".","./a11y","./WidgetSet","./_base/focus","./_base/manager","./_base/place","./_base/popup","./_base/scroll","./_base/sniff","./_base/typematic","./_base/wai","./_base/window"],function(_b3){return _b3._base;});},"dijit/form/_FormWidgetMixin":function(){define("dijit/form/_FormWidgetMixin",["dojo/_base/array","dojo/_base/declare","dojo/dom-attr","dojo/dom-style","dojo/_base/lang","dojo/mouse","dojo/_base/sniff","dojo/_base/window","dojo/window","../a11y"],function(_b4,_b5,_b6,_b7,_b8,_b9,has,win,_ba,_bb){return _b5("dijit.form._FormWidgetMixin",null,{name:"",alt:"",value:"",type:"text",tabIndex:"0",_setTabIndexAttr:"focusNode",disabled:false,intermediateChanges:false,scrollOnFocus:true,_setIdAttr:"focusNode",postCreate:function(){this.inherited(arguments);this.connect(this.domNode,"onmousedown","_onMouseDown");},_setDisabledAttr:function(_bc){this._set("disabled",_bc);_b6.set(this.focusNode,"disabled",_bc);if(this.valueNode){_b6.set(this.valueNode,"disabled",_bc);}this.focusNode.setAttribute("aria-disabled",_bc);if(_bc){this._set("hovering",false);this._set("active",false);var _bd="tabIndex" in this.attributeMap?this.attributeMap.tabIndex:("_setTabIndexAttr" in this)?this._setTabIndexAttr:"focusNode";_b4.forEach(_b8.isArray(_bd)?_bd:[_bd],function(_be){var _bf=this[_be];if(has("webkit")||_bb.hasDefaultTabStop(_bf)){_bf.setAttribute("tabIndex","-1");}else{_bf.removeAttribute("tabIndex");}},this);}else{if(this.tabIndex!=""){this.set("tabIndex",this.tabIndex);}}},_onFocus:function(e){if(this.scrollOnFocus){_ba.scrollIntoView(this.domNode);}this.inherited(arguments);},isFocusable:function(){return !this.disabled&&this.focusNode&&(_b7.get(this.domNode,"display")!="none");},focus:function(){if(!this.disabled&&this.focusNode.focus){try{this.focusNode.focus();}catch(e){}}},compare:function(_c0,_c1){if(typeof _c0=="number"&&typeof _c1=="number"){return (isNaN(_c0)&&isNaN(_c1))?0:_c0-_c1;}else{if(_c0>_c1){return 1;}else{if(_c0<_c1){return -1;}else{return 0;}}}},onChange:function(){},_onChangeActive:false,_handleOnChange:function(_c2,_c3){if(this._lastValueReported==undefined&&(_c3===null||!this._onChangeActive)){this._resetValue=this._lastValueReported=_c2;}this._pendingOnChange=this._pendingOnChange||(typeof _c2!=typeof this._lastValueReported)||(this.compare(_c2,this._lastValueReported)!=0);if((this.intermediateChanges||_c3||_c3===undefined)&&this._pendingOnChange){this._lastValueReported=_c2;this._pendingOnChange=false;if(this._onChangeActive){if(this._onChangeHandle){clearTimeout(this._onChangeHandle);}this._onChangeHandle=setTimeout(_b8.hitch(this,function(){this._onChangeHandle=null;this.onChange(_c2);}),0);}}},create:function(){this.inherited(arguments);this._onChangeActive=true;},destroy:function(){if(this._onChangeHandle){clearTimeout(this._onChangeHandle);this.onChange(this._lastValueReported);}this.inherited(arguments);},_onMouseDown:function(e){if((!this.focused||!has("ie"))&&!e.ctrlKey&&_b9.isLeft(e)&&this.isFocusable()){var _c4=this.connect(win.body(),"onmouseup",function(){if(this.isFocusable()){this.focus();}this.disconnect(_c4);});}}});});},"dijit/BackgroundIframe":function(){define("dijit/BackgroundIframe",["require",".","dojo/_base/config","dojo/dom-construct","dojo/dom-style","dojo/_base/lang","dojo/on","dojo/_base/sniff","dojo/_base/window"],function(_c5,_c6,_c7,_c8,_c9,_ca,on,has,win){var _cb=new function(){var _cc=[];this.pop=function(){var _cd;if(_cc.length){_cd=_cc.pop();_cd.style.display="";}else{if(has("ie")<9){var _ce=_c7["dojoBlankHtmlUrl"]||_c5.toUrl("dojo/resources/blank.html")||"javascript:\"\"";var _cf="<iframe src='"+_ce+"' role='presentation'"+" style='position: absolute; left: 0px; top: 0px;"+"z-index: -1; filter:Alpha(Opacity=\"0\");'>";_cd=win.doc.createElement(_cf);}else{_cd=_c8.create("iframe");_cd.src="javascript:\"\"";_cd.className="dijitBackgroundIframe";_cd.setAttribute("role","presentation");_c9.set(_cd,"opacity",0.1);}_cd.tabIndex=-1;}return _cd;};this.push=function(_d0){_d0.style.display="none";_cc.push(_d0);};}();_c6.BackgroundIframe=function(_d1){if(!_d1.id){throw new Error("no id");}if(has("ie")||has("mozilla")){var _d2=(this.iframe=_cb.pop());_d1.appendChild(_d2);if(has("ie")<7||has("quirks")){this.resize(_d1);this._conn=on(_d1,"resize",_ca.hitch(this,function(){this.resize(_d1);}));}else{_c9.set(_d2,{width:"100%",height:"100%"});}}};_ca.extend(_c6.BackgroundIframe,{resize:function(_d3){if(this.iframe){_c9.set(this.iframe,{width:_d3.offsetWidth+"px",height:_d3.offsetHeight+"px"});}},destroy:function(){if(this._conn){this._conn.remove();this._conn=null;}if(this.iframe){_cb.push(this.iframe);delete this.iframe;}}});return _c6.BackgroundIframe;});},"dijit/form/_FormValueMixin":function(){define("dijit/form/_FormValueMixin",["dojo/_base/declare","dojo/dom-attr","dojo/keys","dojo/_base/sniff","./_FormWidgetMixin"],function(_d4,_d5,_d6,has,_d7){return _d4("dijit.form._FormValueMixin",_d7,{readOnly:false,_setReadOnlyAttr:function(_d8){_d5.set(this.focusNode,"readOnly",_d8);this.focusNode.setAttribute("aria-readonly",_d8);this._set("readOnly",_d8);},postCreate:function(){this.inherited(arguments);if(has("ie")){this.connect(this.focusNode||this.domNode,"onkeydown",this._onKeyDown);}if(this._resetValue===undefined){this._lastValueReported=this._resetValue=this.value;}},_setValueAttr:function(_d9,_da){this._handleOnChange(_d9,_da);},_handleOnChange:function(_db,_dc){this._set("value",_db);this.inherited(arguments);},undo:function(){this._setValueAttr(this._lastValueReported,false);},reset:function(){this._hasBeenBlurred=false;this._setValueAttr(this._resetValue,true);},_onKeyDown:function(e){if(e.keyCode==_d6.ESCAPE&&!(e.ctrlKey||e.altKey||e.metaKey)){var te;if(has("ie")<9||(has("ie")&&has("quirks"))){e.preventDefault();te=document.createEventObject();te.keyCode=_d6.ESCAPE;te.shiftKey=e.shiftKey;e.srcElement.fireEvent("onkeypress",te);}}}});});},"dojo/Stateful":function(){define(["./_base/kernel","./_base/declare","./_base/lang","./_base/array"],function(_dd,_de,_df,_e0){return _dd.declare("dojo.Stateful",null,{postscript:function(_e1){if(_e1){_df.mixin(this,_e1);}},get:function(_e2){return this[_e2];},set:function(_e3,_e4){if(typeof _e3==="object"){for(var x in _e3){this.set(x,_e3[x]);}return this;}var _e5=this[_e3];this[_e3]=_e4;if(this._watchCallbacks){this._watchCallbacks(_e3,_e5,_e4);}return this;},watch:function(_e6,_e7){var _e8=this._watchCallbacks;if(!_e8){var _e9=this;_e8=this._watchCallbacks=function(_ea,_eb,_ec,_ed){var _ee=function(_ef){if(_ef){_ef=_ef.slice();for(var i=0,l=_ef.length;i<l;i++){try{_ef[i].call(_e9,_ea,_eb,_ec);}catch(e){console.error(e);}}}};_ee(_e8["_"+_ea]);if(!_ed){_ee(_e8["*"]);}};}if(!_e7&&typeof _e6==="function"){_e7=_e6;_e6="*";}else{_e6="_"+_e6;}var _f0=_e8[_e6];if(typeof _f0!=="object"){_f0=_e8[_e6]=[];}_f0.push(_e7);return {unwatch:function(){_f0.splice(_e0.indexOf(_f0,_e7),1);}};}});});},"dojo/touch":function(){define(["./_base/kernel","./on","./has","./mouse"],function(_f1,on,has,_f2){function _f3(_f4){return function(_f5,_f6){return on(_f5,_f4,_f6);};};var _f7=has("touch");_f1.touch={press:_f3(_f7?"touchstart":"mousedown"),move:_f3(_f7?"touchmove":"mousemove"),release:_f3(_f7?"touchend":"mouseup"),cancel:_f7?_f3("touchcancel"):_f2.leave};return _f1.touch;});},"dijit/_CssStateMixin":function(){define("dijit/_CssStateMixin",["dojo/touch","dojo/_base/array","dojo/_base/declare","dojo/dom-class","dojo/_base/lang","dojo/_base/window"],function(_f8,_f9,_fa,_fb,_fc,win){return _fa("dijit._CssStateMixin",[],{cssStateNodes:{},hovering:false,active:false,_applyAttributes:function(){this.inherited(arguments);_f9.forEach(["onmouseenter","onmouseleave",_f8.press],function(e){this.connect(this.domNode,e,"_cssMouseEvent");},this);_f9.forEach(["disabled","readOnly","checked","selected","focused","state","hovering","active"],function(_fd){this.watch(_fd,_fc.hitch(this,"_setStateClass"));},this);for(var ap in this.cssStateNodes){this._trackMouseState(this[ap],this.cssStateNodes[ap]);}this._setStateClass();},_cssMouseEvent:function(_fe){if(!this.disabled){switch(_fe.type){case "mouseenter":case "mouseover":this._set("hovering",true);this._set("active",this._mouseDown);break;case "mouseleave":case "mouseout":this._set("hovering",false);this._set("active",false);break;case "mousedown":case "touchpress":this._set("active",true);this._mouseDown=true;var _ff=this.connect(win.body(),_f8.release,function(){this._mouseDown=false;this._set("active",false);this.disconnect(_ff);});break;}}},_setStateClass:function(){var _100=this.baseClass.split(" ");function _101(_102){_100=_100.concat(_f9.map(_100,function(c){return c+_102;}),"dijit"+_102);};if(!this.isLeftToRight()){_101("Rtl");}var _103=this.checked=="mixed"?"Mixed":(this.checked?"Checked":"");if(this.checked){_101(_103);}if(this.state){_101(this.state);}if(this.selected){_101("Selected");}if(this.disabled){_101("Disabled");}else{if(this.readOnly){_101("ReadOnly");}else{if(this.active){_101("Active");}else{if(this.hovering){_101("Hover");}}}}if(this.focused){_101("Focused");}var tn=this.stateNode||this.domNode,_104={};_f9.forEach(tn.className.split(" "),function(c){_104[c]=true;});if("_stateClasses" in this){_f9.forEach(this._stateClasses,function(c){delete _104[c];});}_f9.forEach(_100,function(c){_104[c]=true;});var _105=[];for(var c in _104){_105.push(c);}tn.className=_105.join(" ");this._stateClasses=_100;},_trackMouseState:function(node,_106){var _107=false,_108=false,_109=false;var self=this,cn=_fc.hitch(this,"connect",node);function _10a(){var _10b=("disabled" in self&&self.disabled)||("readonly" in self&&self.readonly);_fb.toggle(node,_106+"Hover",_107&&!_108&&!_10b);_fb.toggle(node,_106+"Active",_108&&!_10b);_fb.toggle(node,_106+"Focused",_109&&!_10b);};cn("onmouseenter",function(){_107=true;_10a();});cn("onmouseleave",function(){_107=false;_108=false;_10a();});cn(_f8.press,function(){_108=true;_10a();});cn(_f8.release,function(){_108=false;_10a();});cn("onfocus",function(){_109=true;_10a();});cn("onblur",function(){_109=false;_10a();});this.watch("disabled",_10a);this.watch("readOnly",_10a);}});});},"dojo/_base/url":function(){define(["./kernel"],function(dojo){var ore=new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$"),ire=new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$"),_10c=function(){var n=null,_10d=arguments,uri=[_10d[0]];for(var i=1;i<_10d.length;i++){if(!_10d[i]){continue;}var _10e=new _10c(_10d[i]+""),_10f=new _10c(uri[0]+"");if(_10e.path==""&&!_10e.scheme&&!_10e.authority&&!_10e.query){if(_10e.fragment!=n){_10f.fragment=_10e.fragment;}_10e=_10f;}else{if(!_10e.scheme){_10e.scheme=_10f.scheme;if(!_10e.authority){_10e.authority=_10f.authority;if(_10e.path.charAt(0)!="/"){var path=_10f.path.substring(0,_10f.path.lastIndexOf("/")+1)+_10e.path;var segs=path.split("/");for(var j=0;j<segs.length;j++){if(segs[j]=="."){if(j==segs.length-1){segs[j]="";}else{segs.splice(j,1);j--;}}else{if(j>0&&!(j==1&&segs[0]=="")&&segs[j]==".."&&segs[j-1]!=".."){if(j==(segs.length-1)){segs.splice(j,1);segs[j-1]="";}else{segs.splice(j-1,2);j-=2;}}}}_10e.path=segs.join("/");}}}}uri=[];if(_10e.scheme){uri.push(_10e.scheme,":");}if(_10e.authority){uri.push("//",_10e.authority);}uri.push(_10e.path);if(_10e.query){uri.push("?",_10e.query);}if(_10e.fragment){uri.push("#",_10e.fragment);}}this.uri=uri.join("");var r=this.uri.match(ore);this.scheme=r[2]||(r[1]?"":n);this.authority=r[4]||(r[3]?"":n);this.path=r[5];this.query=r[7]||(r[6]?"":n);this.fragment=r[9]||(r[8]?"":n);if(this.authority!=n){r=this.authority.match(ire);this.user=r[3]||n;this.password=r[4]||n;this.host=r[6]||r[7];this.port=r[9]||n;}};_10c.prototype.toString=function(){return this.uri;};return dojo._Url=_10c;});},"dojo/string":function(){define(["./_base/kernel","./_base/lang"],function(dojo,lang){lang.getObject("string",true,dojo);dojo.string.rep=function(str,num){if(num<=0||!str){return "";}var buf=[];for(;;){if(num&1){buf.push(str);}if(!(num>>=1)){break;}str+=str;}return buf.join("");};dojo.string.pad=function(text,size,ch,end){if(!ch){ch="0";}var out=String(text),pad=dojo.string.rep(ch,Math.ceil((size-out.length)/ch.length));return end?out+pad:pad+out;};dojo.string.substitute=function(_110,map,_111,_112){_112=_112||dojo.global;_111=_111?lang.hitch(_112,_111):function(v){return v;};return _110.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,function(_113,key,_114){var _115=lang.getObject(key,false,map);if(_114){_115=lang.getObject(_114,false,_112).call(_112,_115,key);}return _111(_115,key).toString();});};dojo.string.trim=String.prototype.trim?lang.trim:function(str){str=str.replace(/^\s+/,"");for(var i=str.length-1;i>=0;i--){if(/\S/.test(str.charAt(i))){str=str.substring(0,i+1);break;}}return str;};return dojo.string;});},"dijit/form/_FormValueWidget":function(){define("dijit/form/_FormValueWidget",["dojo/_base/declare","dojo/_base/sniff","./_FormWidget","./_FormValueMixin"],function(_116,has,_117,_118){return _116("dijit.form._FormValueWidget",[_117,_118],{_layoutHackIE7:function(){if(has("ie")==7){var _119=this.domNode;var _11a=_119.parentNode;var _11b=_119.firstChild||_119;var _11c=_11b.style.filter;var _11d=this;while(_11a&&_11a.clientHeight==0){(function ping(){var _11e=_11d.connect(_11a,"onscroll",function(){_11d.disconnect(_11e);_11b.style.filter=(new Date()).getMilliseconds();setTimeout(function(){_11b.style.filter=_11c;},0);});})();_11a=_11a.parentNode;}}}});});},"dijit/registry":function(){define("dijit/registry",["dojo/_base/array","dojo/_base/sniff","dojo/_base/unload","dojo/_base/window","."],function(_11f,has,_120,win,_121){var _122={},hash={};var _123={length:0,add:function(_124){if(hash[_124.id]){throw new Error("Tried to register widget with id=="+_124.id+" but that id is already registered");}hash[_124.id]=_124;this.length++;},remove:function(id){if(hash[id]){delete hash[id];this.length--;}},byId:function(id){return typeof id=="string"?hash[id]:id;},byNode:function(node){return hash[node.getAttribute("widgetId")];},toArray:function(){var ar=[];for(var id in hash){ar.push(hash[id]);}return ar;},getUniqueId:function(_125){var id;do{id=_125+"_"+(_125 in _122?++_122[_125]:_122[_125]=0);}while(hash[id]);return _121._scopeName=="dijit"?id:_121._scopeName+"_"+id;},findWidgets:function(root){var _126=[];function _127(root){for(var node=root.firstChild;node;node=node.nextSibling){if(node.nodeType==1){var _128=node.getAttribute("widgetId");if(_128){var _129=hash[_128];if(_129){_126.push(_129);}}else{_127(node);}}}};_127(root);return _126;},_destroyAll:function(){_121._curFocus=null;_121._prevFocus=null;_121._activeStack=[];_11f.forEach(_123.findWidgets(win.body()),function(_12a){if(!_12a._destroyed){if(_12a.destroyRecursive){_12a.destroyRecursive();}else{if(_12a.destroy){_12a.destroy();}}}});},getEnclosingWidget:function(node){while(node){var id=node.getAttribute&&node.getAttribute("widgetId");if(id){return hash[id];}node=node.parentNode;}return null;},_hash:hash};if(has("ie")){_120.addOnWindowUnload(function(){_123._destroyAll();});}_121.registry=_123;return _123;});},"dijit/_base/manager":function(){define("dijit/_base/manager",["dojo/_base/array","dojo/_base/config","../registry",".."],function(_12b,_12c,_12d,_12e){_12b.forEach(["byId","getUniqueId","findWidgets","_destroyAll","byNode","getEnclosingWidget"],function(name){_12e[name]=_12d[name];});_12e.defaultDuration=_12c["defaultDuration"]||200;return _12e;});},"dijit/_base/place":function(){define("dijit/_base/place",["dojo/_base/array","dojo/_base/lang","dojo/window","../place",".."],function(_12f,lang,_130,_131,_132){_132.getViewport=function(){return _130.getBox();};_132.placeOnScreen=_131.at;_132.placeOnScreenAroundElement=function(node,_133,_134,_135){var _136;if(lang.isArray(_134)){_136=_134;}else{_136=[];for(var key in _134){_136.push({aroundCorner:key,corner:_134[key]});}}return _131.around(node,_133,_136,true,_135);};_132.placeOnScreenAroundNode=_132.placeOnScreenAroundElement;_132.placeOnScreenAroundRectangle=_132.placeOnScreenAroundElement;_132.getPopupAroundAlignment=function(_137,_138){var _139={};_12f.forEach(_137,function(pos){var ltr=_138;switch(pos){case "after":_139[_138?"BR":"BL"]=_138?"BL":"BR";break;case "before":_139[_138?"BL":"BR"]=_138?"BR":"BL";break;case "below-alt":ltr=!ltr;case "below":_139[ltr?"BL":"BR"]=ltr?"TL":"TR";_139[ltr?"BR":"BL"]=ltr?"TR":"TL";break;case "above-alt":ltr=!ltr;case "above":default:_139[ltr?"TL":"TR"]=ltr?"BL":"BR";_139[ltr?"TR":"TL"]=ltr?"BR":"BL";break;}});return _139;};return _132;});},"dijit/WidgetSet":function(){define("dijit/WidgetSet",["dojo/_base/array","dojo/_base/declare","dojo/_base/window","./registry"],function(_13a,_13b,win,_13c){var _13d=_13b("dijit.WidgetSet",null,{constructor:function(){this._hash={};this.length=0;},add:function(_13e){if(this._hash[_13e.id]){throw new Error("Tried to register widget with id=="+_13e.id+" but that id is already registered");}this._hash[_13e.id]=_13e;this.length++;},remove:function(id){if(this._hash[id]){delete this._hash[id];this.length--;}},forEach:function(func,_13f){_13f=_13f||win.global;var i=0,id;for(id in this._hash){func.call(_13f,this._hash[id],i++,this._hash);}return this;},filter:function(_140,_141){_141=_141||win.global;var res=new _13d(),i=0,id;for(id in this._hash){var w=this._hash[id];if(_140.call(_141,w,i++,this._hash)){res.add(w);}}return res;},byId:function(id){return this._hash[id];},byClass:function(cls){var res=new _13d(),id,_142;for(id in this._hash){_142=this._hash[id];if(_142.declaredClass==cls){res.add(_142);}}return res;},toArray:function(){var ar=[];for(var id in this._hash){ar.push(this._hash[id]);}return ar;},map:function(func,_143){return _13a.map(this.toArray(),func,_143);},every:function(func,_144){_144=_144||win.global;var x=0,i;for(i in this._hash){if(!func.call(_144,this._hash[i],x++,this._hash)){return false;}}return true;},some:function(func,_145){_145=_145||win.global;var x=0,i;for(i in this._hash){if(func.call(_145,this._hash[i],x++,this._hash)){return true;}}return false;}});_13a.forEach(["forEach","filter","byClass","map","every","some"],function(func){_13c[func]=_13d.prototype[func];});return _13d;});},"dijit/a11y":function(){define("dijit/a11y",["dojo/_base/array","dojo/_base/config","dojo/_base/declare","dojo/dom","dojo/dom-attr","dojo/dom-style","dojo/_base/sniff","./_base/manager","."],function(_146,_147,_148,dom,_149,_14a,has,_14b,_14c){var _14d=(_14c._isElementShown=function(elem){var s=_14a.get(elem);return (s.visibility!="hidden")&&(s.visibility!="collapsed")&&(s.display!="none")&&(_149.get(elem,"type")!="hidden");});_14c.hasDefaultTabStop=function(elem){switch(elem.nodeName.toLowerCase()){case "a":return _149.has(elem,"href");case "area":case "button":case "input":case "object":case "select":case "textarea":return true;case "iframe":var body;try{var _14e=elem.contentDocument;if("designMode" in _14e&&_14e.designMode=="on"){return true;}body=_14e.body;}catch(e1){try{body=elem.contentWindow.document.body;}catch(e2){return false;}}return body&&(body.contentEditable=="true"||(body.firstChild&&body.firstChild.contentEditable=="true"));default:return elem.contentEditable=="true";}};var _14f=(_14c.isTabNavigable=function(elem){if(_149.get(elem,"disabled")){return false;}else{if(_149.has(elem,"tabIndex")){return _149.get(elem,"tabIndex")>=0;}else{return _14c.hasDefaultTabStop(elem);}}});_14c._getTabNavigable=function(root){var _150,last,_151,_152,_153,_154,_155={};function _156(node){return node&&node.tagName.toLowerCase()=="input"&&node.type&&node.type.toLowerCase()=="radio"&&node.name&&node.name.toLowerCase();};var _157=function(_158){for(var _159=_158.firstChild;_159;_159=_159.nextSibling){if(_159.nodeType!=1||(has("ie")&&_159.scopeName!=="HTML")||!_14d(_159)){continue;}if(_14f(_159)){var _15a=_149.get(_159,"tabIndex");if(!_149.has(_159,"tabIndex")||_15a==0){if(!_150){_150=_159;}last=_159;}else{if(_15a>0){if(!_151||_15a<_152){_152=_15a;_151=_159;}if(!_153||_15a>=_154){_154=_15a;_153=_159;}}}var rn=_156(_159);if(_149.get(_159,"checked")&&rn){_155[rn]=_159;}}if(_159.nodeName.toUpperCase()!="SELECT"){_157(_159);}}};if(_14d(root)){_157(root);}function rs(node){return _155[_156(node)]||node;};return {first:rs(_150),last:rs(last),lowest:rs(_151),highest:rs(_153)};};_14c.getFirstInTabbingOrder=function(root){var _15b=_14c._getTabNavigable(dom.byId(root));return _15b.lowest?_15b.lowest:_15b.first;};_14c.getLastInTabbingOrder=function(root){var _15c=_14c._getTabNavigable(dom.byId(root));return _15c.last?_15c.last:_15c.highest;};return {hasDefaultTabStop:_14c.hasDefaultTabStop,isTabNavigable:_14c.isTabNavigable,_getTabNavigable:_14c._getTabNavigable,getFirstInTabbingOrder:_14c.getFirstInTabbingOrder,getLastInTabbingOrder:_14c.getLastInTabbingOrder};});},"dijit/typematic":function(){define(["dojo/_base/array","dojo/_base/connect","dojo/_base/event","dojo/_base/kernel","dojo/_base/lang","dojo/on","dojo/_base/sniff","."],function(_15d,_15e,_15f,_160,lang,on,has,_161){var _162=(_161.typematic={_fireEventAndReload:function(){this._timer=null;this._callback(++this._count,this._node,this._evt);this._currentTimeout=Math.max(this._currentTimeout<0?this._initialDelay:(this._subsequentDelay>1?this._subsequentDelay:Math.round(this._currentTimeout*this._subsequentDelay)),this._minDelay);this._timer=setTimeout(lang.hitch(this,"_fireEventAndReload"),this._currentTimeout);},trigger:function(evt,_163,node,_164,obj,_165,_166,_167){if(obj!=this._obj){this.stop();this._initialDelay=_166||500;this._subsequentDelay=_165||0.9;this._minDelay=_167||10;this._obj=obj;this._evt=evt;this._node=node;this._currentTimeout=-1;this._count=-1;this._callback=lang.hitch(_163,_164);this._fireEventAndReload();this._evt=lang.mixin({faux:true},evt);}},stop:function(){if(this._timer){clearTimeout(this._timer);this._timer=null;}if(this._obj){this._callback(-1,this._node,this._evt);this._obj=null;}},addKeyListener:function(node,_168,_169,_16a,_16b,_16c,_16d){if(_168.keyCode){_168.charOrCode=_168.keyCode;_160.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.","","2.0");}else{if(_168.charCode){_168.charOrCode=String.fromCharCode(_168.charCode);_160.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.","","2.0");}}var _16e=[on(node,_15e._keypress,lang.hitch(this,function(evt){if(evt.charOrCode==_168.charOrCode&&(_168.ctrlKey===undefined||_168.ctrlKey==evt.ctrlKey)&&(_168.altKey===undefined||_168.altKey==evt.altKey)&&(_168.metaKey===undefined||_168.metaKey==(evt.metaKey||false))&&(_168.shiftKey===undefined||_168.shiftKey==evt.shiftKey)){_15f.stop(evt);_162.trigger(evt,_169,node,_16a,_168,_16b,_16c,_16d);}else{if(_162._obj==_168){_162.stop();}}})),on(node,"keyup",lang.hitch(this,function(){if(_162._obj==_168){_162.stop();}}))];return {remove:function(){_15d.forEach(_16e,function(h){h.remove();});}};},addMouseListener:function(node,_16f,_170,_171,_172,_173){var _174=[on(node,"mousedown",lang.hitch(this,function(evt){_15f.stop(evt);_162.trigger(evt,_16f,node,_170,node,_171,_172,_173);})),on(node,"mouseup",lang.hitch(this,function(evt){if(this._obj){_15f.stop(evt);}_162.stop();})),on(node,"mouseout",lang.hitch(this,function(evt){_15f.stop(evt);_162.stop();})),on(node,"mousemove",lang.hitch(this,function(evt){evt.preventDefault();})),on(node,"dblclick",lang.hitch(this,function(evt){_15f.stop(evt);if(has("ie")){_162.trigger(evt,_16f,node,_170,node,_171,_172,_173);setTimeout(lang.hitch(this,_162.stop),50);}}))];return {remove:function(){_15d.forEach(_174,function(h){h.remove();});}};},addListener:function(_175,_176,_177,_178,_179,_17a,_17b,_17c){var _17d=[this.addKeyListener(_176,_177,_178,_179,_17a,_17b,_17c),this.addMouseListener(_175,_178,_179,_17a,_17b,_17c)];return {remove:function(){_15d.forEach(_17d,function(h){h.remove();});}};}});return _162;});},"dijit/_base/focus":function(){define("dijit/_base/focus",["dojo/_base/array","dojo/dom","dojo/_base/lang","dojo/topic","dojo/_base/window","../focus",".."],function(_17e,dom,lang,_17f,win,_180,_181){lang.mixin(_181,{_curFocus:null,_prevFocus:null,isCollapsed:function(){return _181.getBookmark().isCollapsed;},getBookmark:function(){var bm,rg,tg,sel=win.doc.selection,cf=_180.curNode;if(win.global.getSelection){sel=win.global.getSelection();if(sel){if(sel.isCollapsed){tg=cf?cf.tagName:"";if(tg){tg=tg.toLowerCase();if(tg=="textarea"||(tg=="input"&&(!cf.type||cf.type.toLowerCase()=="text"))){sel={start:cf.selectionStart,end:cf.selectionEnd,node:cf,pRange:true};return {isCollapsed:(sel.end<=sel.start),mark:sel};}}bm={isCollapsed:true};if(sel.rangeCount){bm.mark=sel.getRangeAt(0).cloneRange();}}else{rg=sel.getRangeAt(0);bm={isCollapsed:false,mark:rg.cloneRange()};}}}else{if(sel){tg=cf?cf.tagName:"";tg=tg.toLowerCase();if(cf&&tg&&(tg=="button"||tg=="textarea"||tg=="input")){if(sel.type&&sel.type.toLowerCase()=="none"){return {isCollapsed:true,mark:null};}else{rg=sel.createRange();return {isCollapsed:rg.text&&rg.text.length?false:true,mark:{range:rg,pRange:true}};}}bm={};try{rg=sel.createRange();bm.isCollapsed=!(sel.type=="Text"?rg.htmlText.length:rg.length);}catch(e){bm.isCollapsed=true;return bm;}if(sel.type.toUpperCase()=="CONTROL"){if(rg.length){bm.mark=[];var i=0,len=rg.length;while(i<len){bm.mark.push(rg.item(i++));}}else{bm.isCollapsed=true;bm.mark=null;}}else{bm.mark=rg.getBookmark();}}else{console.warn("No idea how to store the current selection for this browser!");}}return bm;},moveToBookmark:function(_182){var _183=win.doc,mark=_182.mark;if(mark){if(win.global.getSelection){var sel=win.global.getSelection();if(sel&&sel.removeAllRanges){if(mark.pRange){var n=mark.node;n.selectionStart=mark.start;n.selectionEnd=mark.end;}else{sel.removeAllRanges();sel.addRange(mark);}}else{console.warn("No idea how to restore selection for this browser!");}}else{if(_183.selection&&mark){var rg;if(mark.pRange){rg=mark.range;}else{if(lang.isArray(mark)){rg=_183.body.createControlRange();_17e.forEach(mark,function(n){rg.addElement(n);});}else{rg=_183.body.createTextRange();rg.moveToBookmark(mark);}}rg.select();}}}},getFocus:function(menu,_184){var node=!_180.curNode||(menu&&dom.isDescendant(_180.curNode,menu.domNode))?_181._prevFocus:_180.curNode;return {node:node,bookmark:node&&(node==_180.curNode)&&win.withGlobal(_184||win.global,_181.getBookmark),openedForWindow:_184};},_activeStack:[],registerIframe:function(_185){return _180.registerIframe(_185);},unregisterIframe:function(_186){_186&&_186.remove();},registerWin:function(_187,_188){return _180.registerWin(_187,_188);},unregisterWin:function(_189){_189&&_189.remove();}});_180.focus=function(_18a){if(!_18a){return;}var node="node" in _18a?_18a.node:_18a,_18b=_18a.bookmark,_18c=_18a.openedForWindow,_18d=_18b?_18b.isCollapsed:false;if(node){var _18e=(node.tagName.toLowerCase()=="iframe")?node.contentWindow:node;if(_18e&&_18e.focus){try{_18e.focus();}catch(e){}}_180._onFocusNode(node);}if(_18b&&win.withGlobal(_18c||win.global,_181.isCollapsed)&&!_18d){if(_18c){_18c.focus();}try{win.withGlobal(_18c||win.global,_181.moveToBookmark,null,[_18b]);}catch(e2){}}};_180.watch("curNode",function(name,_18f,_190){_181._curFocus=_190;_181._prevFocus=_18f;if(_190){_17f.publish("focusNode",_190);}});_180.watch("activeStack",function(name,_191,_192){_181._activeStack=_192;});_180.on("widget-blur",function(_193,by){_17f.publish("widgetBlur",_193,by);});_180.on("widget-focus",function(_194,by){_17f.publish("widgetFocus",_194,by);});return _181;});},"dijit/place":function(){define("dijit/place",["dojo/_base/array","dojo/dom-geometry","dojo/dom-style","dojo/_base/kernel","dojo/_base/window","dojo/window","."],function(_195,_196,_197,_198,win,_199,_19a){function _19b(node,_19c,_19d,_19e){var view=_199.getBox();if(!node.parentNode||String(node.parentNode.tagName).toLowerCase()!="body"){win.body().appendChild(node);}var best=null;_195.some(_19c,function(_19f){var _1a0=_19f.corner;var pos=_19f.pos;var _1a1=0;var _1a2={w:{"L":view.l+view.w-pos.x,"R":pos.x-view.l,"M":view.w}[_1a0.charAt(1)],h:{"T":view.t+view.h-pos.y,"B":pos.y-view.t,"M":view.h}[_1a0.charAt(0)]};if(_19d){var res=_19d(node,_19f.aroundCorner,_1a0,_1a2,_19e);_1a1=typeof res=="undefined"?0:res;}var _1a3=node.style;var _1a4=_1a3.display;var _1a5=_1a3.visibility;if(_1a3.display=="none"){_1a3.visibility="hidden";_1a3.display="";}var mb=_196.getMarginBox(node);_1a3.display=_1a4;_1a3.visibility=_1a5;var _1a6={"L":pos.x,"R":pos.x-mb.w,"M":Math.max(view.l,Math.min(view.l+view.w,pos.x+(mb.w>>1))-mb.w)}[_1a0.charAt(1)],_1a7={"T":pos.y,"B":pos.y-mb.h,"M":Math.max(view.t,Math.min(view.t+view.h,pos.y+(mb.h>>1))-mb.h)}[_1a0.charAt(0)],_1a8=Math.max(view.l,_1a6),_1a9=Math.max(view.t,_1a7),endX=Math.min(view.l+view.w,_1a6+mb.w),endY=Math.min(view.t+view.h,_1a7+mb.h),_1aa=endX-_1a8,_1ab=endY-_1a9;_1a1+=(mb.w-_1aa)+(mb.h-_1ab);if(best==null||_1a1<best.overflow){best={corner:_1a0,aroundCorner:_19f.aroundCorner,x:_1a8,y:_1a9,w:_1aa,h:_1ab,overflow:_1a1,spaceAvailable:_1a2};}return !_1a1;});if(best.overflow&&_19d){_19d(node,best.aroundCorner,best.corner,best.spaceAvailable,_19e);}var l=_196.isBodyLtr(),s=node.style;s.top=best.y+"px";s[l?"left":"right"]=(l?best.x:view.w-best.x-best.w)+"px";s[l?"right":"left"]="auto";return best;};return (_19a.place={at:function(node,pos,_1ac,_1ad){var _1ae=_195.map(_1ac,function(_1af){var c={corner:_1af,pos:{x:pos.x,y:pos.y}};if(_1ad){c.pos.x+=_1af.charAt(1)=="L"?_1ad.x:-_1ad.x;c.pos.y+=_1af.charAt(0)=="T"?_1ad.y:-_1ad.y;}return c;});return _19b(node,_1ae);},around:function(node,_1b0,_1b1,_1b2,_1b3){var _1b4=(typeof _1b0=="string"||"offsetWidth" in _1b0)?_196.position(_1b0,true):_1b0;if(_1b0.parentNode){var _1b5=_1b0.parentNode;while(_1b5&&_1b5.nodeType==1&&_1b5.nodeName!="BODY"){var _1b6=_196.position(_1b5,true);var _1b7=_197.getComputedStyle(_1b5).overflow;if(_1b7=="hidden"||_1b7=="auto"||_1b7=="scroll"){var _1b8=Math.min(_1b4.y+_1b4.h,_1b6.y+_1b6.h);var _1b9=Math.min(_1b4.x+_1b4.w,_1b6.x+_1b6.w);_1b4.x=Math.max(_1b4.x,_1b6.x);_1b4.y=Math.max(_1b4.y,_1b6.y);_1b4.h=_1b8-_1b4.y;_1b4.w=_1b9-_1b4.x;}_1b5=_1b5.parentNode;}}var x=_1b4.x,y=_1b4.y,_1ba="w" in _1b4?_1b4.w:(_1b4.w=_1b4.width),_1bb="h" in _1b4?_1b4.h:(_198.deprecated("place.around: dijit.place.__Rectangle: { x:"+x+", y:"+y+", height:"+_1b4.height+", width:"+_1ba+" } has been deprecated.  Please use { x:"+x+", y:"+y+", h:"+_1b4.height+", w:"+_1ba+" }","","2.0"),_1b4.h=_1b4.height);var _1bc=[];function push(_1bd,_1be){_1bc.push({aroundCorner:_1bd,corner:_1be,pos:{x:{"L":x,"R":x+_1ba,"M":x+(_1ba>>1)}[_1bd.charAt(1)],y:{"T":y,"B":y+_1bb,"M":y+(_1bb>>1)}[_1bd.charAt(0)]}});};_195.forEach(_1b1,function(pos){var ltr=_1b2;switch(pos){case "above-centered":push("TM","BM");break;case "below-centered":push("BM","TM");break;case "after":ltr=!ltr;case "before":push(ltr?"ML":"MR",ltr?"MR":"ML");break;case "below-alt":ltr=!ltr;case "below":push(ltr?"BL":"BR",ltr?"TL":"TR");push(ltr?"BR":"BL",ltr?"TR":"TL");break;case "above-alt":ltr=!ltr;case "above":push(ltr?"TL":"TR",ltr?"BL":"BR");push(ltr?"TR":"TL",ltr?"BR":"BL");break;default:push(pos.aroundCorner,pos.corner);}});var _1bf=_19b(node,_1bc,_1b3,{w:_1ba,h:_1bb});_1bf.aroundNodePos=_1b4;return _1bf;}});});},"dijit/_Widget":function(){define("dijit/_Widget",["dojo/aspect","dojo/_base/config","dojo/_base/connect","dojo/_base/declare","dojo/_base/kernel","dojo/_base/lang","dojo/query","dojo/ready","./registry","./_WidgetBase","./_OnDijitClickMixin","./_FocusMixin","dojo/uacss","./hccss"],function(_1c0,_1c1,_1c2,_1c3,_1c4,lang,_1c5,_1c6,_1c7,_1c8,_1c9,_1ca){function _1cb(){};function _1cc(_1cd){return function(obj,_1ce,_1cf,_1d0){if(obj&&typeof _1ce=="string"&&obj[_1ce]==_1cb){return obj.on(_1ce.substring(2).toLowerCase(),lang.hitch(_1cf,_1d0));}return _1cd.apply(_1c2,arguments);};};_1c0.around(_1c2,"connect",_1cc);if(_1c4.connect){_1c0.around(_1c4,"connect",_1cc);}var _1d1=_1c3("dijit._Widget",[_1c8,_1c9,_1ca],{onClick:_1cb,onDblClick:_1cb,onKeyDown:_1cb,onKeyPress:_1cb,onKeyUp:_1cb,onMouseDown:_1cb,onMouseMove:_1cb,onMouseOut:_1cb,onMouseOver:_1cb,onMouseLeave:_1cb,onMouseEnter:_1cb,onMouseUp:_1cb,constructor:function(_1d2){this._toConnect={};for(var name in _1d2){if(this[name]===_1cb){this._toConnect[name.replace(/^on/,"").toLowerCase()]=_1d2[name];delete _1d2[name];}}},postCreate:function(){this.inherited(arguments);for(var name in this._toConnect){this.on(name,this._toConnect[name]);}delete this._toConnect;},on:function(type,func){if(this[this._onMap(type)]===_1cb){return _1c2.connect(this.domNode,type.toLowerCase(),this,func);}return this.inherited(arguments);},_setFocusedAttr:function(val){this._focused=val;this._set("focused",val);},setAttribute:function(attr,_1d3){_1c4.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.","","2.0");this.set(attr,_1d3);},attr:function(name,_1d4){if(_1c1.isDebug){var _1d5=arguments.callee._ach||(arguments.callee._ach={}),_1d6=(arguments.callee.caller||"unknown caller").toString();if(!_1d5[_1d6]){_1c4.deprecated(this.declaredClass+"::attr() is deprecated. Use get() or set() instead, called from "+_1d6,"","2.0");_1d5[_1d6]=true;}}var args=arguments.length;if(args>=2||typeof name==="object"){return this.set.apply(this,arguments);}else{return this.get(name);}},getDescendants:function(){_1c4.deprecated(this.declaredClass+"::getDescendants() is deprecated. Use getChildren() instead.","","2.0");return this.containerNode?_1c5("[widgetId]",this.containerNode).map(_1c7.byNode):[];},_onShow:function(){this.onShow();},onShow:function(){},onHide:function(){},onClose:function(){return true;}});if(!_1c4.isAsync){_1c6(0,function(){var _1d7=["dijit/_base"];require(_1d7);});}return _1d1;});},"dojo/cache":function(){define(["./_base/kernel","./text"],function(dojo,text){return dojo.cache;});},"dijit/_FocusMixin":function(){define(["./focus","./_WidgetBase","dojo/_base/declare","dojo/_base/lang"],function(_1d8,_1d9,_1da,lang){lang.extend(_1d9,{focused:false,onFocus:function(){},onBlur:function(){},_onFocus:function(){this.onFocus();},_onBlur:function(){this.onBlur();}});return _1da("dijit._FocusMixin",null,{_focusManager:_1d8});});},"dijit/_OnDijitClickMixin":function(){define("dijit/_OnDijitClickMixin",["dojo/on","dojo/_base/array","dojo/keys","dojo/_base/declare","dojo/_base/sniff","dojo/_base/unload","dojo/_base/window"],function(on,_1db,keys,_1dc,has,_1dd,win){var _1de=null;if(has("ie")){(function(){var _1df=function(evt){_1de=evt.srcElement;};win.doc.attachEvent("onkeydown",_1df);_1dd.addOnWindowUnload(function(){win.doc.detachEvent("onkeydown",_1df);});})();}else{win.doc.addEventListener("keydown",function(evt){_1de=evt.target;},true);}var _1e0=function(node,_1e1){if(/input|button/i.test(node.nodeName)){return on(node,"click",_1e1);}else{function _1e2(e){return (e.keyCode==keys.ENTER||e.keyCode==keys.SPACE)&&!e.ctrlKey&&!e.shiftKey&&!e.altKey&&!e.metaKey;};var _1e3=[on(node,"keypress",function(e){if(_1e2(e)){_1de=e.target;e.preventDefault();}}),on(node,"keyup",function(e){if(_1e2(e)&&e.target==_1de){_1de=null;_1e1.call(this,e);}}),on(node,"click",function(e){_1e1.call(this,e);})];return {remove:function(){_1db.forEach(_1e3,function(h){h.remove();});}};}};return _1dc("dijit._OnDijitClickMixin",null,{connect:function(obj,_1e4,_1e5){return this.inherited(arguments,[obj,_1e4=="ondijitclick"?_1e0:_1e4,_1e5]);}});});},"dijit/focus":function(){define("dijit/focus",["dojo/aspect","dojo/_base/declare","dojo/dom","dojo/dom-attr","dojo/dom-construct","dojo/Evented","dojo/_base/lang","dojo/on","dojo/ready","dojo/_base/sniff","dojo/Stateful","dojo/_base/unload","dojo/_base/window","dojo/window","./a11y","./registry","."],function(_1e6,_1e7,dom,_1e8,_1e9,_1ea,lang,on,_1eb,has,_1ec,_1ed,win,_1ee,a11y,_1ef,_1f0){var _1f1=_1e7([_1ec,_1ea],{curNode:null,activeStack:[],constructor:function(){var _1f2=lang.hitch(this,function(node){if(dom.isDescendant(this.curNode,node)){this.set("curNode",null);}if(dom.isDescendant(this.prevNode,node)){this.set("prevNode",null);}});_1e6.before(_1e9,"empty",_1f2);_1e6.before(_1e9,"destroy",_1f2);},registerIframe:function(_1f3){return this.registerWin(_1f3.contentWindow,_1f3);},registerWin:function(_1f4,_1f5){var _1f6=this;var _1f7=function(evt){_1f6._justMouseDowned=true;setTimeout(function(){_1f6._justMouseDowned=false;},0);if(has("ie")&&evt&&evt.srcElement&&evt.srcElement.parentNode==null){return;}_1f6._onTouchNode(_1f5||evt.target||evt.srcElement,"mouse");};var doc=has("ie")?_1f4.document.documentElement:_1f4.document;if(doc){if(has("ie")){_1f4.document.body.attachEvent("onmousedown",_1f7);var _1f8=function(evt){var tag=evt.srcElement.tagName.toLowerCase();if(tag=="#document"||tag=="body"){return;}if(a11y.isTabNavigable(evt.srcElement)){_1f6._onFocusNode(_1f5||evt.srcElement);}else{_1f6._onTouchNode(_1f5||evt.srcElement);}};doc.attachEvent("onactivate",_1f8);var _1f9=function(evt){_1f6._onBlurNode(_1f5||evt.srcElement);};doc.attachEvent("ondeactivate",_1f9);return {remove:function(){_1f4.document.detachEvent("onmousedown",_1f7);doc.detachEvent("onactivate",_1f8);doc.detachEvent("ondeactivate",_1f9);doc=null;}};}else{doc.body.addEventListener("mousedown",_1f7,true);doc.body.addEventListener("touchstart",_1f7,true);var _1fa=function(evt){_1f6._onFocusNode(_1f5||evt.target);};doc.addEventListener("focus",_1fa,true);var _1fb=function(evt){_1f6._onBlurNode(_1f5||evt.target);};doc.addEventListener("blur",_1fb,true);return {remove:function(){doc.body.removeEventListener("mousedown",_1f7,true);doc.body.removeEventListener("touchstart",_1f7,true);doc.removeEventListener("focus",_1fa,true);doc.removeEventListener("blur",_1fb,true);doc=null;}};}}},_onBlurNode:function(){this.set("prevNode",this.curNode);this.set("curNode",null);if(this._justMouseDowned){return;}if(this._clearActiveWidgetsTimer){clearTimeout(this._clearActiveWidgetsTimer);}this._clearActiveWidgetsTimer=setTimeout(lang.hitch(this,function(){delete this._clearActiveWidgetsTimer;this._setStack([]);this.prevNode=null;}),100);},_onTouchNode:function(node,by){if(this._clearActiveWidgetsTimer){clearTimeout(this._clearActiveWidgetsTimer);delete this._clearActiveWidgetsTimer;}var _1fc=[];try{while(node){var _1fd=_1e8.get(node,"dijitPopupParent");if(_1fd){node=_1ef.byId(_1fd).domNode;}else{if(node.tagName&&node.tagName.toLowerCase()=="body"){if(node===win.body()){break;}node=_1ee.get(node.ownerDocument).frameElement;}else{var id=node.getAttribute&&node.getAttribute("widgetId"),_1fe=id&&_1ef.byId(id);if(_1fe&&!(by=="mouse"&&_1fe.get("disabled"))){_1fc.unshift(id);}node=node.parentNode;}}}}catch(e){}this._setStack(_1fc,by);},_onFocusNode:function(node){if(!node){return;}if(node.nodeType==9){return;}this._onTouchNode(node);if(node==this.curNode){return;}this.set("curNode",node);},_setStack:function(_1ff,by){var _200=this.activeStack;this.set("activeStack",_1ff);for(var _201=0;_201<Math.min(_200.length,_1ff.length);_201++){if(_200[_201]!=_1ff[_201]){break;}}var _202;for(var i=_200.length-1;i>=_201;i--){_202=_1ef.byId(_200[i]);if(_202){_202._hasBeenBlurred=true;_202.set("focused",false);if(_202._focusManager==this){_202._onBlur(by);}this.emit("widget-blur",_202,by);}}for(i=_201;i<_1ff.length;i++){_202=_1ef.byId(_1ff[i]);if(_202){_202.set("focused",true);if(_202._focusManager==this){_202._onFocus(by);}this.emit("widget-focus",_202,by);}}},focus:function(node){if(node){try{node.focus();}catch(e){}}}});var _203=new _1f1();_1eb(function(){var _204=_203.registerWin(win.doc.parentWindow||win.doc.defaultView);if(has("ie")){_1ed.addOnWindowUnload(function(){_204.remove();_204=null;});}});_1f0.focus=function(node){_203.focus(node);};for(var attr in _203){if(!/^_/.test(attr)){_1f0.focus[attr]=typeof _203[attr]=="function"?lang.hitch(_203,attr):_203[attr];}}_203.watch(function(attr,_205,_206){_1f0.focus[attr]=_206;});return _203;});},"dijit/_base/sniff":function(){define("dijit/_base/sniff",["dojo/uacss"],function(){});},"dijit/main":function(){define("dijit/main",["dojo/_base/kernel"],function(dojo){return dojo.dijit;});},"dojo/date/stamp":function(){define(["../_base/kernel","../_base/lang","../_base/array"],function(dojo,lang,_207){lang.getObject("date.stamp",true,dojo);dojo.date.stamp.fromISOString=function(_208,_209){if(!dojo.date.stamp._isoRegExp){dojo.date.stamp._isoRegExp=/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;}var _20a=dojo.date.stamp._isoRegExp.exec(_208),_20b=null;if(_20a){_20a.shift();if(_20a[1]){_20a[1]--;}if(_20a[6]){_20a[6]*=1000;}if(_209){_209=new Date(_209);_207.forEach(_207.map(["FullYear","Month","Date","Hours","Minutes","Seconds","Milliseconds"],function(prop){return _209["get"+prop]();}),function(_20c,_20d){_20a[_20d]=_20a[_20d]||_20c;});}_20b=new Date(_20a[0]||1970,_20a[1]||0,_20a[2]||1,_20a[3]||0,_20a[4]||0,_20a[5]||0,_20a[6]||0);if(_20a[0]<100){_20b.setFullYear(_20a[0]||1970);}var _20e=0,_20f=_20a[7]&&_20a[7].charAt(0);if(_20f!="Z"){_20e=((_20a[8]||0)*60)+(Number(_20a[9])||0);if(_20f!="-"){_20e*=-1;}}if(_20f){_20e-=_20b.getTimezoneOffset();}if(_20e){_20b.setTime(_20b.getTime()+_20e*60000);}}return _20b;};dojo.date.stamp.toISOString=function(_210,_211){var _212=function(n){return (n<10)?"0"+n:n;};_211=_211||{};var _213=[],_214=_211.zulu?"getUTC":"get",date="";if(_211.selector!="time"){var year=_210[_214+"FullYear"]();date=["0000".substr((year+"").length)+year,_212(_210[_214+"Month"]()+1),_212(_210[_214+"Date"]())].join("-");}_213.push(date);if(_211.selector!="date"){var time=[_212(_210[_214+"Hours"]()),_212(_210[_214+"Minutes"]()),_212(_210[_214+"Seconds"]())].join(":");var _215=_210[_214+"Milliseconds"]();if(_211.milliseconds){time+="."+(_215<100?"0":"")+_212(_215);}if(_211.zulu){time+="Z";}else{if(_211.selector!="time"){var _216=_210.getTimezoneOffset();var _217=Math.abs(_216);time+=(_216>0?"-":"+")+_212(Math.floor(_217/60))+":"+_212(_217%60);}}_213.push(time);}return _213.join("T");};return dojo.date.stamp;});},"dijit/form/_FormWidget":function(){define("dijit/form/_FormWidget",["dojo/_base/declare","dojo/_base/kernel","dojo/ready","../_Widget","../_CssStateMixin","../_TemplatedMixin","./_FormWidgetMixin"],function(_218,_219,_21a,_21b,_21c,_21d,_21e){if(!_219.isAsync){_21a(0,function(){var _21f=["dijit/form/_FormValueWidget"];require(_21f);});}return _218("dijit.form._FormWidget",[_21b,_21d,_21c,_21e],{setDisabled:function(_220){_219.deprecated("setDisabled("+_220+") is deprecated. Use set('disabled',"+_220+") instead.","","2.0");this.set("disabled",_220);},setValue:function(_221){_219.deprecated("dijit.form._FormWidget:setValue("+_221+") is deprecated.  Use set('value',"+_221+") instead.","","2.0");this.set("value",_221);},getValue:function(){_219.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.","","2.0");return this.get("value");},postMixInProperties:function(){this.nameAttrSetting=this.name?("name=\""+this.name.replace(/'/g,"&quot;")+"\""):"";this.inherited(arguments);},_setTypeAttr:null});});},"dijit/_base/typematic":function(){define(["../typematic"],function(){});},"dijit/_base/popup":function(){define("dijit/_base/popup",["dojo/dom-class","../popup","../BackgroundIframe"],function(_222,_223){var _224=_223._createWrapper;_223._createWrapper=function(_225){if(!_225.declaredClass){_225={_popupWrapper:(_225.parentNode&&_222.contains(_225.parentNode,"dijitPopup"))?_225.parentNode:null,domNode:_225,destroy:function(){}};}return _224.call(this,_225);};var _226=_223.open;_223.open=function(args){if(args.orient&&typeof args.orient!="string"&&!("length" in args.orient)){var ary=[];for(var key in args.orient){ary.push({aroundCorner:key,corner:args.orient[key]});}args.orient=ary;}return _226.call(this,args);};return _223;});},"dijit/_TemplatedMixin":function(){define("dijit/_TemplatedMixin",["dojo/_base/lang","dojo/touch","./_WidgetBase","dojo/string","dojo/cache","dojo/_base/array","dojo/_base/declare","dojo/dom-construct","dojo/_base/sniff","dojo/_base/unload","dojo/_base/window"],function(lang,_227,_228,_229,_22a,_22b,_22c,_22d,has,_22e,win){var _22f=_22c("dijit._TemplatedMixin",null,{templateString:null,templatePath:null,_skipNodeCache:false,_earlyTemplatedStartup:false,constructor:function(){this._attachPoints=[];this._attachEvents=[];},_stringRepl:function(tmpl){var _230=this.declaredClass,_231=this;return _229.substitute(tmpl,this,function(_232,key){if(key.charAt(0)=="!"){_232=lang.getObject(key.substr(1),false,_231);}if(typeof _232=="undefined"){throw new Error(_230+" template:"+key);}if(_232==null){return "";}return key.charAt(0)=="!"?_232:_232.toString().replace(/"/g,"&quot;");},this);},buildRendering:function(){if(!this.templateString){this.templateString=_22a(this.templatePath,{sanitize:true});}var _233=_22f.getCachedTemplate(this.templateString,this._skipNodeCache);var node;if(lang.isString(_233)){node=_22d.toDom(this._stringRepl(_233));if(node.nodeType!=1){throw new Error("Invalid template: "+_233);}}else{node=_233.cloneNode(true);}this.domNode=node;this.inherited(arguments);this._attachTemplateNodes(node,function(n,p){return n.getAttribute(p);});this._beforeFillContent();this._fillContent(this.srcNodeRef);},_beforeFillContent:function(){},_fillContent:function(_234){var dest=this.containerNode;if(_234&&dest){while(_234.hasChildNodes()){dest.appendChild(_234.firstChild);}}},_attachTemplateNodes:function(_235,_236){var _237=lang.isArray(_235)?_235:(_235.all||_235.getElementsByTagName("*"));var x=lang.isArray(_235)?0:-1;for(;x<_237.length;x++){var _238=(x==-1)?_235:_237[x];if(this.widgetsInTemplate&&(_236(_238,"dojoType")||_236(_238,"data-dojo-type"))){continue;}var _239=_236(_238,"dojoAttachPoint")||_236(_238,"data-dojo-attach-point");if(_239){var _23a,_23b=_239.split(/\s*,\s*/);while((_23a=_23b.shift())){if(lang.isArray(this[_23a])){this[_23a].push(_238);}else{this[_23a]=_238;}this._attachPoints.push(_23a);}}var _23c=_236(_238,"dojoAttachEvent")||_236(_238,"data-dojo-attach-event");if(_23c){var _23d,_23e=_23c.split(/\s*,\s*/);var trim=lang.trim;while((_23d=_23e.shift())){if(_23d){var _23f=null;if(_23d.indexOf(":")!=-1){var _240=_23d.split(":");_23d=trim(_240[0]);_23f=trim(_240[1]);}else{_23d=trim(_23d);}if(!_23f){_23f=_23d;}this._attachEvents.push(this.connect(_238,_227[_23d]||_23d,_23f));}}}}},destroyRendering:function(){_22b.forEach(this._attachPoints,function(_241){delete this[_241];},this);this._attachPoints=[];_22b.forEach(this._attachEvents,this.disconnect,this);this._attachEvents=[];this.inherited(arguments);}});_22f._templateCache={};_22f.getCachedTemplate=function(_242,_243){var _244=_22f._templateCache;var key=_242;var _245=_244[key];if(_245){try{if(!_245.ownerDocument||_245.ownerDocument==win.doc){return _245;}}catch(e){}_22d.destroy(_245);}_242=_229.trim(_242);if(_243||_242.match(/\$\{([^\}]+)\}/g)){return (_244[key]=_242);}else{var node=_22d.toDom(_242);if(node.nodeType!=1){throw new Error("Invalid template: "+_242);}return (_244[key]=node);}};if(has("ie")){_22e.addOnWindowUnload(function(){var _246=_22f._templateCache;for(var key in _246){var _247=_246[key];if(typeof _247=="object"){_22d.destroy(_247);}delete _246[key];}});}lang.extend(_228,{dojoAttachEvent:"",dojoAttachPoint:""});return _22f;});},"dijit/_base/wai":function(){define("dijit/_base/wai",["dojo/dom-attr","dojo/_base/lang","..","../hccss"],function(_248,lang,_249){lang.mixin(_249,{hasWaiRole:function(elem,role){var _24a=this.getWaiRole(elem);return role?(_24a.indexOf(role)>-1):(_24a.length>0);},getWaiRole:function(elem){return lang.trim((_248.get(elem,"role")||"").replace("wairole:",""));},setWaiRole:function(elem,role){_248.set(elem,"role",role);},removeWaiRole:function(elem,role){var _24b=_248.get(elem,"role");if(!_24b){return;}if(role){var t=lang.trim((" "+_24b+" ").replace(" "+role+" "," "));_248.set(elem,"role",t);}else{elem.removeAttribute("role");}},hasWaiState:function(elem,_24c){return elem.hasAttribute?elem.hasAttribute("aria-"+_24c):!!elem.getAttribute("aria-"+_24c);},getWaiState:function(elem,_24d){return elem.getAttribute("aria-"+_24d)||"";},setWaiState:function(elem,_24e,_24f){elem.setAttribute("aria-"+_24e,_24f);},removeWaiState:function(elem,_250){elem.removeAttribute("aria-"+_250);}});return _249;});},"dojo/window":function(){define(["./_base/kernel","./_base/lang","./_base/sniff","./_base/window","./dom","./dom-geometry","./dom-style"],function(dojo,lang,has,_251,dom,geom,_252){lang.getObject("window",true,dojo);dojo.window.getBox=function(){var _253=(_251.doc.compatMode=="BackCompat")?_251.body():_251.doc.documentElement;var _254=geom.docScroll();var _255=_251.doc.parentWindow||_251.doc.defaultView;return {l:_254.x,t:_254.y,w:_255.innerWidth||_253.clientWidth,h:_255.innerHeight||_253.clientHeight};};dojo.window.get=function(doc){if(has("ie")&&window!==document.parentWindow){doc.parentWindow.execScript("document._parentWindow = window;","Javascript");var win=doc._parentWindow;doc._parentWindow=null;return win;}return doc.parentWindow||doc.defaultView;};dojo.window.scrollIntoView=function(node,pos){try{node=dom.byId(node);var doc=node.ownerDocument||_251.doc,body=doc.body||_251.body(),html=doc.documentElement||body.parentNode,isIE=has("ie"),isWK=has("webkit");if((!(has("mozilla")||isIE||isWK||has("opera"))||node==body||node==html)&&(typeof node.scrollIntoView!="undefined")){node.scrollIntoView(false);return;}var _256=doc.compatMode=="BackCompat",_257=(isIE>=9&&node.ownerDocument.parentWindow.frameElement)?((html.clientHeight>0&&html.clientWidth>0&&(body.clientHeight==0||body.clientWidth==0||body.clientHeight>html.clientHeight||body.clientWidth>html.clientWidth))?html:body):(_256?body:html),_258=isWK?body:_257,_259=_257.clientWidth,_25a=_257.clientHeight,rtl=!geom.isBodyLtr(),_25b=pos||geom.position(node),el=node.parentNode,_25c=function(el){return ((isIE<=6||(isIE&&_256))?false:(_252.get(el,"position").toLowerCase()=="fixed"));};if(_25c(node)){return;}while(el){if(el==body){el=_258;}var _25d=geom.position(el),_25e=_25c(el);if(el==_258){_25d.w=_259;_25d.h=_25a;if(_258==html&&isIE&&rtl){_25d.x+=_258.offsetWidth-_25d.w;}if(_25d.x<0||!isIE){_25d.x=0;}if(_25d.y<0||!isIE){_25d.y=0;}}else{var pb=geom.getPadBorderExtents(el);_25d.w-=pb.w;_25d.h-=pb.h;_25d.x+=pb.l;_25d.y+=pb.t;var _25f=el.clientWidth,_260=_25d.w-_25f;if(_25f>0&&_260>0){_25d.w=_25f;_25d.x+=(rtl&&(isIE||el.clientLeft>pb.l))?_260:0;}_25f=el.clientHeight;_260=_25d.h-_25f;if(_25f>0&&_260>0){_25d.h=_25f;}}if(_25e){if(_25d.y<0){_25d.h+=_25d.y;_25d.y=0;}if(_25d.x<0){_25d.w+=_25d.x;_25d.x=0;}if(_25d.y+_25d.h>_25a){_25d.h=_25a-_25d.y;}if(_25d.x+_25d.w>_259){_25d.w=_259-_25d.x;}}var l=_25b.x-_25d.x,t=_25b.y-Math.max(_25d.y,0),r=l+_25b.w-_25d.w,bot=t+_25b.h-_25d.h;if(r*l>0){var s=Math[l<0?"max":"min"](l,r);if(rtl&&((isIE==8&&!_256)||isIE>=9)){s=-s;}_25b.x+=el.scrollLeft;el.scrollLeft+=s;_25b.x-=el.scrollLeft;}if(bot*t>0){_25b.y+=el.scrollTop;el.scrollTop+=Math[t<0?"max":"min"](t,bot);_25b.y-=el.scrollTop;}el=(el!=_258)&&!_25e&&el.parentNode;}}catch(error){console.error("scrollIntoView: "+error);node.scrollIntoView(false);}};return dojo.window;});},"dijit/popup":function(){define("dijit/popup",["dojo/_base/array","dojo/aspect","dojo/_base/connect","dojo/_base/declare","dojo/dom","dojo/dom-attr","dojo/dom-construct","dojo/dom-geometry","dojo/dom-style","dojo/_base/event","dojo/keys","dojo/_base/lang","dojo/on","dojo/_base/sniff","dojo/_base/window","./place","./BackgroundIframe","."],function(_261,_262,_263,_264,dom,_265,_266,_267,_268,_269,keys,lang,on,has,win,_26a,_26b,_26c){var _26d=_264(null,{_stack:[],_beginZIndex:1000,_idGen:1,_createWrapper:function(_26e){var _26f=_26e._popupWrapper,node=_26e.domNode;if(!_26f){_26f=_266.create("div",{"class":"dijitPopup",style:{display:"none"},role:"presentation"},win.body());_26f.appendChild(node);var s=node.style;s.display="";s.visibility="";s.position="";s.top="0px";_26e._popupWrapper=_26f;_262.after(_26e,"destroy",function(){_266.destroy(_26f);delete _26e._popupWrapper;});}return _26f;},moveOffScreen:function(_270){var _271=this._createWrapper(_270);_268.set(_271,{visibility:"hidden",top:"-9999px",display:""});},hide:function(_272){var _273=this._createWrapper(_272);_268.set(_273,"display","none");},getTopPopup:function(){var _274=this._stack;for(var pi=_274.length-1;pi>0&&_274[pi].parent===_274[pi-1].widget;pi--){}return _274[pi];},open:function(args){var _275=this._stack,_276=args.popup,_277=args.orient||["below","below-alt","above","above-alt"],ltr=args.parent?args.parent.isLeftToRight():_267.isBodyLtr(),_278=args.around,id=(args.around&&args.around.id)?(args.around.id+"_dropdown"):("popup_"+this._idGen++);while(_275.length&&(!args.parent||!dom.isDescendant(args.parent.domNode,_275[_275.length-1].widget.domNode))){this.close(_275[_275.length-1].widget);}var _279=this._createWrapper(_276);_265.set(_279,{id:id,style:{zIndex:this._beginZIndex+_275.length},"class":"dijitPopup "+(_276.baseClass||_276["class"]||"").split(" ")[0]+"Popup",dijitPopupParent:args.parent?args.parent.id:""});if(has("ie")||has("mozilla")){if(!_276.bgIframe){_276.bgIframe=new _26b(_279);}}var best=_278?_26a.around(_279,_278,_277,ltr,_276.orient?lang.hitch(_276,"orient"):null):_26a.at(_279,args,_277=="R"?["TR","BR","TL","BL"]:["TL","BL","TR","BR"],args.padding);_279.style.display="";_279.style.visibility="visible";_276.domNode.style.visibility="visible";var _27a=[];_27a.push(on(_279,_263._keypress,lang.hitch(this,function(evt){if(evt.charOrCode==keys.ESCAPE&&args.onCancel){_269.stop(evt);args.onCancel();}else{if(evt.charOrCode===keys.TAB){_269.stop(evt);var _27b=this.getTopPopup();if(_27b&&_27b.onCancel){_27b.onCancel();}}}})));if(_276.onCancel&&args.onCancel){_27a.push(_276.on("cancel",args.onCancel));}_27a.push(_276.on(_276.onExecute?"execute":"change",lang.hitch(this,function(){var _27c=this.getTopPopup();if(_27c&&_27c.onExecute){_27c.onExecute();}})));_275.push({widget:_276,parent:args.parent,onExecute:args.onExecute,onCancel:args.onCancel,onClose:args.onClose,handlers:_27a});if(_276.onOpen){_276.onOpen(best);}return best;},close:function(_27d){var _27e=this._stack;while((_27d&&_261.some(_27e,function(elem){return elem.widget==_27d;}))||(!_27d&&_27e.length)){var top=_27e.pop(),_27f=top.widget,_280=top.onClose;if(_27f.onClose){_27f.onClose();}var h;while(h=top.handlers.pop()){h.remove();}if(_27f&&_27f.domNode){this.hide(_27f);}if(_280){_280();}}}});return (_26c.popup=new _26d());});},"dijit/_base/window":function(){define("dijit/_base/window",["dojo/window",".."],function(_281,_282){_282.getDocumentWindow=function(doc){return _281.get(doc);};});},"dijit/_WidgetBase":function(){define("dijit/_WidgetBase",["require","dojo/_base/array","dojo/aspect","dojo/_base/config","dojo/_base/connect","dojo/_base/declare","dojo/dom","dojo/dom-attr","dojo/dom-class","dojo/dom-construct","dojo/dom-geometry","dojo/dom-style","dojo/_base/kernel","dojo/_base/lang","dojo/on","dojo/ready","dojo/Stateful","dojo/topic","dojo/_base/window","./registry"],function(_283,_284,_285,_286,_287,_288,dom,_289,_28a,_28b,_28c,_28d,_28e,lang,on,_28f,_290,_291,win,_292){if(!_28e.isAsync){_28f(0,function(){var _293=["dijit/_base/manager"];_283(_293);});}var _294={};function _295(obj){var ret={};for(var attr in obj){ret[attr.toLowerCase()]=true;}return ret;};function _296(attr){return function(val){_289[val?"set":"remove"](this.domNode,attr,val);this._set(attr,val);};};return _288("dijit._WidgetBase",_290,{id:"",_setIdAttr:"domNode",lang:"",_setLangAttr:_296("lang"),dir:"",_setDirAttr:_296("dir"),textDir:"","class":"",_setClassAttr:{node:"domNode",type:"class"},style:"",title:"",tooltip:"",baseClass:"",srcNodeRef:null,domNode:null,containerNode:null,attributeMap:{},_blankGif:_286.blankGif||_283.toUrl("dojo/resources/blank.gif"),postscript:function(_297,_298){this.create(_297,_298);},create:function(_299,_29a){this.srcNodeRef=dom.byId(_29a);this._connects=[];this._supportingWidgets=[];if(this.srcNodeRef&&(typeof this.srcNodeRef.id=="string")){this.id=this.srcNodeRef.id;}if(_299){this.params=_299;lang.mixin(this,_299);}this.postMixInProperties();if(!this.id){this.id=_292.getUniqueId(this.declaredClass.replace(/\./g,"_"));}_292.add(this);this.buildRendering();if(this.domNode){this._applyAttributes();var _29b=this.srcNodeRef;if(_29b&&_29b.parentNode&&this.domNode!==_29b){_29b.parentNode.replaceChild(this.domNode,_29b);}}if(this.domNode){this.domNode.setAttribute("widgetId",this.id);}this.postCreate();if(this.srcNodeRef&&!this.srcNodeRef.parentNode){delete this.srcNodeRef;}this._created=true;},_applyAttributes:function(){var ctor=this.constructor,list=ctor._setterAttrs;if(!list){list=(ctor._setterAttrs=[]);for(var attr in this.attributeMap){list.push(attr);}var _29c=ctor.prototype;for(var _29d in _29c){if(_29d in this.attributeMap){continue;}var _29e="_set"+_29d.replace(/^[a-z]|-[a-zA-Z]/g,function(c){return c.charAt(c.length-1).toUpperCase();})+"Attr";if(_29e in _29c){list.push(_29d);}}}_284.forEach(list,function(attr){if(this.params&&attr in this.params){}else{if(this[attr]){this.set(attr,this[attr]);}}},this);for(var _29f in this.params){this.set(_29f,this[_29f]);}},postMixInProperties:function(){},buildRendering:function(){if(!this.domNode){this.domNode=this.srcNodeRef||_28b.create("div");}if(this.baseClass){var _2a0=this.baseClass.split(" ");if(!this.isLeftToRight()){_2a0=_2a0.concat(_284.map(_2a0,function(name){return name+"Rtl";}));}_28a.add(this.domNode,_2a0);}},postCreate:function(){},startup:function(){if(this._started){return;}this._started=true;_284.forEach(this.getChildren(),function(obj){if(!obj._started&&!obj._destroyed&&lang.isFunction(obj.startup)){obj.startup();obj._started=true;}});},destroyRecursive:function(_2a1){this._beingDestroyed=true;this.destroyDescendants(_2a1);this.destroy(_2a1);},destroy:function(_2a2){this._beingDestroyed=true;this.uninitialize();var c;while(c=this._connects.pop()){c.remove();}var w;while(w=this._supportingWidgets.pop()){if(w.destroyRecursive){w.destroyRecursive();}else{if(w.destroy){w.destroy();}}}this.destroyRendering(_2a2);_292.remove(this.id);this._destroyed=true;},destroyRendering:function(_2a3){if(this.bgIframe){this.bgIframe.destroy(_2a3);delete this.bgIframe;}if(this.domNode){if(_2a3){_289.remove(this.domNode,"widgetId");}else{_28b.destroy(this.domNode);}delete this.domNode;}if(this.srcNodeRef){if(!_2a3){_28b.destroy(this.srcNodeRef);}delete this.srcNodeRef;}},destroyDescendants:function(_2a4){_284.forEach(this.getChildren(),function(_2a5){if(_2a5.destroyRecursive){_2a5.destroyRecursive(_2a4);}});},uninitialize:function(){return false;},_setStyleAttr:function(_2a6){var _2a7=this.domNode;if(lang.isObject(_2a6)){_28d.set(_2a7,_2a6);}else{if(_2a7.style.cssText){_2a7.style.cssText+="; "+_2a6;}else{_2a7.style.cssText=_2a6;}}this._set("style",_2a6);},_attrToDom:function(attr,_2a8,_2a9){_2a9=arguments.length>=3?_2a9:this.attributeMap[attr];_284.forEach(lang.isArray(_2a9)?_2a9:[_2a9],function(_2aa){var _2ab=this[_2aa.node||_2aa||"domNode"];var type=_2aa.type||"attribute";switch(type){case "attribute":if(lang.isFunction(_2a8)){_2a8=lang.hitch(this,_2a8);}var _2ac=_2aa.attribute?_2aa.attribute:(/^on[A-Z][a-zA-Z]*$/.test(attr)?attr.toLowerCase():attr);_289.set(_2ab,_2ac,_2a8);break;case "innerText":_2ab.innerHTML="";_2ab.appendChild(win.doc.createTextNode(_2a8));break;case "innerHTML":_2ab.innerHTML=_2a8;break;case "class":_28a.replace(_2ab,_2a8,this[attr]);break;}},this);},get:function(name){var _2ad=this._getAttrNames(name);return this[_2ad.g]?this[_2ad.g]():this[name];},set:function(name,_2ae){if(typeof name==="object"){for(var x in name){this.set(x,name[x]);}return this;}var _2af=this._getAttrNames(name),_2b0=this[_2af.s];if(lang.isFunction(_2b0)){var _2b1=_2b0.apply(this,Array.prototype.slice.call(arguments,1));}else{var _2b2=this.focusNode&&!lang.isFunction(this.focusNode)?"focusNode":"domNode",tag=this[_2b2].tagName,_2b3=_294[tag]||(_294[tag]=_295(this[_2b2])),map=name in this.attributeMap?this.attributeMap[name]:_2af.s in this?this[_2af.s]:((_2af.l in _2b3&&typeof _2ae!="function")||/^aria-|^data-|^role$/.test(name))?_2b2:null;if(map!=null){this._attrToDom(name,_2ae,map);}this._set(name,_2ae);}return _2b1||this;},_attrPairNames:{},_getAttrNames:function(name){var apn=this._attrPairNames;if(apn[name]){return apn[name];}var uc=name.replace(/^[a-z]|-[a-zA-Z]/g,function(c){return c.charAt(c.length-1).toUpperCase();});return (apn[name]={n:name+"Node",s:"_set"+uc+"Attr",g:"_get"+uc+"Attr",l:uc.toLowerCase()});},_set:function(name,_2b4){var _2b5=this[name];this[name]=_2b4;if(this._watchCallbacks&&this._created&&_2b4!==_2b5){this._watchCallbacks(name,_2b5,_2b4);}},on:function(type,func){return _285.after(this,this._onMap(type),func,true);},_onMap:function(type){var ctor=this.constructor,map=ctor._onMap;if(!map){map=(ctor._onMap={});for(var attr in ctor.prototype){if(/^on/.test(attr)){map[attr.replace(/^on/,"").toLowerCase()]=attr;}}}return map[type.toLowerCase()];},toString:function(){return "[Widget "+this.declaredClass+", "+(this.id||"NO ID")+"]";},getChildren:function(){return this.containerNode?_292.findWidgets(this.containerNode):[];},getParent:function(){return _292.getEnclosingWidget(this.domNode.parentNode);},connect:function(obj,_2b6,_2b7){var _2b8=_287.connect(obj,_2b6,this,_2b7);this._connects.push(_2b8);return _2b8;},disconnect:function(_2b9){var i=_284.indexOf(this._connects,_2b9);if(i!=-1){_2b9.remove();this._connects.splice(i,1);}},subscribe:function(t,_2ba){var _2bb=_291.subscribe(t,lang.hitch(this,_2ba));this._connects.push(_2bb);return _2bb;},unsubscribe:function(_2bc){this.disconnect(_2bc);},isLeftToRight:function(){return this.dir?(this.dir=="ltr"):_28c.isBodyLtr();},isFocusable:function(){return this.focus&&(_28d.get(this.domNode,"display")!="none");},placeAt:function(_2bd,_2be){if(_2bd.declaredClass&&_2bd.addChild){_2bd.addChild(this,_2be);}else{_28b.place(this.domNode,_2bd,_2be);}return this;},getTextDir:function(text,_2bf){return _2bf;},applyTextDir:function(){}});});}}});define("dijit/dijit",[".","./_base","dojo/parser","./_Widget","./_TemplatedMixin","./_Container","./layout/_LayoutWidget","./form/_FormWidget","./form/_FormValueWidget"],function(_2c0){return _2c0;});