//>>built
define("dojox/charting/themes/Ireland", ["../Theme", "./common"], function(Theme, themes){
	
	themes.Ireland=new Theme({
		colors: [
			"#abdbcb",
			"#435a51",
			"#70998b",
			"#78d596",
			"#5f8074"
		]
	});
	
	return themes.Ireland;
});
