//>>built
define(
//begin v1.x content
({
	'bold': '굵게',
	'copy': '복사',
	'cut': '잘라내기',
	'delete': '삭제',
	'indent': '들여쓰기',
	'insertHorizontalRule': '수평 자',
	'insertOrderedList': '번호 목록',
	'insertUnorderedList': '글머리표 목록',
	'italic': '기울임꼴',
	'justifyCenter': '가운데 맞춤',
	'justifyFull': '양쪽 맞춤',
	'justifyLeft': '왼쪽 맞춤',
	'justifyRight': '오른쪽 맞춤',
	'outdent': '내어쓰기',
	'paste': '붙여넣기',
	'redo': '다시 실행',
	'removeFormat': '형식 제거',
	'selectAll': '모두 선택',
	'strikethrough': '취소선',
	'subscript': '아래첨자',
	'superscript': '위첨자',
	'underline': '밑줄',
	'undo': '실행 취소',
	'unlink': '링크 제거',
	'createLink': '링크 작성',
	'toggleDir': '방향 토글',
	'insertImage': '이미지 삽입',
	'insertTable': '테이블 삽입/편집',
	'toggleTableBorder': '테이블 외곽선 토글',
	'deleteTable': '테이블 삭제',
	'tableProp': '테이블 특성',
	'htmlToggle': 'HTML 소스',
	'foreColor': '전경색',
	'hiliteColor': '배경색',
	'plainFormatBlock': '단락 양식',
	'formatBlock': '단락 양식',
	'fontSize': '글꼴 크기',
	'fontName': '글꼴 이름',
	'tabIndent': '탭 들여쓰기',
	"fullScreen": "전체 화면 토글",
	"viewSource": "HTML 소스 보기",
	"print": "인쇄",
	"newPage": "새 페이지",
	/* Error messages */
	'systemShortcut': '"${0}" 조치는 브라우저에서 키보드 단축키를 통해서만 사용 가능합니다. ${1}을(를) 사용하십시오.'
})
//end v1.x content
);
