//>>built
define(
//begin v1.x content
({
	'bold': 'Fet',
	'copy': 'Kopier',
	'cut': 'Klipp ut',
	'delete': 'Slett',
	'indent': 'Innrykk',
	'insertHorizontalRule': 'Vannrett strek',
	'insertOrderedList': 'Nummerert liste',
	'insertUnorderedList': 'Punktliste',
	'italic': 'Kursiv',
	'justifyCenter': 'Midtstill',
	'justifyFull': 'Juster',
	'justifyLeft': 'Venstrejuster',
	'justifyRight': 'Høyrejuster',
	'outdent': 'Fjern innrykk',
	'paste': 'Lim inn',
	'redo': 'Gjør om',
	'removeFormat': 'Fjern format',
	'selectAll': 'Velg alle',
	'strikethrough': 'Gjennomstreking',
	'subscript': 'Senket skrift',
	'superscript': 'Hevet skrift',
	'underline': 'Understreking',
	'undo': 'Angre',
	'unlink': 'Fjern kobling',
	'createLink': 'Opprett kobling',
	'toggleDir': 'Bytt retning',
	'insertImage': 'Sett inn bilde',
	'insertTable': 'Sett inn/rediger tabell',
	'toggleTableBorder': 'Bytt tabellkant',
	'deleteTable': 'Slett tabell',
	'tableProp': 'Tabellegenskap',
	'htmlToggle': 'HTML-kilde',
	'foreColor': 'Forgrunnsfarge',
	'hiliteColor': 'Bakgrunnsfarge',
	'plainFormatBlock': 'Avsnittsstil',
	'formatBlock': 'Avsnittsstil',
	'fontSize': 'Skriftstørrelse',
	'fontName': 'Skriftnavn',
	'tabIndent': 'Tabulatorinnrykk',
	"fullScreen": "Slå på/av full skjerm",
	"viewSource": "Vis HTML-kilde",
	"print": "Skriv ut",
	"newPage": "Ny side",
	/* Error messages */
	'systemShortcut': 'Handlingen "${0}" er bare tilgjengelig i nettleseren ved hjelp av en tastatursnarvei. Bruk ${1}.',
	'ctrlKey':'ctrl+${0}',
	'appleKey':'\u2318${0}' // "command" or open-apple key on Macintosh
})
//end v1.x content
);
