
.soria .dijitCalendarIncrementControl {
	
	width:15px;
	height:15px;
	background-image: url("images/spriteRoundedIconsSmall.png");
	background-repeat: no-repeat;
}
.dj_ie6 .soria .dijitCalendarIncrementControl {
	font-size:.1em;
	background-image: url("images/spriteRoundedIconsSmall.gif");
}
.soria .dijitA11ySideArrow {
	display: none;
}
.soria .dijitCalendarDecrease {
	background-position: top left;
}
.soria .dijitCalendarIncrease {
	background-position: -30px top;
}
.soria table.dijitCalendarContainer {
	font-size: 100%;
	border-spacing: 0;
	border-collapse: separate;
	border: 1px solid #b1badf;
	margin: 0;
}
.soria .dijitCalendarMonthContainer th {
	
	background:#bed7f0 url("images/titleBar.png") repeat-x top;
	padding-top:.3em;
	padding-bottom:.2em;
	text-align:center;
}
.dj_ie6 .soria .dijitCalendarMonthContainer th {
	padding-top:.2em;
	padding-bottom:.1em;
}
.soria .dijitCalendarDayLabelTemplate {
	
	background:#bed7f0;
	font-weight:normal;
	padding-top:.15em;
	padding-bottom:.2em;
	border-bottom: 1px solid #b1badf;
	color:#293a4b;
	text-align:center;
}
.soria .dijitCalendarBodyContainer {
	border-bottom: 1px solid #eeeeee;
}
.soria .dijitCalendarMonthLabel {
	color:#293a4b;
	font-weight: bold;
	padding: 0 4px;
}
.soria .dijitCalendarDateTemplate {
	
	font-size: 0.9em;
	font-weight: bold;
	text-align: center;
	padding: 0.3em 0.3em 0.05em 0.3em;
	letter-spacing: 1px;
	background-color: #fff;
	border:#fff solid 1px !important;
}
.dj_ie .soria .dijitCalendarDateTemplate {
	padding: 0.1em .33em 0.02em .33em;
}
.soria .dijitCalendarPreviousMonth,
.soria .dijitCalendarNextMonth 		{
	
	color:#999999;
	background-color:#fdfdfd !important;
	border:#fdfdfd solid 1px !important;
}
.soria .dijitCalendarCurrentMonth {
	
}
.soria .dijitCalendarDisabledDate {
	text-decoration:line-through !important;
}
.soria .dijitCalendarCurrentDate {
	
	text-decoration:underline;
	font-weight:bold;
}
.soria .dijitCalendarSelectedDate {
	
	background-color:#b9cbf1 !important;
	color:black !important;
	border:#4b5aaa solid 1px !important;
}
.soria .dijitCalendarYearContainer {
	
	background:white url("images/titleBar.png") repeat-x top;
}
.soria .dijitCalendarYearLabel {
	
	margin:0;
	padding:0.4em 0 0.25em 0;
	text-align:center;
	font-size: 1.17em;
}
.soria .dijitCalendarSelectedYear {
	
	font-weight:bolder;
	color:black;
	padding:0.2em;
	padding-bottom:0.1em;
	background-color:#b9cbf1 !important;
	border:#4b5aaa solid 1px !important;
}
.soria .dijitCalendarNextYear,
.soria .dijitCalendarPreviousYear {
	
	color:black !important;
	font-weight:normal;
}
.soria .dijitCalendar .dijitDropDownButton {
	margin: 0;
}
.soria .dijitCalendar .dijitButtonText {
	padding: 0;
}
.soria .dijitCalendar .dijitDropDownButton .dijitButtonNode {
	background-color: transparent;
	background-image: none;
	padding: 0;
}
.soria .dijitCalendarMonthMenu .dijitCalendarMonthLabelHover {
	background-color: #d9e6f9;
	color: #243C5F;
}
