
.dijitRtl .dijitPlaceHolder {
	left: auto;
	right: 0;
}
.dijitMenuItemRtl {
	text-align: right;
}
.dj_iequirks .dijitComboButtonRtl button {
	
	float:left;
}
.dijitTextBoxRtl .dijitValidationContainer,
.dijitTextBoxRtl .dijitSpinnerButtonContainer,
.dijitComboBoxRtl .dijitArrowButtonContainer {
	
	border-right-width: 1px !important;
	border-left-width: 0 !important;
}
.dijitSpinnerRtl .dijitSpinnerButtonContainer .dijitArrowButton {
	right: 0;
	left: auto;
}
.dj_ie7 .dijitInputContainer {
	
	display: inline-block;
}
.dijitTextBoxRtl .dijitSpinnerButtonContainer,
.dijitTextBoxRtl .dijitValidationContainer,
.dijitTextBoxRtl .dijitArrowButtonContainer {
	float: left;
}
.dijitCalendarRtl .dijitCalendarNextYear {
	margin:0 0.55em 0 0;
}
.dijitCalendarRtl .dijitCalendarPreviousYear {
	margin:0 0 0 0.55em;
}
.dijitSliderRtl .dijitSliderImageHandleV {
	left:auto;
}
.dijitSliderRtl .dijitSliderImageHandleH {
	left:-50%;
}
.dijitSliderRtl .dijitSliderMoveableH {
	right:auto;
	left:0;
}
.dijitSliderRtl .dijitRuleContainerV {
	float:right;
}
.dj_ie .dijitSliderRtl .dijitRuleContainerV {
	text-align:right;
}
.dj_ie .dijitSliderRtl .dijitRuleLabelV {
	text-align:left;
}
.dj_ie .dijitSliderRtl .dijitRuleLabelH {
	zoom:1;
}
.dijitSliderRtl .dijitSliderProgressBarH {
	
	float:right;
	right:0;
	left:auto;
}
.dijitRtl .dijitContentPaneLoading .dijitIconLoading,
.dijitRtl .dijitContentPaneError .dijitIconError {
	margin-right: 0;
	margin-left: 9px;
}
.dijitTabControllerRtl .nowrapTabStrip {
    text-align: right;  
}
.dijitTabRtl .dijitTabCloseButton {
	margin-left: 0;
	margin-right: 1em;
}
.dj_ie .dijitTimePickerRtl .dijitTimePickerItem {
	width:100%;
}
.dijitColorPaletteRtl .dijitColorPaletteUnder {
	
	left: auto;
	right: 0;
}
.dijitSelectRtl .dijitButtonContents {
	text-align: right;
}
.tundra .dijitCalendarRtl .dijitCalendarDecrease {
	background-position: -30px top;
}
.tundra .dijitCalendarRtl .dijitCalendarIncrease {
	background-position: 0 top;
}
 .dj_ie6 .tundra .dijitTimePickerRtl .dijitTimePickerMarkerHover,
.dj_ie7 .tundra .dijitTimePickerRtl .dijitTimePickerMarkerHover {
        border-top: 0; 
}
.tundra .dijitDialogRtl .dijitDialogCloseIcon {
	right: auto;
	left: 5px;
}
.tundra .dijitEditorRtl .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsEnabled_rtl.png); 
}
.tundra .dijitEditorRtlDisabled .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsDisabled_rtl.png); 
}
.dijitEditorRtl .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsEnabled_rtl.png); 
}
.dijitEditorRtlDisabled .dijitEditorIcon {
	background-image: url(../../icons/images/editorIconsDisabled_rtl.png); 
}
.dijitToolbarRtl .dijitToolbarSeparator {
	background-image: url(../../icons/images/editorIconsEnabled_rtl.png);
}
.tundra .dijitMenuItemRtl .dijitMenuExpand {
	background-position: -7px 0;
}
.tundra .dijitTreeNodeRtl,
.tundra .dijitTreeNodeRtl .dijitTreeExpandoLeaf {
	
    background-image: none;
}
.tundra .dijitTreeNodeRtl .dijitTreeExpandoOpened {
	
	background-image: url(images/treeExpand_minus_rtl.gif);
}
.tundra .dijitTreeNodeRtl .dijitTreeExpandoClosed {
	background-image: url(images/treeExpand_plus_rtl.gif);
}
.tundra .dijitTitlePaneRtl .dijitClosed .dijitArrowNode {
	background-position: -7px 0;
}
.tundra .dijitTabRtl {
	-moz-box-orient:horizontal;
	text-align: right;
}
.tundra .dijitTabRtl .dijitTabInnerDiv {
	padding:2px 9px 2px 8px;
}
.tundra .tabStrip-disabled .tabStripButtonRtl .dijitTabInnerDiv {
	
	padding-bottom: 3px;
	padding-top: 1px;
}
.tundra .tabStripButtonRtl .dijitTabInnerDiv {
	padding: 3px 2px 4px 2px;
}
.tundra .dijitTabPaneWrapper {
	#zoom: 1;
}
.dj_ie-rtl .tundra .dijitTabContainerLeft-tabs {
	margin-left: 1px !important;
}
.dj_ie-rtl .tundra .dijitTabContainerRight-tabs {
	margin-right: 1px !important;
}
.tundra .dijitTabContainerLeft-tabs .dijitTabRtl,
.tundra .dijitTabContainerRight-tabs .dijitTabRtl {
	margin-left:0;
}
.dj_ie .tundra .dijitTabRtl .dijitTabInnerDiv {
	
	width : 0.1% !important;
}
.dj_iequirks-rtl .tundra .dijitTabContainerTopNone,
.dj_iequirks-rtl .tundra .dijitTabContainerBottomNone {
	
	border-left: 1px solid #fff;
	border-right: 1px solid #fff;
}
.tundra .dijitSliderRtl .dijitSliderProgressBarH,
.tundra .dijitSliderRtl .dijitSliderRemainingBarH,
.tundra .dijitSliderRtl .dijitSliderLeftBumper,
.tundra .dijitSliderRtl .dijitSliderRightBumper,
.tundra .dijitSliderRtl .dijitSliderTopBumper {
	background-position: top right;
}
.tundra .dijitSliderRtl .dijitSliderProgressBarV,
.tundra .dijitSliderRtl .dijitSliderRemainingBarV,
.tundra .dijitSliderRtl .dijitSliderBottomBumper {
	background-position: bottom right;
}
.tundra .dijitSliderRtl .dijitSliderLeftBumper {
	border-left-width: 0;
	border-right-width: 1px;
}
.tundra .dijitSliderRtl .dijitSliderRightBumper {
	border-left-width: 1px;
	border-right-width: 0;
}
.tundra .dijitSliderRtl .dijitSliderIncrementIconH {
	background-position: -7px 0;
}
.tundra .dijitSliderRtl .dijitSliderDecrementIconH {
	background-position: -14px 0;
}
