//>>built
define(
//begin v1.x content
({
	fontSize: "Velikost",
	fontName: "Písmo",
	formatBlock: "Formát",

	serif: "serif",
	"sans-serif": "sans-serif",
	monospace: "monospace",
	cursive: "cursive",
	fantasy: "fantasy",

	noFormat: "Žádný",
	p: "Odstavec",
	h1: "Nadpis",
	h2: "Podnadpis",
	h3: "Podnadpis 2",
	pre: "Předformátované",

	1: "extra malé",
	2: "velmi malé",
	3: "malé",
	4: "střední",
	5: "velké",
	6: "velmi velké",
	7: "extra velké"
})
//end v1.x content
);
