
.claro .dijitContentPane {
  padding: 8px;
}
.claro .dijitTabContainerTop-dijitContentPane,
.claro .dijitTabContainerLeft-dijitContentPane,
.claro .dijitTabContainerBottom-dijitContentPane,
.claro .dijitTabContainerRight-dijitContentPane,
.claro .dijitAccordionContainer-dijitContentPane {
  background-color: #ffffff;
  padding: 8px;
}
.claro .dijitSplitContainer-dijitContentPane, .claro .dijitBorderContainer-dijitContentPane {
  background-color: #ffffff;
  padding: 8px;
}
