
.claro .dijitAccordionContainer {
  border: none;
}
.claro .dijitAccordionInnerContainer {
  background-color: #efefef;
  
  border: solid 1px #b5bcc7;
  margin-bottom: 1px;
  -webkit-transition-property: background-color, border;
  -moz-transition-property: background-color, border;
  transition-property: background-color, border;
  -webkit-transition-duration: 0.3s;
  -moz-transition-duration: 0.3s;
  transition-duration: 0.3s;
  -webkit-transition-timing-function: linear;
  -moz-transition-timing-function: linear;
  transition-timing-function: linear;
}
.claro .dijitAccordionTitle {
  background-color: transparent;
  
  background-image: url("../layout/images/accordion.png");
  background-position: 0 0;
  background-repeat: repeat-x;
  padding: 5px 7px 2px 7px;
  min-height: 17px;
  color: #494949;
}
.dj_ie6 .claro .dijitAccordionTitle {
  background-image: none;
}
.claro .dijitAccordionContainer .dijitAccordionChildWrapper {
  
  background-color: #ffffff;
  border: 1px solid #759dc0;
  margin: 0 2px 2px;
}
.claro .dijitAccordionContainer .dijitAccordionContainer-child {
  
  padding: 9px;
}
.claro .dijitAccordionInnerContainerActive {
  border: 1px solid #759dc0;
  background-color: #7dbdfa;
  -webkit-transition-duration: 0.1s;
  -moz-transition-duration: 0.1s;
  transition-duration: 0.1s;
}
.claro .dijitAccordionInnerContainerActive .dijitAccordionTitle {
  background-position: 0 -136px;
  color: #000000;
}
.claro .dijitAccordionInnerContainerSelected {
  border-color: #759dc0;
  background-color: #cfe5fa;
}
.claro .dijitAccordionInnerContainerSelected .dijitAccordionTitle {
  color: #000000;
  background-position: 0 0;
  
}
.claro .dijitAccordionInnerContainerHover .dijitAccordionTitle {
  
  color: #000000;
}
.claro .dijitAccordionInnerContainerHover, .claro .dijitAccordionInnerContainerSelectedActive {
  
  border: 1px solid #759dc0;
  background-color: #abd6ff;
  -webkit-transition-duration: 0.2s;
  -moz-transition-duration: 0.2s;
  transition-duration: 0.2s;
}
.claro .dijitAccordionInnerContainerSelectedHover .dijitAccordionChildWrapper, .claro .dijitAccordionInnerContainerSelectedActive .dijitAccordionChildWrapper {
  background-color: #ffffff;
  border: 1px solid #759dc0 !important;
  -webkit-box-shadow: inset 0 0 3px rgba(0, 0, 0, 0.25);
  -moz-box-shadow: inset 0 0 3px rgba(0, 0, 0, 0.25);
  box-shadow: inset 0 0 3px rgba(0, 0, 0, 0.25);
}
